#========================================================================================
# package      DE_EPAGES::Coupon::Hooks::LineItemContainer
# state        public
# description  Tests the coupon and change LineItems.
#========================================================================================
package DE_EPAGES::Coupon::Hooks::LineItemContainer;

use strict;

#========================================================================================
# function     LineItemPrice
#----------------------------------------------------------------------------------------
# syntax       LineItemPrice( $hParams );
#----------------------------------------------------------------------------------------
# description  LineItemPrice is triggered to interrupt the recalculation process of basket
#               to set the correct shipping price for coupons with free shipping
#               Hook is triggered by Order::API::Object::LineItemContainer::recalculate()
#----------------------------------------------------------------------------------------
# input        $hParams | parameters
#               <ul>
#                   <li>Object | LineItem | object
#                   <li>Price  | LineItemPrice default: undef | string
#               </ul> | ref.hash.*
#========================================================================================
sub LineItemPrice {
    my ($hParams) = @_;
    my $LineItem = $hParams->{'Object'};

    # test if the LineItem is a shipping lineitem
    if ($LineItem->instanceOf('LineItemShipping')) {

        # test, if the cpoupon is for free shipping, then set the shipping lineitemprice to 0
        my $CouponLineItem = $LineItem->parent()->get('Coupon');
        return unless defined $CouponLineItem;
        my $CouponCode = $CouponLineItem->get('CouponCode');
        return unless defined $CouponCode;
        my $CouponCampaign = $CouponCode->get('CouponCampaign');
        my $Type =  $CouponCampaign->get('Type');

        if ( $Type eq  'SHIPPING' ) {
            # test for validation errors...
            my $ahValidationErrors = $CouponLineItem->get('ValidationErrors');
            my $IsShippingMisMatch = 0;

            if (scalar(@$ahValidationErrors)) {
                foreach my $hError (@{$ahValidationErrors}) {
                    $IsShippingMisMatch = 1 if $hError->{'Message'} eq 'ShippingMethodMisMatch';
                    $IsShippingMisMatch = 1 if $hError->{'Message'} eq 'LineItemsSubTotalTooSmall';
                }
            }
            # set the shipping price to 0 if the coupon is for free shipping
            # and  we have the correct shipping method from coupon conditions
            if ( not $IsShippingMisMatch ) {
                $hParams->{'Price'} = 0;
                $LineItem->set({'LineItemPrice' => 0});
                
                #-- The situation for payment discounts/fees is this:
                #-- When $Container->recalculate is called, all line items are recalculated via their package ->recalculate()
                #-- Accordingly, payment discounts/fees are recalculated after coupons
                #-- However, for coupons which waive shipping, the shipping price is not 'waived' in the coupon' recalculate function, it is done here
                #-- This means that when the payment discount line is recalculated, it still includes the shipping price in its discount/fee calculation, which it should not
                #-- This hook is responsible for waiving the shipping price, and it occurs after all recalculate functions have been called
                #-- We therefore need to recalculate payment discount line(s) here as well
                #-- Also: see DE_EPAGES::Order::API::Object::LineItemContainer::recalculate()
                my $Container = $LineItem->parent();
                my $PaymentDiscount = $Container->get('PaymentDiscount');
                if ( $PaymentDiscount ) {
                    $PaymentDiscount->recalculate($Container, {});
                    #-- For consistency' sake, this function should also be called as it is in DE_EPAGES::Order::API::Object::LineItemContainer::recalculate()
                    #-- Although payment discount/fee lines never seem to have a tax class anyway, so the call is rather superflous . . .
                    $Container->_addLineItemTax($PaymentDiscount); ## no critic qw(ProhibitPrivateSubroutineUse)
                }
            }
        }
    }
    return ;
}

1;

