#========================================================================================
# package      DE_EPAGES::DataCache::API::Cartridge
# base         DE_EPAGES::XML::API::XMLInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::DataCache::API::Cartridge;
use base DE_EPAGES::XML::API::XMLInstaller;

use strict;
use DE_EPAGES::DataCache::API::Config qw(DataCacheConfig);
use DE_EPAGES::Core::API::IniConfig;

## no critic qw(ProhibitMagicNumbers)

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::DataCache::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::DataCache.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw(ProhibitLongLines)
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/DataCache',
        'Version'            => '7.94.0',
        'Patches'            => ['7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0','7.85.0','7.86.0','7.87.0','7.88.0','7.89.0','7.90.0','7.91.0','7.92.0','7.93.0'],
        'HasDbSchema'        => 0,
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub doPatch {
    my $self = shift;

    unless( $ENV{EPAGES_NO_PATCH_DATACACHE_CONF} ) {
        my $FileName = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory,  'Patches/DataCache.conf');
        _patchDataCacheConfigFile( $FileName, {} );
    }

    return $self->SUPER::doPatch;
}

#========================================================================================
# function     _patchDataCacheConfigFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       _patchDataCacheConfigFile( $FileName, $hOverwrite );
#----------------------------------------------------------------------------------------
# description  patch a existing DataCache.conf file compared to passed file.
#               Adds keys that exist $FileName and are missing in DataCache.conf.
#               Changes the value if the value in $FileName is 0 or 1.
#               Changes the value if the current value is 200 in DataCache.conf and
#               the value is different in $FileName.
#----------------------------------------------------------------------------------------
# input        $FileName | name of the reference file of the current version | string
# input        $hOverwrite | (optional) cache keys to override, example:
#                           { $CacheKey => [ $OldValue, $NewValue ],
#                             ...
#                           }
#                           If the current value is $OldValue, then set it to $NewValue.
#                           This is useful to reduce the cache size. | ref.hash.array
#========================================================================================
sub _patchDataCacheConfigFile {
    my ( $FileName, $hOverwrite ) = @_;

    my $PatchIniConfig = DE_EPAGES::Core::API::IniConfig->new( FileName => $FileName, CaseSensitive => 1 );
    my $IniConfig = DataCacheConfig();
    my $Rewrite = 0;

    # update section AllStoresCaches
    my $hNew = $PatchIniConfig->section( 'AllStoresCaches');
    my $hOld = $IniConfig->section( 'AllStoresCaches');
    foreach my $CacheName (keys %$hNew) {
        if (exists $hOld->{$CacheName}) {
            next if $hNew->{$CacheName} <= $hOld->{$CacheName};
            $IniConfig->set('AllStoresCaches', $CacheName, $hNew->{$CacheName});
            $Rewrite = 1;
        } else {
            $IniConfig->set('AllStoresCaches', $CacheName, $hNew->{$CacheName});
            $Rewrite = 1;
        }
    }

    #remove all keys with value=200 because its default
    foreach my $CacheName (keys %$hOld) {
        if ($IniConfig->get('AllStoresCaches', $CacheName) eq '200'){
            $IniConfig->delete('AllStoresCaches', $CacheName);
            $Rewrite = 1;
        }
    }

    # remove empty sections
    foreach my $Section ( @{$IniConfig->sectionNames} ) {
        if( 0 == scalar keys %{$IniConfig->section( $Section )} ) {
            $IniConfig->deleteSection( $Section );
            $Rewrite = 1;
        }
    }
    if (defined $hOverwrite) {
        foreach my $CacheName (keys %$hOverwrite) {
            next if $IniConfig->get('AllStoresCaches', $CacheName) != $hOverwrite->{$CacheName}->[0];

            $IniConfig->set('AllStoresCaches', $CacheName, $hOverwrite->{$CacheName}->[1]);
            $Rewrite = 1;
        }
    }

    $IniConfig->rewrite if $Rewrite;
}

1;
