#========================================================================================
# package      DE_EPAGES::Ebay::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Ebay::API::Cartridge;
use base DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;

use strict;
use DE_EPAGES::Ebay::API::Constants;
use DE_EPAGES::Core::API::File qw ( GetFileContent SetAccessRights );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Config qw( GetSQLiteDBFile );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::Database::API::DBInterface qw ( ProcessSqlGo );
use DE_EPAGES::Database::API::OrderFile qw ( GetSqlFileOrder );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

## no critic qw(ProhibitLongLines)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Ebay::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Ebay.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Ebay',
        'Version'            => '7.94.0',
        'Patches'            => ['7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0','7.85.0','7.86.0','7.87.0','7.88.0','7.89.0','7.90.0','7.91.0','7.92.0','7.93.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    ImportObjects( $self->xmlDirectory .'/EbayEnvironment.xml' );

    # import regions
    ImportObjects( $self->xmlDirectory . '/EbayRegions.xml');
    # import call types
    ImportObjects( $self->xmlDirectory . '/EbayCallTypes.xml');
    # import ebay system templates
    ImportObjects( $self->xmlDirectory . '/DefaultSystemTemplate.xml');
    # import ebay mail types
    ImportObjects( $self->xmlDirectory . '/MailTypes.xml');

    my $EbayCategoriesDB = GetSQLiteDBFile('EbayCategories') // EBAY_CATEGORIES_DB_FILENAME;
    $self->installEbayCategories(EBAY_CATEGORIES_DB_NAME, $EbayCategoriesDB);
    $self->finishInstall;
    return;
}

sub installEbayCategories {
    my $self = shift;
    my ($DBName, $DBFilename, $ForceUpdate) = @_;

    RunOnStore(
        Store => $DBName,
        Sub => sub  {
            my $dbi = GetCurrentDBHandle();

            if (not $dbi->existsTable('ebaysite') or $ForceUpdate) {
                my $Dir = join '/', $ENV{'EPAGES_CARTRIDGES'}, $self->cartridgeDirectory , 'Database', $dbi->driverName;
                my $aFiles = GetSqlFileOrder( $Dir );
                foreach my $File (@$aFiles) {
                    next if $File =~ /\/delete_/;
                    my $rContent = GetFileContent( $File );
                    ProcessSqlGo( $$rContent );
                }
                SetAccessRights( $DBFilename );
            }
        }
    );
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;

    # EbayCallTypes.xml and EbayRegions.xml are cleaned up
    # on uninstall by deleting the respective database tables
    $self->call( sub { DeleteObjects( $self->xmlDirectory . '/MailTypes.xml') } );
    $self->call( sub { DeleteObjects( $self->xmlDirectory .'/DefaultSystemTemplate.xml' ) } );
    $self->call( sub { DeleteObjects( $self->xmlDirectory .'/EbayEnvironment.xml' ) } );

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     encryptData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->encryptData;
#----------------------------------------------------------------------------------------
# description  Encrpyts data of this cartridge.
#========================================================================================
sub encryptData {
    my $self = shift;

    $self->encryptDataByClass('EbayShopEnvironment');

    GetLog()->info('encrypting ebay attributes of System class');
    my $System = LoadRootObject();

    my @EncrpytedAttributes = qw (
        EbayCertId EbayProviderPassword EbayProviderToken EbayRuName EbaySessionID
        EbaySandBoxCertId EbaySandBoxProviderPassword EbaySandBoxProviderToken EbaySandBoxRuName
    );

    $System->set( $System->get(\@EncrpytedAttributes) );

}

#========================================================================================
# function     patchDBStructure_7_9_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_7_9_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '7.9.0' to version '7.10.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '7.10.0' | string
#========================================================================================
sub patchDBStructure_7_9_0 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content
    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicypaymentid')) {
            $self->logInfo('add column ebayauction.businesspolicypaymentid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'returnswithin', 'businesspolicypaymentid varchar(30) null');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicyshippingid')) {
            $self->logInfo('add column ebayauction.businesspolicyshippingid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'businesspolicypaymentid', 'businesspolicyshippingid varchar(30) null');
        }
        unless ($dbi->existsTableColumn('ebayauction', 'businesspolicyreturnpolicyid')) {
            $self->logInfo('add column ebayauction.businesspolicyreturnpolicyid');
            $dbi->addTableColumnAfterColumn('ebayauction', 'businesspolicyshippingid', 'businesspolicyreturnpolicyid varchar(30) null');
        }

    }

    return '7.10.0';
}

#========================================================================================
# function     patchDBStructure_7_75_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_7_75_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '7.75.0' to version '7.76.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '7.76.0' | string
#========================================================================================
sub patchDBStructure_7_75_0 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content
    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/ebayauction.sql
        if ($dbi->existsTableColumn('ebayauction', 'hitcounter')) {
            $self->logInfo('drop column ebayauction.hitcounter');
            $dbi->dropTableColumn('ebayauction','hitcounter');
        }

    }

    return '7.76.0';
}

#========================================================================================
# function     patchDBStructure_7_92_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_7_92_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '7.92.0' to version '7.93.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '7.93.0' | string
#========================================================================================
sub patchDBStructure_7_92_0 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        unless ($dbi->existsTableColumn('ebayauction', 'isebayplus')) {
            $self->logInfo('add column ebayauction.isebayplus after column ebayauction.isgetitfast');
            $dbi->addTableColumnAfterColumn('ebayauction', 'isgetitfast', 'isebayplus bool not null default 0');
            $dbi->modifyTableColumnDropDefault('ebayauction','isebayplus');
        }

        unless ($dbi->existsTableColumn('ebayenvironment', 'hasebayplus')) {
            $self->logInfo('add column ebayenvironment.hasebayplus after column ebayenvironment.hasgetitfast');
            $dbi->addTableColumnAfterColumn('ebayenvironment', 'hasgetitfast', 'hasebayplus bool not null default 0');
            $dbi->modifyTableColumnDropDefault('ebayenvironment','hasebayplus');
        }

        unless ($dbi->existsTableColumn('ebayshopenvironment', 'ebayplus')) {
            $self->logInfo('add column ebayshopenvironment.ebayplus');
            $dbi->addTableColumnAfterColumn('ebayshopenvironment', 'ebayregion', 'ebayplus integer null');
        }

    }

    return '7.93.0';
}

1;
