#========================================================================================
# package      DE_EPAGES::Ebay::Hooks::Order
# state        public
# description  Order related hook functions.
#========================================================================================
package DE_EPAGES::Ebay::Hooks::Order;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadClassByAlias );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );

#========================================================================================
# function     OnOrderCreated
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreated( $hParams );
#----------------------------------------------------------------------------------------
# description  When a order is created, check if this order has an EbayTransaction.
#               Sets Transaction to "purchased". Saves eBay's user name for customer.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Order
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreated {
    my ($hParams) = @_;

    my $order     = $hParams->{'Order'};
    my $container = $order->get('LineItemContainer');
    my $customer  = $order->get('Customer');

    return unless defined $container;

    my @EbayLineItems = grep { $_->instanceOf('LineItemEbayTransaction') } @{$container->children};

    return unless scalar @EbayLineItems;

    foreach my $lineitem (@EbayLineItems) {
        my $transaction = $lineitem->get('EbayTransaction');

        $transaction->set({
            'IsPurchased'   => 1,
            'CustomerOrder' => $order,
            'Customer' => $customer,
        });

        if (not $customer->get('EbayUserName')) {
            $customer->set({
                'EbayUserName'          => $transaction->get('EbayUserName'),
                'EbayShopEnvironment'   => $transaction->get('EbayAuction')->get('EbayShopEnvironment'),
            });
        }
    }

    return 1;
}

#========================================================================================
# function     BeforeEditOrder2Order
#----------------------------------------------------------------------------------------
# syntax       BeforeEditOrder2Order( $hParams );
#----------------------------------------------------------------------------------------
# description  Re-assigns new order object to transaction, when order was edited.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Order
#               </ul> | ref.hash.*
#========================================================================================
sub BeforeEditOrder2Order {
    my ($hParams) = @_;

    my $order = $hParams->{'Order'};
    my $origOrder = $order->get('OriginalOrder');

    my $aTransactions = LoadClassByAlias('EbayTransaction')->getObjects('ByCustomerOrderID',[$origOrder->id]);
    return unless scalar @$aTransactions;

    foreach my $transaction (@$aTransactions) {
        $transaction->set({'CustomerOrder' => $order});
    }

    return 1;
}

#========================================================================================
# function     OnDeleteOrder
#----------------------------------------------------------------------------------------
# syntax       OnDeleteOrder( $hParams );
#----------------------------------------------------------------------------------------
# description  Sets order of transaction to undef, if order was deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Order
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteOrder {
    my ($hParams) = @_;

    my $order = $hParams->{'Object'};

    my $aTransactionIDs = LoadClassByAlias('EbayTransaction')->findObjects('ByCustomerOrderID', [$order->id]);
    return unless scalar @$aTransactionIDs;

    my $transaction = LoadObject($aTransactionIDs->[0]);

    $transaction->set({'CustomerOrder' => undef});

    return 1;
}

#========================================================================================
# function     OnBeforeUpdateCustomerOrder
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateCustomerOrder( $hParams );
#----------------------------------------------------------------------------------------
# description  Send shipping informations to ebay
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object</li>
#                   <li>OrderID</li>
#                   <li>Values</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateCustomerOrder {
    my ($hParams) = @_;
    my $Order = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};

    if ( exists $hValues->{'DispatchedOn'} ) {
        my $EbayTransaction;
        my $externalTransactionID;
        my $EbayAuction;
        my $EbayAuctionId;

        my $hParamsDispatched;
        my $ShippedStatus = 0;
        my $DispatchedOn = undef;
        # pre-set ShipmentTrackingNumber and ShippingCarrierUsed if we have no shipping informations
        # we do not want to set these fields to an value
        $hParamsDispatched->{'ShipmentTrackingNumber'} = '';
        $hParamsDispatched->{'ShippingCarrierUsed'} = '';
        if (defined $hValues->{'DispatchedOn'} ) {
            #ebay needs the date in UTC
            $DispatchedOn = $hValues->{'DispatchedOn'};
            $DispatchedOn->set_time_zone('UTC');

            $ShippedStatus = 1;
            $hParamsDispatched->{'ShippedTime'} = $DispatchedOn->iso8601() . 'Z';
            $hParamsDispatched->{'Shipped'} = 1;
        } else {
            #unset shipped status if merchant unchecked the DispatchedOn checkbox
            $hParamsDispatched->{'Shipped'} = 0;
            $hParamsDispatched->{'ShippedTime'} = undef;
        }

        #fetch all data from the order lineitems
        foreach my $LineItem (@{$Order->get('LineItemContainer')->children}) {

            #get all ebay related data
            if ($LineItem->instanceOf('LineItemEbayTransaction')) {
                $EbayTransaction = $LineItem->get('EbayTransaction');
                $externalTransactionID = $EbayTransaction->get('ExternalTransactionId');
                $EbayAuction = $EbayTransaction->get('EbayAuction');
                $EbayAuctionId = $EbayAuction->get('ExternalAuctionId');
            }

            # we check if the shipping from the ebay transaction is with DHL or Colissimo,
            # then we send trackinginformations if available
            if ($ShippedStatus && $LineItem->instanceOf('LineItemShipping ')) {
                my $ShippingLabelsFolder = $Order->folder('ShippingLabels')->children;
                if (scalar @{$ShippingLabelsFolder}) {
                    foreach my $ShippingLabel (@{$ShippingLabelsFolder}) {
                        #Trackinginformations for DHL
                        if ($LineItem->get('Name') =~ /^DHL/ && $ShippingLabel->instanceOf('ShippingLabelDHLIntraship') && defined($ShippingLabel->get('TrackingID'))) {
                            #set Trackingnumber and Shippingcarrier
                            $hParamsDispatched->{'ShipmentTrackingNumber'} = $ShippingLabel->get('TrackingID');
                            $hParamsDispatched->{'ShippingCarrierUsed'} = 'DHL';
                        }
                        #Trackinginformations for Colissimo
                        if ($LineItem->get('Name') =~ /^Colissimo/ && $ShippingLabel->instanceOf('ShippingOrderColissimo') && defined($ShippingLabel->get('TrackingID'))) {
                            #set Trackingnumber and Shippingcarrier
                            $hParamsDispatched->{'ShipmentTrackingNumber'} = $ShippingLabel->get('TrackingID');
                            $hParamsDispatched->{'ShippingCarrierUsed'} = 'DHL';
                        }
                    }
                }
            }
        }

        # we only make the CompleteSale call if an EbayAuction exists
        if (defined $EbayAuction) {
            eval {
                my $ShopEnv = $EbayAuction->get('EbayShopEnvironment');

                $ShopEnv->remoteEbay()->completeSale(
                    $externalTransactionID,
                    $EbayAuctionId,
                    $hParamsDispatched
                );
            };
            if (ExistsError) {
                my $Error = GetError();
                LogError($Error);
            }
        }
    }
    return;
}

1;

