use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings);
use DE_EPAGES::Database::API::Connection qw (  RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadObject );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName = 'Store';
    my $ShopAlias;
    my $EnvironmentAlias;
    my $StartDate;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'              => \$Help,
        'nooutput'          => \$NoOutput,
        'passwd=s'          => \$Password,
        'storename=s'       => \$StoreName,
        'shop=s'            => \$ShopAlias,
        'environment=s'     => \$EnvironmentAlias,
        'startdate=s'       => \$StartDate,
    );

    usage() if $Help;
    print "\nMissing shop!\n" and usage() unless $ShopAlias;
    print "\nMissing date!\n" and usage() unless $StartDate;

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {

            # make sure that the cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::Ebay')) {
                print "Store: $StoreName : Cartridge Ebay is not installed.\n" unless ($NoOutput);
                return;
            }

            my $aWarnings;
            if ( defined $ShopAlias) {
                my $Shop;
                eval { $Shop = LoadObjectByPath('/Shops')->child($ShopAlias); };
                if ( ExistsError() ) {
                    printf STDERR "[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias;
                    return;
                }
                $aWarnings = BlockWarnings( sub {
                    _UpdateShopAuctionsStatus( $Shop, $EnvironmentAlias, $StartDate, $NoOutput )
                });
            }
            foreach my $Warning (@$aWarnings) {
                my $LogMessage = substr($Warning->shortMessage, 0, 4096); ## no critic qw(ProhibitMagicNumbers)
                LogError($Warning);
                print "WARN: $LogMessage\n";
            }
        }
    );

    my $logmsg = sprintf ('need %.1f seconds', time - $time1);
    print $logmsg."\n" unless $NoOutput;
    GetLog()->debug("Update auctions: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
    Sets the ebayauction status to 2 (Online) for failed ebayauctions status 16 if ebay API returns a ListingStatus of 'Active'


Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password                    ()
    -storename    name of store                             (Store)
    -shop         shop alias                                ()
    -environment  ebay environment                          (EbayShopEnvironment1)
    -startdate    retrive auctions with starttime since     (2023-09-27)


flags:
    -nooutput     do not print any output
    -help         show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -environment EbayShopEnvironment1 -startdate 2023-09-29
END_USAGE
    exit 2;
}

sub _UpdateShopAuctionsStatus {
    my ($Shop, $EnvironmentAlias, $StartDate, $NoOutput) = @_;
    my $ShopID = $Shop->id;
    my $aEbayShopEnvironments = $Shop->get('EbayShopEnvironments');

    my @ShopEnv = grep {$_->alias eq $EnvironmentAlias} @$aEbayShopEnvironments;
    for (my $i=0; $i<scalar(@ShopEnv); $i++) {
        my $ShopEnv = $ShopEnv[$i];

        my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('ShopEnvironment'=>$ShopEnv);
        my $dbi = GetCurrentDBHandle();
        my $sql = 'select externalauctionid, auctionid from ebayauction where shopid=? and status=16 and starttime<? and externalauctionid is not null ;';
        my $aaResult = $dbi->execute($sql, [$ShopID, $StartDate]);

        my $CountActiveAuctions = 0;
        my $CountCompletedAuctions = 0;
        my $CountEndedAuctions = 0;

        print "Found ".scalar(@$aaResult)." ebay auctions with AUCTION_FAILED (16) status for EbayShopEnvironment: ".$ShopEnv[$i]->alias()."\n\n";
        foreach my $aEbayAuctionItem (@$aaResult) {
            if (defined @$aEbayAuctionItem[0]) {
                my $ExternalAuctionID =  @$aEbayAuctionItem[0];
                my $EbayAuctionID = @$aEbayAuctionItem[1];
                my $EbayAuction = LoadObject($EbayAuctionID);
                my $EbayAuctionAlias = $EbayAuction->alias;


                my $Response = $RemoteEbay->getItem($ExternalAuctionID);
                my $ItemListingStatus = $Response->{'SellingStatus'}->{'ListingStatus'};

                if ($ItemListingStatus eq 'Active') {
                    $EbayAuction->set({'Status' => 2});
                    print "Set Status for EbayAuction $EbayAuctionAlias ($ExternalAuctionID) from 16 to 2 (online/active) \n" unless ($NoOutput);
                    $CountActiveAuctions++

                } elsif ($ItemListingStatus eq 'Completed') {
                    $EbayAuction->set({'Status' => 4});
                    print "EbayAuction '$EbayAuctionAlias' ($ExternalAuctionID) already has status 'Completed'. Set status from 16 to 4 \n" unless ($NoOutput);
                    $CountCompletedAuctions++

                } elsif ($ItemListingStatus eq 'Ended') {
                    $EbayAuction->set({'Status' => 32});
                    print "EbayAuction '$EbayAuctionAlias' ($ExternalAuctionID) already has status 'Ended'. Set status from 16 to 32 \n" unless ($NoOutput);
                    $CountEndedAuctions++
                } else {
                    print "GetItem call returned ListingStatus '$ItemListingStatus' for EbayAuction '$EbayAuctionAlias' ($ExternalAuctionID) \n";
                }
            }
        }
        print "Set  $CountActiveAuctions ebay auctions to status 'Online' \n"  unless ($NoOutput);
        print "Set  $CountCompletedAuctions ebay auction(s) to status 'Completed' \n"  unless ($NoOutput);
        print "Set  $CountEndedAuctions ebay auction(s) to status 'Ended' \n" unless ($NoOutput);

    }
}

RunScript(
    'Sub' => \&Main
);
