#========================================================================================
# package      DE_EPAGES::ExternalShipping::API::CSV::ShippingLabelExportDriver
# base         DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a csv driver to export object portal products
#
# example      use DE_EPAGES::ExternalShipping::API::CSV::ShippingLabelExportDriver;
#
#               my $Driver = DE_EPAGES::ExternalShipping::API::CSV::ShippingLabelExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->($FileName, $hFileFormat, $aColumnNames, $ViewObject);
#========================================================================================
package DE_EPAGES::ExternalShipping::API::CSV::ShippingLabelExportDriver;
use base DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver;

use strict;
use DE_EPAGES::Object::API::Factory                 qw (LoadObject );
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);

#========================================================================================
# function     exportCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportCSV($FileName, $hFileFormat, $aColumnNames, $ViewObject, $NoMonitor);
#----------------------------------------------------------------------------------------
# description  starts the csv export
#----------------------------------------------------------------------------------------
# input        $FileName | export to file | string
# input        $hFileFormat | file format <function DE_EPAGES::Presentation::API::CSV::MonitoredExportDriver::exportCSV> | ref.hash
# input        $aColumnNames | name of column | ref.array.string
# input        $ViewObject | export based on this object | PortalConfig
# input        $NoMonitor | deactivate monitor, eg. for scripts/scheduler | boolean
#========================================================================================
sub exportCSV {
    my $self = shift;
    my ($FileName, $hFileFormat, $aColumnNames, $ViewObject, $NoMonitor) = @_;

    $self->{'FulfillmentMethod'} = $ViewObject->get('ShippingFulfillmentMethod');
    $self->{'Object'} = $ViewObject;
    $self->{'NoMonitor'} = $NoMonitor;

    $self->SUPER::exportCSV($FileName, $hFileFormat, $aColumnNames, $ViewObject,[]);
    return;
}

#========================================================================================
# function     currencyID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CurrencyID = $self->currencyID;
#----------------------------------------------------------------------------------------
# description  
#----------------------------------------------------------------------------------------
# return       $CurrencyID | currency id (e.g. 'EUR') | char(3)
#========================================================================================
sub currencyID { return shift->{'CurrencyID'}; }

#========================================================================================
# function     languageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LanguageID = $self->languageID;
#----------------------------------------------------------------------------------------
# description  
#----------------------------------------------------------------------------------------
# return       $LanguageID | language | object
#========================================================================================
sub languageID { return shift->{'LanguageID'}; }

#========================================================================================
# function     shippingFulfillmentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentMethod = $self->shippingFulfillmentMethod;
#----------------------------------------------------------------------------------------
# description  used shipping fulfillment method
#----------------------------------------------------------------------------------------
# return       $ShippingFulfillmentMethod | shipping fulfillment method | object
#========================================================================================
sub shippingFulfillmentMethod { return shift->{'ShippingFulfillmentMethod'}; }

#========================================================================================
# function     startMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->startMonitor($PageTypeName);
#----------------------------------------------------------------------------------------
# description  starts the monitor (if plain "\rexport shipping labels (%d / %d)")
#----------------------------------------------------------------------------------------
# input        $PageTypeName | name of pagetype | string
#========================================================================================
sub startMonitor {
    my $self = shift;
    my ($PageTypeName) = @_;

    return if $self->{'NoMonitor'};
    return if exists $self->{'Monitor'};
    my $AppServer = GetAppServer();
    if (defined $AppServer) {
        my $Monitor = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
            'PageTypeName' => $PageTypeName,
            'Object'       => $self->object,
        );
        $self->{'Monitor'} = $Monitor;
    } else {
        my $Monitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
            'ContentTemplate' => "\rexport shipping labels (%d / %d)",
        );
        $self->{'Monitor'} = $Monitor;
    }
    return;
}

#========================================================================================
# function     writeHeader
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->writeHeader();
#----------------------------------------------------------------------------------------
# description  Writes the header line using the attribute names of the columns.
#========================================================================================
sub writeHeader {
    my $self = shift;

    $self->writeLine($self->columns);
    return;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->items;
#----------------------------------------------------------------------------------------
# description  Returns all product identifier of portal config, initialize monitor.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub items {
    my $self = shift;

    my $aLabelIDs = $self->object->items;
    my $Monitor = $self->monitor;
    $Monitor->set({'Total'=> scalar @$aLabelIDs}) if defined $Monitor;
    return $aLabelIDs;
}

#========================================================================================
# function     getExportValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getExportValues($Object);
#----------------------------------------------------------------------------------------
# description  helper method of <function exportLine> to convert column names to methods
#               names of this exporter and calls the method to get the values of the
#               exported object. The order of return values depends on order of column
#               names.
#----------------------------------------------------------------------------------------
# return       $aValues | list of values | ref.array.string
#========================================================================================
sub getExportValues {
    my $self = shift;
    my ($Object) = @_;
    my @Values;

    foreach my $ColumnName (@{$self->columns}) {
        my $MethodName = $ColumnName;
        $MethodName =~ s/\W/_/g;
        my $Value = $self->$MethodName( $Object );
        $Value = '' unless defined $Value;
        push @Values, $Value;
    }
    return \@Values;
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->exportLine;
#----------------------------------------------------------------------------------------
# description  Export one line, increments monitor.
#========================================================================================
sub exportLine {
    my $self = shift;
    my $Object = LoadObject($self->topID('Item'));

    my $Monitor = $self->monitor;
    if (defined $Monitor) {
        $self->monitorIncrement;
        $self->monitorContent;
    }

    $self->writeLine($self->getExportValues($Object));
    return;
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive($aObjects);
#----------------------------------------------------------------------------------------
# description  Overwrite standard drive methode to start monitor 'ShippingLabelMonitor'.
#========================================================================================
sub drive {
    my $self = shift;
    my ($aObjects) = @_;

    $self->startMonitor('ShippingLabelMonitor');
    $self->SUPER::drive($aObjects);
}

1;
