#======================================================================================
# package      DE_EPAGES::Kelkoo::API::XML::KelkooPremiumProductsExportDriver
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a SAX2 driver to export object Kelkoo products
#                Inherit from DE_EPAGES::Kelkoo::API::XML::KelkooProductsExportDriver
#
# example      use DE_EPAGES::Kelkoo::API::XML::KelkooPremiumProductsExportDriver;
#
#               my $Driver = DE_EPAGES::Kelkoo::API::XML::KelkooProductsExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->addKelkooHooks;
#               $Driver->exportXML;
#========================================================================================
package DE_EPAGES::Kelkoo::API::XML::KelkooPremiumProductsExportDriver;
use base DE_EPAGES::Kelkoo::API::XML::KelkooProductsExportDriver;

use strict;
use DE_EPAGES::Core::API::Warning               qw (Warning);
use DE_EPAGES::Core::API::Html                  qw (RemoveHtmlTags);
use DE_EPAGES::Kelkoo::API::Constants   qw ( %MAX_LENGTHS );

#======================================================================================
my @ADD_KELKOO_ATTRIBUTES = qw (
    PortalMerchantID
    ShortDescriptionAttribute
);

#======================================================================================
sub ErrorDefinition {
    return {
        'ProductNotVisible' => 'product ( #ProductID ) is not visible, can not export to Kelkoo.',
        'ProductNoName' => 'product ( #ProductID ) has no name, can not export to Kelkoo.',
        'ProductNoPrice' => 'product ( #ProductID ) has no price, can not export to Kelkoo.',
        'ProductNoDescription' => 'product ( #ProductID ) has no description, can not export to Kelkoo.',
        'ValueTooLong' => 'attribute ( #AttributeName ) for product ( #ProductID ) is too long (Maximum #MaxLength characters). Value has been cut.',
        'ProductNoImage' => 'product ( #ProductID ) has no image, can not export to Kelkoo.',
        'ProductNoURL' => 'product ( #ProductID ) has no URL, can not export to Kelkoo.',
        'ProductNoCategory' => 'product ( #ProductID ) has no category, can not export to Kelkoo.',
        'ProductNoAvailability' => 'product ( #ProductID ) has no availability, can not export to Kelkoo.',
        'ProductNoDeliveryPeriod' => 'product ( #ProductID ) has no delivery period, can not export to Kelkoo.',
    }
}

#======================================================================================
sub addKelkooHooks {
    my $self = shift;
    $self->addHook('products' => 'FeedOfKelkooConfig');

    my $ProductTagName = 'product';
    $self->addHook($ProductTagName  => 'TitleByProduct');
    $self->addHook($ProductTagName  => 'ProductURLByProduct');
    $self->addHook($ProductTagName  => 'PriceByProduct');
    $self->addHook($ProductTagName  => 'BrandByProduct');
    $self->addHook($ProductTagName  => 'DescriptionByProduct');
    $self->addHook($ProductTagName  => 'ImageUrlByProduct');
    $self->addHook($ProductTagName  => 'EANByProduct');
    $self->addHook($ProductTagName  => 'MerchantCategoryByProduct');
    $self->addHook($ProductTagName  => 'AvailabilityByProduct');
    $self->addHook($ProductTagName  => 'ShippingCostByProduct');
    $self->addHook($ProductTagName  => 'ShippingTimeByProduct');
    $self->addHook($ProductTagName  => 'ConditionByProduct');
    $self->addHook($ProductTagName  => 'EcotaxByProduct');
    $self->addHook($ProductTagName  => 'WarrantyByProduct');
    $self->addHook($ProductTagName  => 'MobileURLByProduct');
    $self->addHook($ProductTagName  => 'KelkooCategoryIDByProduct');
    $self->addHook($ProductTagName  => 'MPNByProduct');
    $self->addHook($ProductTagName  => 'SkuByProduct');
    $self->addHook($ProductTagName  => 'ColorByProduct');
    $self->addHook($ProductTagName  => 'UnitPriceByProduct');
    $self->addHook($ProductTagName  => 'OfferTypeByProduct');
    $self->addHook($ProductTagName  => 'MerchantInfoByProduct');
    $self->addHook($ProductTagName  => 'CurrencyByProduct');
    $self->addHook($ProductTagName  => 'ImageURL2ByProduct');
    $self->addHook($ProductTagName  => 'ImageURL3ByProduct');
    $self->addHook($ProductTagName  => 'ImageURL4ByProduct');
    $self->addHook($ProductTagName  => 'GreenProductByProduct');
    $self->addHook($ProductTagName  => 'GreenLabelByProduct');
    $self->addHook($ProductTagName  => 'SalesRankByProduct');
    $self->addHook($ProductTagName  => 'UnitQuantityByProduct');
    $self->addHook($ProductTagName  => 'MadeInByProduct');
    $self->addHook($ProductTagName  => 'OccasionByProduct');
    $self->addHook($ProductTagName  => 'ShippingMethodByProduct');
    $self->addHook($ProductTagName  => 'DeliveryCost2ByProduct');
    $self->addHook($ProductTagName  => 'ShippingMethod2ByProduct');
    $self->addHook($ProductTagName  => 'DeliveryCost3ByProduct');
    $self->addHook($ProductTagName  => 'ShippingMethod3ByProduct');
    $self->addHook($ProductTagName  => 'DeliveryCost4ByProduct');
    $self->addHook($ProductTagName  => 'ShippingMethod4ByProduct');
    $self->addHook($ProductTagName  => 'ZipCodeByProduct');
    $self->addHook($ProductTagName  => 'StockQuantityByProduct');
    $self->addHook($ProductTagName  => 'ShippingWeightByProduct');
    $self->addHook($ProductTagName  => 'PaymentMethodsByProduct');
    $self->addHook($ProductTagName  => 'VoucherTitleByProduct');
    $self->addHook($ProductTagName  => 'VoucherDescriptionByProduct');
    $self->addHook($ProductTagName  => 'VoucherUrlByProduct');
    $self->addHook($ProductTagName  => 'VoucherCodeByProduct');
    $self->addHook($ProductTagName  => 'VoucherStartDateByProduct');
    $self->addHook($ProductTagName  => 'VoucherEndDateByProduct');
    $self->addHook($ProductTagName  => 'PriceNoRebateByProduct');
    $self->addHook($ProductTagName  => 'PercentagePromoByProduct');
    $self->addHook($ProductTagName  => 'PromoStartDateByProduct');
    $self->addHook($ProductTagName  => 'PromoEndDateByProduct');
    $self->addHook($ProductTagName  => 'UserRatingByProduct');
    $self->addHook($ProductTagName  => 'NbReviewsByProduct');
    $self->addHook($ProductTagName  => 'UserReviewLinkByProduct');
    $self->addHook($ProductTagName  => 'VideoLinkByProduct');
    $self->addHook($ProductTagName  => 'VideoTitleByProduct');
}

#======================================================================================
# export Description
sub DescriptionByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();
    my $LanguageID = $self->languageID;

    my $DescriptionAttribute = $self->descriptionAttribute;
    return unless defined $DescriptionAttribute;

    my $Description = $Product->getWithParent($DescriptionAttribute->alias, $LanguageID);
    return Warning('ProductNoDescription', {
            'ProductID'      => $Product->id,
            'Product'        => $Product
        })  unless defined $Description;

    $Description = RemoveHtmlTags($Description);
    if (length($Description) > $MAX_LENGTHS{'Description'}) {
        Warning('ValueTooLong', {
            'ProductID'     => $Product->id,
            'Product'       => $Product,
            'AttributeName' => $DescriptionAttribute->get('NameOrAlias', $LanguageID),
            'MaxLength'     => $MAX_LENGTHS{'Description'}
        });
        $Description = substr($Description, 0, $MAX_LENGTHS{'Description'});
    }
    $self->addElement('description', {}, $Description);
    return;
}

#======================================================================================
# export Availability
#
# 1: StockLevel-MinOrder >= 0 and IsAvailable
# 5: StockLevel-MinOrder >= 0 but not IsAvailable
# 5: StockLevel-MinOrder < 0  but IsAvailable
# 5: StockLevel not defined   but IsAvailable
#
# 1 means the product is enougth on stock
# 5 means the product available to order
#=======================================================================================
sub AvailabilityByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();

    my $Code = undef;

    my $StockLevel = $Product->getWithParent('StockLevel');
    my $IsAvailable = $Product->getWithParent('IsAvailable');
    if ( defined $StockLevel ) {

        my $MinOrder = $Product->getWithParent('MinOrder');
           $MinOrder = $Product->getWithParent('IntervalOrder') unless $MinOrder;
           $MinOrder = 1 unless $MinOrder;
        my $IsStockOK = ($StockLevel-$MinOrder >= 0);

        $Code = 1 if  $IsStockOK &&  $IsAvailable;
        $Code = 5 if  $IsStockOK && !$IsAvailable;
        $Code = 5 if !$IsStockOK &&  $IsAvailable;

    } else { #not defined $StockLevel means no inventory is used

        $Code = 5 if $IsAvailable;

    }

    $self->addElement('availability', {}, $Code);
    return;
}

#======================================================================================
# export DeliveryPeriod
sub ShippingTimeByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();

    my $DeliveryPeriod =  $Product->getWithParent('DeliveryPeriod');
    $self->addElement('delivery-time', {}, $DeliveryPeriod);
    return;

}
#======================================================================================
# export manufacturer product number
sub MPNByProduct {
    my $self = shift;

    $self->addElement('mpn', {}, '');
    return;
}

#======================================================================================
# export EAN
sub EANByProduct {
    my $self = shift;
    my $Product = $self->getCurrentProduct();
    my $ean = $Product->getWithParent('UPCEAN');

    $self->addElement('ean', {}, $ean);
    return;
}

#======================================================================================
#export condition
sub ConditionByProduct {
    my $self = shift;

    $self->addElement('condition', {}, 0);
    return;
}
#======================================================================================
sub EcotaxByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();

    my $EcoTax = $Product->getEcoParticipation($self->currencyID, $self->taxModel);
    #return $EcoTax ? $self->formatPrice($EcoTax) : undef;

    $self->addElement('ecotax', {}, $EcoTax);
    return;
}
#======================================================================================
sub MobileURLByProduct {
    my $self = shift;

    $self->addElement('mobile-url', {}, '');
    return;
}
#======================================================================================
sub KelkooCategoryIDByProduct {
    my $self = shift;

    $self->addElement('kelkoo-category-id', {}, '');
    return;
}
#======================================================================================
sub SkuByProduct {
    my $self = shift;

    $self->addElement('sku', {}, '');
    return;
}
#======================================================================================
sub ColorByProduct {
    my $self = shift;

    $self->addElement('color', {}, 0);
    return;
}
#======================================================================================
sub UnitPriceByProduct {
    my $self = shift;
    my $Product = $self->getCurrentProduct();

    my $RefAmount = $Product->getWithParent('RefAmount');
    return '' if (!defined($RefAmount) || $RefAmount <= 0);

    my $RefContentAmount = $Product->getWithParent('RefContentAmount');
    return '' if (!defined($RefContentAmount) || $RefContentAmount <= 0);

    my $RefUnit = $Product->getWithParent('RefUnit');
    return '' unless defined $RefUnit;

    my $RefPrice = $self->productPrice($Product) * $RefAmount / $RefContentAmount;
    my $CurrencyID = $self->currencyID;
    my $Locale = $Product->getSite()->siteLocale($self->portalSite->get('LocaleID'));
    $RefPrice = $Locale->roundMoney($RefPrice, $CurrencyID);
    $RefPrice = $Locale->formatMoney($RefPrice, $CurrencyID); # don't use formatPrice as we need the currency symbol here

    my $unitPrice = $RefPrice . '/';
    $unitPrice .= $RefAmount unless $RefAmount eq '1';
    $unitPrice .= $RefUnit->get('Abbreviation') // $RefUnit->get('NameOrAlias', $self->languageID);

    $self->addElement('unit-price', {}, $unitPrice);
    return;
}
#======================================================================================
sub OfferTypeByProduct {
    my $self = shift;

    $self->addElement('offer-type', {}, '');
    return;
}
#======================================================================================
sub MerchantInfoByProduct {
    my $self = shift;

    $self->addElement('merchant-info', {}, '');
    return;
}
#======================================================================================
sub ImageURL2ByProduct {
    my $self = shift;

    $self->addElement('image-url-2', {}, '');
    return;
}
#======================================================================================
sub ImageURL3ByProduct {
    my $self = shift;

    $self->addElement('image-url-3', {}, '');
    return;
}
#======================================================================================
sub ImageURL4ByProduct {
    my $self = shift;

    $self->addElement('image-url-4', {}, '');
    return;
}
#======================================================================================
sub GreenProductByProduct {
    my $self = shift;

    $self->addElement('green-product', {}, '');
    return;
}
#=======================================================================================
sub GreenLabelByProduct {
    my $self = shift;

    $self->addElement('green-label', {}, '');
    return;
}
#======================================================================================
sub SalesRankByProduct {
    my $self = shift;

    $self->addElement('sales-rank', {}, '');
    return;
}
#======================================================================================
sub UnitQuantityByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();

    my $PriceQuantity = $Product->getWithParent('PriceQuantity');

    $self->addElement('unit-quantity', {}, $PriceQuantity);
    return;
}
#======================================================================================
sub MadeInByProduct {
    my $self = shift;

    $self->addElement('made-in', {}, '');
    return;
}
#======================================================================================
sub OccasionByProduct {
    my $self = shift;

    $self->addElement('occasion', {}, '');
    return;
}
#======================================================================================
sub ShippingMethodByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();

    my $ShippingMethod;
    if ($Product->get('IsDownloadProduct'))
    {
        $ShippingMethod = '';
    }
    else
    {
        my $LanguageID = $self->languageID;
        $ShippingMethod = $self->shippingMethod->get('NameOrAlias',$LanguageID);
    }

    $self->addElement('shipping-method', {}, $ShippingMethod);
    return;
}
#======================================================================================
sub DeliveryCost2ByProduct {
    my $self = shift;

    $self->addElement('delivery-cost-2', {}, '');
    return;
}
#======================================================================================
sub ShippingMethod2ByProduct {
    my $self = shift;

    $self->addElement('shipping-method-2', {}, '');
    return;
}
#======================================================================================
sub DeliveryCost3ByProduct {
    my $self = shift;

    $self->addElement('delivery-cost-3', {}, '');
    return;
}
#======================================================================================
sub ShippingMethod3ByProduct {
    my $self = shift;

    $self->addElement('shipping-method-3', {}, '');
    return;
}
#======================================================================================
sub DeliveryCost4ByProduct {
    my $self = shift;

    $self->addElement('delivery-cost-4', {}, '');
    return;
}
#======================================================================================
sub ShippingMethod4ByProduct {
    my $self = shift;

    $self->addElement('shipping-method-4', {}, '');
    return;
}
#======================================================================================
sub ZipCodeByProduct {
    my $self = shift;
    my $ZipCode = $self->topID('ShopAddress') ? $self->topID('ShopAddress')->get('Zipcode') : '';

    $self->addElement('zip-code', {}, $ZipCode);
    return;
}
#======================================================================================
sub StockQuantityByProduct {
    my $self = shift;

    my $Product = $self->getCurrentProduct();
    my $StockLevel = $Product->getWithParent('StockLevel');

    if ( defined $StockLevel ) {
        $self->addElement('stock-quantity', {}, $StockLevel);
    } else {
        $self->addElement('stock-quantity', {}, '');
    }
    return;
}
#======================================================================================
sub ShippingWeightByProduct {
    my $self = shift;

    $self->addElement('shipping-weight', {}, '');
    return;
}
#======================================================================================
sub PaymentMethodsByProduct {
    my $self = shift;

    $self->addElement('payment-methods', {}, '');
    return;
}
#======================================================================================
sub VoucherTitleByProduct {
    my $self = shift;

    $self->addElement('voucher-title', {}, '');
    return;
}
#======================================================================================
sub VoucherDescriptionByProduct {
    my $self = shift;

    $self->addElement('voucher-description', {}, '');
    return;
}
#======================================================================================
sub VoucherUrlByProduct {
    my $self = shift;

    $self->addElement('voucher-url', {}, '');
    return;
}
#======================================================================================
sub VoucherCodeByProduct {
    my $self = shift;

    $self->addElement('voucher-code', {}, '');
    return;
}
#======================================================================================
sub VoucherStartDateByProduct {
    my $self = shift;

    $self->addElement('voucher-start-date', {}, '');
    return;
}
#======================================================================================
sub VoucherEndDateByProduct {
    my $self = shift;

    $self->addElement('voucher-end-date', {}, '');
    return;
}
#======================================================================================
sub PriceNoRebateByProduct {
    my $self = shift;
    my $Product    = $self->getCurrentProduct();
    my $PriceNoRebateByProduct = $Product->getManufacturerPrice($self->currencyID, $self->taxModel);
    $self->addElement('price-no-rebate', {}, $PriceNoRebateByProduct);
    return;
}
#======================================================================================
sub PercentagePromoByProduct {
    my $self = shift;

    $self->addElement('percentage-promo', {}, '');
    return;
}
#======================================================================================
sub PromoStartDateByProduct {
    my $self = shift;

    $self->addElement('promo-start-date', {}, '');
    return;
}
#======================================================================================
sub PromoEndDateByProduct {
    my $self = shift;

    $self->addElement('promo-end-date', {}, '');
    return;
}
#======================================================================================
sub UserRatingByProduct {
    my $self = shift;

    $self->addElement('user-rating', {}, '');
    return;
}
#======================================================================================
sub NbReviewsByProduct {
    my $self = shift;

    $self->addElement('nb-reviews', {}, '');
    return;
}
#======================================================================================
sub UserReviewLinkByProduct {
    my $self = shift;

    $self->addElement('user-review-link', {}, '');
    return;
}
#======================================================================================
sub VideoLinkByProduct {
    my $self = shift;

    $self->addElement('video-link', {}, '');
    return;
}
#======================================================================================
sub VideoTitleByProduct {
    my $self = shift;

    $self->addElement('video-title', {}, '');
    return;
}

1;
