use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
JoinObjectPath
ExistsObjectByPath
LoadObjectByPath
LoadRootObject
);
use DE_EPAGES::Kelkoo::API::XML::KelkooPremiumProductsExportDriver;
use DE_EPAGES::Kelkoo::API::XML::KelkooProductsExportDriver;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use File::Temp qw ( tempdir tempfile );
use Readonly;
use Test::More;
use XML::Simple;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                testKelkooPremiumProductsExportDriver();
                testKelkooNormalProductsExportDriver();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub testKelkooPremiumProductsExportDriver {
    my $ExportDriver = DE_EPAGES::Kelkoo::API::XML::KelkooPremiumProductsExportDriver->new();
    my ( $FileName, $hFormats, $KelkooPortalConfig, $NoMonitor ) = getExportXMLParameters();

    $ExportDriver->exportXML( $FileName, $hFormats, $KelkooPortalConfig, $NoMonitor );

    my $ref = XMLin( $FileName, 'KeepRoot' => 1 );
    my $expected = getExpectedKelkooPremiumProductsExport();

    is_deeply( $ref, $expected, 'Correct Kelkoo Premium Products XML Export' );
}

sub testKelkooNormalProductsExportDriver {
    my $ExportDriver = DE_EPAGES::Kelkoo::API::XML::KelkooProductsExportDriver->new();
    my ( $FileName, $hFormats, $KelkooPortalConfig, $NoMonitor ) = getExportXMLParameters();

    $ExportDriver->exportXML( $FileName, $hFormats, $KelkooPortalConfig, $NoMonitor );

    my $ref = XMLin( $FileName, 'KeepRoot' => 1 );
    my $expected = getExpectedKelkooProductsExport();

    is_deeply( $ref, $expected, 'Correct Kelkoo Normal Products XML Export' );
}

sub getExportXMLParameters {

    my $KelkooPortalConfig
    = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShop', 'PortalConfigs', 'Kelkoo_en_GB' ] ) );
    my $dir = tempdir( CLEANUP => 1 );
    my ( undef, $FileName ) = tempfile( DIR => $dir );
    my $hFormats  = {};
    my $noMonitor = 1;

    return ( $FileName, $hFormats, $KelkooPortalConfig, $noMonitor );
}

sub getExpectedKelkooPremiumProductsExport {
    my $DomainName = LoadObjectByPath( JoinObjectPath( [ 'Test', 'TestShop' ] ) )->get('ProtocolAndServer');
    return {
        'products' => {
            'product' => [ {
                    'delivery-time' => {},
                    'nb-reviews'    => {},
                    'image-url-4'   => {},
                    'ean'           => '1123456987',
                    'product-url' =>
                    "$DomainName/epages/Store.sf/?Locale=en_GB&ObjectPath=/Test/TestShop/Products/cg_0101004270&ViewAction=ViewProductViaPortal",
                    'payment-methods'  => {},
                    'promo-start-date' => {},
                    'zip-code'         => {},
                    'delivery-cost-3'  => {},
                    'green-product'    => {},
                    'promo-end-date'   => {},
                    'description'      => 'Camping AZ CV270 valve gas canister Description',
                    'image-url' =>
                    "$DomainName/WebRoot/Store/Test/TestShop/Products/cg_0101004270/cg_0101004270_ml.jpg",
                    'stock-quantity'      => 15,
                    'price'               => '3.95',
                    'title'               => 'Camping AZ CV270 valve gas canister',
                    'user-rating'         => {},
                    'warranty'            => {},
                    'mobile-url'          => {},
                    'shipping-weight'     => {},
                    'shipping-method-2'   => {},
                    'sku'                 => {},
                    'unit-price'          => "\x{a3}1.72/100g",
                    'merchant-info'       => {},
                    'offer-type'          => {},
                    'unit-quantity'       => '1',
                    'shipping-method-3'   => {},
                    'brand'               => 'Campingaz',
                    'kelkoo-category-id'  => {},
                    'voucher-end-date'    => {},
                    'video-title'         => {},
                    'image-url-2'         => {},
                    'image-url-3'         => {},
                    'voucher-start-date'  => {},
                    'voucher-code'        => {},
                    'currency'            => 'GBP',
                    'delivery-cost-4'     => {},
                    'availability'        => '1',
                    'color'               => '0',
                    'shipping-method'     => 'Standard delivery',
                    'voucher-title'       => {},
                    'price-no-rebate'     => {},
                    'shipping-method-4'   => {},
                    'sales-rank'          => {},
                    'user-review-link'    => {},
                    'ecotax'              => {},
                    'occasion'            => {},
                    'percentage-promo'    => {},
                    'voucher-description' => {},
                    'delivery-cost-2'     => {},
                    'mpn'                 => {},
                    'green-label'         => {},
                    'delivery-cost'       => '7.45',
                    'voucher-url'         => {},
                    'made-in'             => {},
                    'video-link'          => {},
                    'condition'           => '0'
                }, {
                    'delivery-time' => {},
                    'nb-reviews'    => {},
                    'image-url-4'   => {},
                    'ean'           => '2312312412',
                    'product-url' =>
                    "$DomainName/epages/Store.sf/?Locale=en_GB&ObjectPath=/Test/TestShop/Products/cg_0101104470&ViewAction=ViewProductViaPortal",
                    'payment-methods'  => {},
                    'promo-start-date' => {},
                    'zip-code'         => {},
                    'delivery-cost-3'  => {},
                    'green-product'    => {},
                    'promo-end-date'   => {},
                    'description'      => 'Camping AZ CV470 valve Gas canister Description',
                    'image-url' =>
                    "$DomainName/WebRoot/Store/Test/TestShop/Products/cg_0101104470/cg_0101104470_ml.jpg",
                    'stock-quantity'      => 5,
                    'price'               => '7.95',
                    'title'               => 'Camping AZ CV470 valve Gas canister',
                    'user-rating'         => {},
                    'warranty'            => {},
                    'mobile-url'          => {},
                    'shipping-weight'     => {},
                    'shipping-method-2'   => {},
                    'sku'                 => {},
                    'unit-price'          => "\x{a3}1.77/100g",
                    'merchant-info'       => {},
                    'offer-type'          => {},
                    'unit-quantity'       => '1',
                    'shipping-method-3'   => {},
                    'brand'               => 'Campingaz',
                    'kelkoo-category-id'  => {},
                    'voucher-end-date'    => {},
                    'video-title'         => {},
                    'image-url-2'         => {},
                    'image-url-3'         => {},
                    'voucher-start-date'  => {},
                    'voucher-code'        => {},
                    'currency'            => 'GBP',
                    'delivery-cost-4'     => {},
                    'availability'        => '1',
                    'color'               => '0',
                    'shipping-method'     => 'Standard delivery',
                    'voucher-title'       => {},
                    'price-no-rebate'     => '9',
                    'shipping-method-4'   => {},
                    'sales-rank'          => {},
                    'user-review-link'    => {},
                    'ecotax'              => {},
                    'occasion'            => {},
                    'percentage-promo'    => {},
                    'voucher-description' => {},
                    'delivery-cost-2'     => {},
                    'mpn'                 => {},
                    'green-label'         => {},
                    'delivery-cost'       => '7.77',
                    'voucher-url'         => {},
                    'made-in'             => {},
                    'video-link'          => {},
                    'condition'           => '0'
                }
            ]
        }
    };
}

sub getExpectedKelkooProductsExport {
    my $DomainName = LoadObjectByPath( JoinObjectPath( [ 'Test', 'TestShop' ] ) )->get('ProtocolAndServer');
    return {
        'products' => {
            'product' => [ {
                    'delivery-time' => {},
                    'unit-price'    => "\x{a3}1.72/100g",
                    'ean'           => '1123456987',
                    'mpn'           => '1324567890',
                    'brand'         => 'Campingaz',
                    'description'   => 'Camping AZ CV270 valve gas canister Description',
                    'product-url' =>
                    "$DomainName/epages/Store.sf/?Locale=en_GB&ObjectPath=/Test/TestShop/Products/cg_0101004270&ViewAction=ViewProductViaPortal",
                    'currency' => 'GBP',
                    'image-url' =>
                    "$DomainName/WebRoot/Store/Test/TestShop/Products/cg_0101004270/cg_0101004270_ml.jpg",
                    'availability'  => '001',
                    'delivery-cost' => '7.45',
                    'title'         => 'Camping AZ CV270 valve gas canister',
                    'price'         => '3.95',
                    'warranty'      => {}
                }, {
                    'delivery-time' => {},
                    'unit-price'    => "\x{a3}1.77/100g",
                    'ean'           => '2312312412',
                    'mpn'           => '1324567890',
                    'brand'         => 'Campingaz',
                    'description'   => 'Camping AZ CV470 valve Gas canister Description',
                    'product-url' =>
                    "$DomainName/epages/Store.sf/?Locale=en_GB&ObjectPath=/Test/TestShop/Products/cg_0101104470&ViewAction=ViewProductViaPortal",
                    'currency' => 'GBP',
                    'image-url' =>
                    "$DomainName/WebRoot/Store/Test/TestShop/Products/cg_0101104470/cg_0101104470_ml.jpg",
                    'availability'  => '001',
                    'delivery-cost' => '7.77',
                    'title'         => 'Camping AZ CV470 valve Gas canister',
                    'price'         => '7.95',
                    'warranty'      => {}
                }
            ]
        }
    };
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'exportKelkoo.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
