#========================================================================================
# package      DE_EPAGES::Object::API::Cartridge
# base         DE_EPAGES::Object::API::ObjectInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Object::API::Cartridge;
use base DE_EPAGES::Object::API::ObjectInstaller;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::File qw ( RunOnGlob CopyDirectory );
use DE_EPAGES::Core::API::Net qw( localhostName );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf trim );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias LoadRootObject Transaction );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Object::Class;
use DE_EPAGES::Object::API::Table::Currency qw( UpdateCurrency );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguagesByCode2 UpdateLanguage );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Object::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Object.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw(ProhibitLongLines)
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Object',
        'Version'            => '7.94.0',
        'Patches'            => ['7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0','7.85.0','7.86.0','7.87.0','7.88.0','7.89.0','7.90.0','7.91.0','7.92.0','7.93.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;

    # note: ObjectInstaller::install() cannot be used here, because it requires initStructure()
    $self->DE_EPAGES::Cartridge::API::CartridgeInstaller::install;

    # insert the root object
    DE_EPAGES::Object::API::Object::Class->initStructure;

    # import languages etc
    Transaction( sub {
        GetLog->debug(sprintf('import standards %s into store %s.', $self->cartridgePackage, $self->storeName));
        ImportObjects( $self->xmlDirectory . '/Languages.xml');
        ImportObjects( $self->xmlDirectory . '/Currencies_4217.xml');

        GetLog->debug(sprintf('install objects %s on store %s.', $self->cartridgePackage, $self->storeName));
        my $XMLDir = $self->xmlDirectory;
        RunOnGlob($XMLDir, 'Attributes*.xml', sub { ImportObjects( $_ ) });

        ImportObjects( $XMLDir . '/Countries_3166.xml');
        RunOnGlob($XMLDir, 'CountrySubdivision*.xml', sub { ImportObjects( $_ ) });

        ImportObjects( $XMLDir . '/Locales.xml');
        ImportObjects( $XMLDir . '/System.xml', '/');
        ImportObjects( $XMLDir . '/UnitsOfMeasurement.xml', '/');

        $self->_activateLocales;

        ImportTranslation( "$XMLDir/Translation.Attributes" );
        ImportTranslation( "$XMLDir/Translation.UnitsOfMeasurement" );
    });

    # update the web server URL
    my $System = LoadRootObject();
    my $SystemDomainName = EpagesConfig()->get( 'DE_EPAGES::Object', 'SystemDomainName',  localhostName() );
    $System->set({ DomainName => $SystemDomainName });
    $System->set({ WebServerScriptNamePart => $self->storeName });

    $self->_copyWebRootIcons;

    $self->finishInstall;
    return;
}

#========================================================================================
# function     _activateLocales
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_activateLocales();
#----------------------------------------------------------------------------------------
# description  Activates languages, locales and currenies for a new database.
#               The defaults can be overloaded by setting environment variables, e.g.:
#               EPAGES_LANGUAGES=de en-US
#               EPAGES_LOCALES=de_DE de_AT en_GB en_US
#               EPAGES_CURRENCIES=EUR GBP USD
#========================================================================================
sub _activateLocales {
    my $self = shift;

    my $Languages = $ENV{EPAGES_LANGUAGES} || 'en de';
    my @LanguageIdentifiers = map { trim($_) } split /\s+/, $Languages;
    my @LanguageCodes = map { /^(\w+)-\w+$/ ? $1 : $_ } @LanguageIdentifiers;
    my @LanguageIDs = map { GetPKeyLanguageByCode( $_ ) } @LanguageCodes;
    Error('NoLanguages') if 0 == scalar @LanguageIDs;

    # first locale becomes the system default
    my $Locales = $ENV{EPAGES_LOCALES} || 'en_GB en_US en_AU en_IE en_CA en_NZ en_CH de_DE de_CH de_AT de_LU de_BE';
    my @LocaleIDs = map { trim($_) } split /\s+/, $Locales;
    Error('NoLocales') if 0 == scalar @LocaleIDs;

    my $Currencies = $ENV{EPAGES_CURRENCIES} || 'USD GBP EUR JPY SEK AUD CHF CAD NZD';
    my @CurrencyIDs = map { trim($_) } split /\s+/, $Currencies;
    Error('NoCurrencies') if 0 == scalar @CurrencyIDs;

    # activate languages
    UpdateLanguage( $_, { IsUsed => 1 } ) foreach @LanguageIDs;

    # switch dialects by importing a file like "Languages.en-US.xml"
    my @Dialects = grep { /^(\w+)-(\w+)/ } @LanguageIdentifiers;
    ImportObjects( $self->xmlDirectory . "/Languages.$_.xml" ) foreach @Dialects;

    # activate locales
    my $LocaleClass = LoadClassByAlias('Locale');
    $LocaleClass->loadObject($_)->set({ IsUsed => 1 }) foreach @LocaleIDs;

    # activate currencies
    UpdateCurrency( $_, { IsUsed => 1 } ) foreach @CurrencyIDs;

    # deactivate german (de)
    my $LanguageID_DE = GetPKeyLanguageByCode('de');
    unless( ElementOf($LanguageID_DE, @LanguageIDs) ) {
        UpdateLanguage( $LanguageID_DE, { IsUsed => 0 } );
    }

    # decativate german (de_DE)
    unless( ElementOf('de_DE', @LocaleIDs) ) {
        $LocaleClass->loadObject('de_DE')->set({ IsUsed => 0 });
    }

    my $System = LoadRootObject();
    # set system default locale
    $System->set({ LocaleID => $LocaleIDs[0] });
    # set system timezone if defined ENV{EPAGES_TIMEZONE}
    if (defined $ENV{EPAGES_TIMEZONE} and DateTime::TimeZone->is_valid_name($ENV{EPAGES_TIMEZONE})) {
        $System->set({'TimeZone' => $ENV{EPAGES_TIMEZONE} });
    }
    return;
}

#========================================================================================
# function     _copyWebRootIcons
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->_copyWebRootIcons();
#----------------------------------------------------------------------------------------
# description  copy images in Data/Public/SF/{Countries,Locales} also to WebRoot/Icons
#========================================================================================
sub _copyWebRootIcons {
    my $self = shift;

    # EPG-27542: copy images in Data/Public/SF/{Countries,Locales} also to WebRoot/Icons
    # overwrites the method from Core::API::FileInstaller
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public/SF');
    my $TargetDir = "$ENV{EPAGES_WEBROOT}/Icons";

    CopyDirectory("$SourceDir/$_", "$TargetDir/$_") foreach(qw(Countries Locales));
}


#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub patch_7_33_0 {
    my $self = shift;

    $self->importPatchDir('Patch_7_33_0', 'Patch');

    $self->_copyWebRootIcons;
    return '7.34.0';
}

sub patch_7_42_0 {
    my $self = shift;

    $self->importPatchDir('Patch_7_42_0', 'Patch');

    # See AD-9649 - add en_CH, copy icons
    if( GetInfoLanguagesByCode2('en')->[0]->{'IsUsed'} ) {
        $self->logInfo('activate system locale en_CH');

        my $Locale = LoadClassByAlias('Locale')->loadObject('en_CH');
        $Locale->set({ IsUsed => 1 });
    }
    $self->_copyWebRootIcons;
    return '7.43.0';
}

1;
