#========================================================================================
# package      DE_EPAGES::Order::API::BasketOrderValidator
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  Functions for checking the Basket or Order object.
#========================================================================================
package DE_EPAGES::Order::API::BasketOrderValidator;
use base Exporter;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );


our @EXPORT_OK = qw ( IsAValidBasketOrOrder IsMaxBasketPositionsReached );

#========================================================================================
# function     IsAValidBasketOrOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsAValidBasketOrOrder($BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Checks if a given object is a valid basket or order.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or order | object
# return       $IsAValidBasketOrOrder | true if the object is a basket or order | boolean
#========================================================================================
sub IsAValidBasketOrOrder {
    my ($BasketOrOrder) = @_;

    return 0 unless defined($BasketOrOrder);
    return 1 if (_isABasket($BasketOrOrder) || _isAnOrder($BasketOrOrder));
    return 0;
}

#========================================================================================
# function     IsMaxBasketPositionsReached
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsMaxBasketPositionsReached($BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Checks if the amount of items in the basket or order is >= than the max allowed amount.
#               The system attribute MaxBasketPositions is used here. (Default:50)
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or order | object
# return       $IsMaxBasketPositionsReached | true if max positions reached  | boolean
#========================================================================================
sub IsMaxBasketPositionsReached {
    my ($BasketOrOrder) = @_;

    return unless IsAValidBasketOrOrder($BasketOrOrder);
    my $Container = $BasketOrOrder->container;

    my $MaxBasketPositions = LoadRootObject()->get('MaxBasketPositions');

    # max allowed basket positions reached?
    return 1 if scalar(@{$Container->get('LineItems')}) >= $MaxBasketPositions;
    return 0;

}

#========================================================================================
# function     _isAnOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isAnOrder($BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Checks if a given object is an order.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or order | object
# return       $IsOrder | true if the object is a order | boolean
#========================================================================================
sub _isAnOrder {
    my ($BasketOrOrder) = @_;

    return $BasketOrOrder->instanceOf('CustomerOrder');
}

#========================================================================================
# function     _isABasket
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isABasket($BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Checks if a given object is a basket.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or order | object
# return       $IsBasket | true if the object is a basket | boolean
#========================================================================================
sub _isABasket {
    my ($BasketOrOrder) = @_;

    return $BasketOrOrder->instanceOf('Basket');
}

1;