#========================================================================================
# package      DE_EPAGES::Presentation::API::Cartridge
# base         DE_EPAGES::Presentation::API::PresentationInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Presentation::API::Cartridge;
use base DE_EPAGES::Presentation::API::PresentationInstaller;

use strict;
use DE_EPAGES::Trigger::API::Table::Hook qw (ExistsHookByName GetPKeyHookByName InsertHook);
use DE_EPAGES::Trigger::API::Table::HookFunction qw (
    DeleteHookFunction
    ExistsHookFunctionByHookID
    GetPKeyHookFunctionByHookID
);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Presentation::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Presentation.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw(ProhibitLongLines)
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Presentation',
        'Version'            => '7.94.0',
        'Patches'            => ['7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0','7.85.0','7.86.0','7.87.0','7.88.0','7.89.0','7.90.0','7.91.0','7.92.0','7.93.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub doPatch {
    my $self = shift;

    # TODO: remove this code when the the patch does no longer support 6.0.7 or early versions
    if( ExistsHookByName('OBJ_AfterUpdateMenu') ) {
        # EPG-13763: avoid error: Undefined subroutine &DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu
        my $HookID = GetPKeyHookByName('OBJ_AfterUpdateMenu');
        if( ExistsHookFunctionByHookID($HookID, 'DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu') ) {
            my $HookFunctionID = GetPKeyHookFunctionByHookID($HookID, 'DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu');
            DeleteHookFunction($HookFunctionID);
        }
    }
    else {
        # avoid warning in patch: Hook doesn't exist (Name=OBJ_AfterUpdateMenu)
        InsertHook({ 'Name' => 'OBJ_AfterUpdateMenu' });
    }

    return $self->SUPER::doPatch;
}

#========================================================================================
# function     patchCartridgeDir_7_12_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $NewVersion = $Installer->patchCartridgeDir_7_12_0;
#----------------------------------------------------------------------------------------
# description  Patches Cartridge directory
#               from version '7.12.0' to version '7.13.0'.
#----------------------------------------------------------------------------------------
# return       $NewVersion | new version | string
#========================================================================================
sub patchCartridgeDir_7_12_0 {
    my $self = shift;

    $self->onPatchDeleteCartridgeDirectory('bin');

    return '7.13.0';
}

1;
