#==============================================================================
# package      DE_EPAGES::Product::API::Product
# state        public
# description  API helper functions for product variations.
#==============================================================================
package DE_EPAGES::Product::API::Product;
use base Exporter;
use strict;
our @EXPORT_OK  = qw (
    CropProductImages
    InsertMultipleSubProducts
    ExistsSubProductByAttributeValues
    GetSubProductByAttributeValues
    SubProductFolder
    UsedVariationValues
    UpdateSelectedVariationsOfMasterProduct
    UpdateAvailability
    AddSubProductPricesFromMaster
    ScaleImageSizes
    TestUPCEAN
    DeleteProductImage
    RenameProductImageFile
    RenameSubProductImageFile
    SmartSearchProducts
    GenerateCustomizedProductPDF
    SendCustomizationDocumentAsPDFMail
    GetVisibleProductIDs
    StoreProductImages
    DeleteProductImageByAttribute
    GetScaledImages
    GetVisibleSubProductIDs
    StartUnityProgressMonitor
    UpdateUnityProgressMonitor
    GetUnityPrettyUrl
    GetProductPrettyUnityUrlsRequest
    GetVariationAttributes
    SaveProductPrettyUrlsResponse
);
use DE_EPAGES::Product::API::Table::SelectedVariation qw (
    GetInfoSelectedVariationsByProductID
);

use Algorithm::CheckDigits;
use Config;
use DateTime;
use Image::Magick;
use JSON;
use DE_EPAGES::Core::API::Constants qw ( IMAGE_FILE_TYPES );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw (
    ConvertPathChars
    CreateDirectory
    DeleteFile
    ExistsFile
    fsplit
    GetAccessRights
    SetAccessRights
    MoveFile
    CleanPath
);
use DE_EPAGES::Core::API::Image qw (
    CorrectImage
    GetImageInfo
    ResizeImageUsingImageRef
    TestImageFormat
);
use DE_EPAGES::Core::API::Log qw ( SetLogLevel );
use DE_EPAGES::Core::API::PerlTools qw (
    isValid
    ElementOf
    GetCurrentDateTime
    SubtractDuration
    unique
);
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestInteger TestObject TestString TestHash);
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::EPagesJ::API::PDF::PdfClient;
use DE_EPAGES::MailType::API::TextMail qw ( SendTextMail );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadObjectByGUID );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::SelectPreDefAttribute qw (
    GetInfoSelectPreDefAttributesByObjectID
);
use DE_EPAGES::Presentation::API::LocaleHandler qw ( AddLanguageToURL );
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage );
use DE_EPAGES::Product::API::Constants qw ( PRODUCT_IMAGE_SIZE_NAMES PRODUCT_IMAGES_SIZES PRODUCT_IMAGES_JPEG_QUALITIES
                                            UNITY_ATTRIBUTES_FROM_MASTERPRODUCT_WRITTEN_ON_SUBS_ON_CREATE
                                            UNITY_LOC_ATTRIBUTES_FROM_MASTERPRODUCT_WRITTEN_ON_SUBS_ON_CREATE
                                            UNITY_IMAGE_ATTRIBUTES);
use DE_EPAGES::Product::API::Shop qw ( GetProductImageSizes );
use DE_EPAGES::Product::API::Stock qw ( SendProductAvailibityNotification );
use DE_EPAGES::Product::DAL::Product qw (
    DAL_GetSmartSearchProductIDs
    DAL_GetSubProductIDsBySuperProductID
);
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );

use constant MAX_PAGE_SIZE          => 10000;
use constant DIGITS_EAN_13          => 13;
use constant DIGITS_UPC_VA          => 12;
use constant DIGITS_ISBN_10         => 10;
use constant DIGITS_EAN_8_o_UPC_VE  => 8;
use constant EPAGES_UI_WEBSERVICE_REQUEST_TIMEOUT => 6;

##  no critic qw(ProhibitLongLines)
sub ErrorDefinition {
    return {
        'VariationCombinationExists' => 'combination of this variations exists already. product: #ProductID ',
        'SubProductsCountFeatureLimitExceeded' => 'Attempt insert a new subproduct for product ( #ProductID ). But only #MaxValue subproducts per product are allowed.',
        'PARAMETER_MISSING'  => 'Missing parameter #Parameter (#ParameterName)',
        'FORMAT_NOT_UPCEAN'  => '#Parameter is not a UPC or EAN number (#Value)',
        'ANOTHERFILEALREADYEXISTS'  => 'Another file with the file name #File already exists',
        'SUBPRODUCT_NOT_EXISTS'  => 'There exists no sub product having the super product id \'#SuperProductID\' and the variation combination \'#Variations\'.',
        'TOO_MANY_SUBPRODUCTS'  => 'More than one sub product exists having the super product id \'#SuperProductID\' and the given variation combination \'#Variations\'. List of sub product ids: \'#SubProductIDs\'.',
        'NO_UNITY_SHOP'  => 'This is not a Unity shop. path: \'#ShopPath\'.',
    }
}

#========================================================================================
# function     SubProductFolder
#----------------------------------------------------------------------------------------
# syntax       $Folder = SubProductFolder( $SuperProduct );
#----------------------------------------------------------------------------------------
# description  Creates/returns a folder for sub products (type ProductFolder).
#----------------------------------------------------------------------------------------
# input        $Product | super product | object
# return       $Folder | sub product folder | object
#========================================================================================
sub SubProductFolder {
    my ($SuperProduct) = @_;
    return $SuperProduct->existsChild('SubProducts') ? $SuperProduct->child('SubProducts') :
        LoadClassByAlias('ProductFolder')->insertObject({
            'Parent' => $SuperProduct,
            'Alias'  => 'SubProducts',
        });
}

#========================================================================================
# function     InsertMultipleSubProducts
#----------------------------------------------------------------------------------------
# syntax       InsertMultipleSubProducts( $SuperProduct, $ahVariationTypes, $hValues );
# syntax       InsertMultipleSubProducts( $SuperProduct, $ahVariationTypes, $hValues, $hOptions );
#----------------------------------------------------------------------------------------
# description  Creates a set of new sub products for a given master product, every combination
#               of the given variations will be reflected as a own sub product.
#----------------------------------------------------------------------------------------
# input        $SuperProduct    | super product     | object
# input        $ahVariationTypes | values for the variation attributes | with following keys:
#               <ul>
#                   <li>Attribute  - variation type attribute - object
#                   <li>Values - Values (PreDefAttributes) - reference to array of objects
#               </ul>|reference to array of hashes
# input        $hValues | non variation attribute values for the new sub product | ref of hash
# input        $hOptions | (optional) params for processing | with following keys:
#               <ul>
#                   <li>(optional) AliasFromAttributeValueAlias - creates the sub product
#                       Alias from the Alias of the product + Aliases of the attribute
#                       values, for example "TshirtRedXXL" - boolean</li>
#                   <li>(optional) Monitor - Monitor options,
#                       if given the Unity progress monitor will be displayed, parameters:
#                       <ul>
#                           <li>Servlet       - servlet object                   - object</li>
#                       </ul>
#                   </li>
#               </ul>|reference to hash
#========================================================================================
sub InsertMultipleSubProducts {
    my ($SuperProduct, $ahVariationTypes, $hValues, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;
    # store monitor variables
    my $Servlet= (exists $hOptions->{'Monitor'} && exists $hOptions->{'Monitor'}->{'Servlet'}) ?
        $hOptions->{'Monitor'}->{'Servlet'} : undef;
    my $ShouldRenderMonitor = defined $Servlet;

    die 'Can\'t create variations without attribute values' if 0 == scalar @$ahVariationTypes;

    my $Shop = $SuperProduct->getSite();
    my $Folder = SubProductFolder($SuperProduct);

    # create new product
    my $Class = $SuperProduct->class;

    my $SubProductsCount = $SuperProduct->countSubProducts;
    my $ProductMaxSubProducts = $Shop->featureMaxValue('ProductMaxSubProducts');

    my %SelectedVariations = map { $_->{'SelectedID'} => 1 } @{$SuperProduct->get('SelectedVariations')};

    # prepare the attribute/value combinations
    my @VariationAttributeValues = ( [] );

    foreach my $hVariationType (@$ahVariationTypes) {
        my $aVariationValues = $hVariationType->{'Values'};

        @VariationAttributeValues = map {
            my $aItem = $_;
            map { [ @$aItem, $_ ] } @$aVariationValues;
        } @VariationAttributeValues;
    }

    my %CommonValues = (
      'Parent'       => $Folder,
      'SuperProduct' => $SuperProduct,
      'IsNew'        => $SuperProduct->get('IsNew'),
      'NewnessDate'  => $SuperProduct->get('NewnessDate'),
      'IsAvailable'  => $SuperProduct->get('IsAvailable'),
      'IsDailyPrice' => $SuperProduct->get('IsDailyPrice'),
      'IsDownloadProduct' => $SuperProduct->get('IsDownloadProduct'),
      'IsVisible' => $hOptions->{'IsVisible'} // 0,
      %$hValues,
    );
    delete $CommonValues{'Alias'};

    my $InsertSubProduct = sub {
        my( $Alias, $aVariationValues ) = @_;

        # make sure all attribute values are included in SelectedVariations
        foreach my $Value ( @$aVariationValues ) {
            if( !exists $SelectedVariations{$Value->id} ) {
                $SelectedVariations{$Value->id} = 1;
                $SuperProduct->set( { 'SelectedVariations' => [ map { {'SelectedID' => $_} } keys %SelectedVariations ] } );
            }
        }

        my %Values = (
            %CommonValues,
            Alias => $Alias,
            map { $_->get('Attribute')->alias => $_ } @$aVariationValues,
        );
        delete $Values{'GUID'};

        # delete language dependend
        delete $Values{$_} foreach grep { ref $Values{$_} eq 'HASH' } keys %Values;

        my $SubProduct = $Class->insertObject( \%Values );

        # set the new default sub product only if we have a default sub product already set
        $SuperProduct->set({'DefaultSubProduct'=>$SubProduct}) if defined $SuperProduct->get('DefaultSubProduct');

        # set unity specific attributes to each sub product
        if($Shop->get('IsUnityRegistered')){

                #own attributes on each subproduct: "all Images, Name, Weight....."
                # set non localized
                my @GetAttributes = (UNITY_IMAGE_ATTRIBUTES, UNITY_ATTRIBUTES_FROM_MASTERPRODUCT_WRITTEN_ON_SUBS_ON_CREATE);
                my $hNonLocValues = $SuperProduct->get(\@GetAttributes);
                $hNonLocValues->{'HasSubOwnPrices'} = 1;
                $SubProduct->set($hNonLocValues);

                # set localized attributes like "name"
                my $aShopLanguages = $Shop->get('Languages');
                my @LocalizedAttributesWrittenOnCreate = UNITY_LOC_ATTRIBUTES_FROM_MASTERPRODUCT_WRITTEN_ON_SUBS_ON_CREATE;
                # add user defined attibute values
                push @LocalizedAttributesWrittenOnCreate, map { $_->{'Attribute'}->alias } @{ $SuperProduct->get('NonVariationUserDefinedAttributeValues') };
                foreach my $Language (@$aShopLanguages) {
                    $SubProduct->set( {$_ => $SuperProduct->get($_, $Language->{'LanguageID'})}, $Language->{'LanguageID'} ) foreach @LocalizedAttributesWrittenOnCreate;
                }

                # set all Prices
                my $InputTaxModel = $Shop->get('InputTaxModel');
                my $aCurrencyIDs = [map {$_->{'CurrencyID'}} @{$Shop->get('Currencies')}];
                foreach my $CurrencyID (@$aCurrencyIDs) {
                    my $ListPrice = $SuperProduct->getListPrice($CurrencyID, $InputTaxModel,1);
                    $ListPrice = undef if defined $ListPrice and $ListPrice !~ /\S/;
                    $SubProduct->setListPrice( $CurrencyID, $InputTaxModel, $ListPrice );

                    my $ManufPrice = $SuperProduct->getManufacturerPrice($CurrencyID, $InputTaxModel,1);
                    $ManufPrice = undef if defined $ManufPrice and $ManufPrice !~ /\S/;
                    $SubProduct->setManufacturerPrice( $CurrencyID, $InputTaxModel, $ManufPrice );

                    my $Deposit = $SuperProduct->getDeposit($CurrencyID, $InputTaxModel,1);
                    $Deposit = undef if defined $Deposit and $Deposit !~ /\S/;
                    $SubProduct->setDeposit( $CurrencyID, $InputTaxModel, $Deposit );
                }

            # set ShippingMethods for Subproduct
            $SubProduct->set({'ShippingMethods' => $SuperProduct->get('ShippingMethods')});
        }

    };

    # start the progress monitor in case one has been handed over
    if ($ShouldRenderMonitor) {
        StartUnityProgressMonitor($Servlet, $SuperProduct, \@VariationAttributeValues);
    }

    foreach my $aVariationValues (@VariationAttributeValues) {

        # compare current count of subproducts
        if ($SubProductsCount >= $ProductMaxSubProducts ) {
            Warning('SubProductsCountFeatureLimitExceeded', {
                'ProductID' => $SuperProduct->id,
                'CurrentCount' => $SubProductsCount ,
                'MaxValue' => $ProductMaxSubProducts,
            });
            return;
        }

        if( ExistsSubProductByAttributeValues($SuperProduct->id, [ map { $_->id } @$aVariationValues ]) ) {
            Warning('VariationCombinationExists', {'ProductID'=>$SuperProduct->id});
            next;
        };
        $SubProductsCount++;

        # generate the alias for the sub product
        my $Alias;
        if ($hOptions->{'AliasFromAttributeValueAlias'}) {
            # create the alias from the attribute aliases
            $Alias = $SuperProduct->alias. join('', map { $_->alias } @$aVariationValues);
        }
        elsif (defined $hValues->{'Alias'} and $hValues->{'Alias'} ne '' ) {
            $Alias = $Folder->existsChild($hValues->{'Alias'})
                ? $Folder->nextAlias($hValues->{'Alias'} . '-%04d')
                : $hValues->{'Alias'};
        }
        else {
            $Alias = $Folder->nextAlias($SuperProduct->alias . '-%04d');
        }

        $InsertSubProduct->( $Alias, $aVariationValues );

        # update the progress monitor in case one has been handed over
        if ($ShouldRenderMonitor) {
            UpdateUnityProgressMonitor($Servlet);
        }
    }

    return;
}

#========================================================================================
# function     GetSubProductByAttributeValues
#----------------------------------------------------------------------------------------
# syntax       $SubProduct = GetSubProductByAttributeValues($SuperProductID, $aVariations);
#----------------------------------------------------------------------------------------
# description  Returns the sub products of a super product having the given set of
#               variation values. In general, one can expect that only a single
#               sub product will match the given variation combination.
#
#               It should be noted, however, that the following example might be valid:
#               Given a super product with various variations (e.g. color, size, weight).
#               A user might only be interested in all sub products having the color
#               'red'. In this case, the result set is likely to be greater than 1.
#
#               Since this subroutine is used to select a specific sub product in the
#               storefront, such a query does de facto not exist. Hence, if more than one
#               sub product matches, this subroutine returns the error
#               'TOO_MANY_MATCHING_SUBPRODUCTS'; the subroutine raises the error
#               'SUBPRODUCT_NOT_EXISTS', if the variation combination ($aVariations)
#               itself does not exist for the given super product.
#----------------------------------------------------------------------------------------
# input        $SuperProductID | product type id (part of primary key) | int
# input        $aVariations | array of PreDefAttribute object ids of the attribute values | ref.array.int
# return       $SubProduct | sub product object | object
#========================================================================================
sub GetSubProductByAttributeValues ($$) {## no critic qw(Subroutines::ProtectPrivateSubs)
    my ($SuperProductID, $aVariations) = @_;

    TestInteger('SuperProductID', $SuperProductID);
    TestArray('aVariations', $aVariations);

    my $aSubProductIDs = DAL_GetSubProductIDsBySuperProductID($SuperProductID, $aVariations);

    Error('SUBPRODUCT_NOT_EXISTS', {
        'SuperProductID' => $SuperProductID,
        'Variations'     => join(',', @{ $aVariations }),
    }) unless @{ $aSubProductIDs };

    Error('TOO_MANY_MATCHING_SUBPRODUCTS', {
        'SuperProductID' => $SuperProductID,
        'Variations'     => join(',', @{ $aVariations }),
        'SubProductIDs'  => join(',', @{ $aSubProductIDs }),
    }) unless scalar @{ $aSubProductIDs } == 1;

    return LoadObject( $aSubProductIDs->[0] );
}

#========================================================================================
# function     ExistsSubProductByAttributeValues
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsSubProductByAttributeValues($SuperProductID, $aVariations);
#----------------------------------------------------------------------------------------
# description  Returns true if a sub product exists having both the given superproduct
#               id and the combination of variations. This subroutine should be preferred
#               over GetSubProductByAttributeValues() when the objective is just to
#               get knowledge about whether or not a sub product exists.
#----------------------------------------------------------------------------------------
# input        $SuperProductID | product type id (part of primary key) | int
# input        $aVariations | array of PreDefAttribute object ids of the attribute values | ref.array.int
# return       $Exists | true if subproduct exists, otherwise false | boolean
#========================================================================================
sub ExistsSubProductByAttributeValues ($$) {
    my ($SuperProductID, $aVariations) = @_;

    TestInteger('SuperProductID', $SuperProductID);
    TestArray('aVarations', $aVariations);

    my $aSubProductIDs = DAL_GetSubProductIDsBySuperProductID($SuperProductID, $aVariations);
    return @{ $aSubProductIDs };
}

#========================================================================================
# function     UsedVariationValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aValueIDs = UsedVariationValues($Product);
# example      my $aValueIDs = UsedVariationValues($Product);
#               foreach my $ObjectID (@$aValueIDs) {
#                   print LoadObject($ObjectID)->get('Name'), "\n";
#               }
#----------------------------------------------------------------------------------------
# description  Returns the list of all variation values that are actually used by sub
#               products of the given super product. Returns undef if a sub product
#               is passed to this subroutine.
#----------------------------------------------------------------------------------------
# input        $Product | super product | object
# return       $aValueIDs | list of value ids | ref.array.integer
#========================================================================================
sub UsedVariationValues {
    my ($Product) = @_;
    TestObject('Product', $Product);

    return undef if $Product->isSubProduct;

    return [
        unique
        map {
            map { $_->{'PreDefAttributeID'} }
            @{ GetInfoSelectPreDefAttributesByObjectID($_) }
        }
        @{ $Product->findSubProducts }
    ];
}

#========================================================================================
# function     UpdateSelectedVariationsOfMasterProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateSelectedVariationsOfMasterProduct( $aProducts );
# example      UpdateSelectedVariationsOfMasterProduct( $aProducts );
#----------------------------------------------------------------------------------------
# description  Update attribute 'SelectedVariations' of a master product to those currently used
#----------------------------------------------------------------------------------------
# input        $aProducts | list of master products | ref.array.object
#========================================================================================
sub UpdateSelectedVariationsOfMasterProduct {
    my ($aProducts) = @_;

    foreach my $Product ( @$aProducts ) {
        next unless $Product->isMaster();
        my %SelectedVariations = map { $_ => 1 } @{ UsedVariationValues($Product) };

        # %SelectedVariations holds every assigned/used PreDefAttribute also the ones that are not visible
        # we need to check if there are existing variation with the PreDefAttributes, if not remove it
        # from the hash so that only existing Variations are taken into account
        foreach my $key (keys %SelectedVariations) {
            # @{GetInfoSelectedVariationsByProductID($Product->id, $key) returns the number of elements in the Array
            delete ($SelectedVariations{$key}) if !@{GetInfoSelectedVariationsByProductID($Product->id, $key)};
        }
        $Product->set( { 'SelectedVariations' => [ map { {'SelectedID' => $_} } keys %SelectedVariations ] } );
    }
}

#========================================================================================
# function     UpdateAvailability
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateAvailability();
#----------------------------------------------------------------------------------------
# description  Updates the IsAvailable attribute of all products in the system based on
#               the AvailabilityDate attribute.
#               Call this function once a day for each shop database.
#========================================================================================
sub UpdateAvailability () {
    my $Class    = LoadClassByAlias('Product');

    # define settings
    my %UpdateSettings = (
        Availability      => { SearchParameters => { 'IsAvailable'  => 0,
                                                'AvailabilityDate'  => GetCurrentDateTime(),
                                              },
                               UpdateParameters => { 'IsAvailable'  => 1,
                                              },
        },
        ExpiryDate      => { SearchParameters => { 'IsAvailable'  => 1,
                                                    'ExpiryDate'  => SubtractDuration(GetCurrentDateTime(), 'days', 1),
                                              },
                               UpdateParameters => { 'IsAvailable'  => 0,
                                              },
        },
        AvailabilityEmail => { SearchParameters => { 'IsAvailable'  => 1,
                                                       'IsVisible'  => 1,
                                                         'InStock'  => 1,
                                         'ExistsAvailabilityEmail'  => 1,
                                              },
        },
        Newness           => { SearchParameters => {       'IsNew'  => 1,
                                                     'NewnessDate'  => SubtractDuration(GetCurrentDateTime(), 'days', 1),
                                              },
                               UpdateParameters => {       'IsNew'  => 0,
                                                     'NewnessDate'  => undef,
                                              },
        },
    );
    # search products
    foreach my $Identifier (sort keys %UpdateSettings) {
        my $Setting = $UpdateSettings{$Identifier};
        my %Options = (
            'ObjectID'          => $Class->id,
            'PageSize'          => MAX_PAGE_SIZE,
            'SearchDefinition'  => GetSearchDefinition($Class, 'Availability'),
            'Parameters'        => $Setting->{SearchParameters},
        );
        my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
        # Initialize Search
        $Search->statistic;

        my %ShopIDs;
        my $PageNo = $Search->get('CountOfPages');
        for (my $Page=1; $Page<=$PageNo; $Page++) {
            my $Items = $Search->items($Page);
            foreach my $Item (@$Items) {
                my $Product = eval { LoadObject($Item->id) };
                if( ExistsError() ){
                    my $Error = GetError();
                    next if $Error->code eq 'NoResults_ObjectID'; # the product has been deleted
                    Error( $Error );
                }

                if ($Identifier eq 'AvailabilityEmail') {
                    SendProductAvailibityNotification($Product);
                }
                else {
                    $Product->set( $Setting->{UpdateParameters} );
                    $ShopIDs{$Product->getSite()->id} = 1;
                }
            }
        }

        foreach my $ShopID (keys %ShopIDs) {
            my $Shop = LoadObject($ShopID);
            $Shop->clearPageCache();
        }
    }

    return;
}

#========================================================================================
# function     ScaleImageSizes
# state        public
#----------------------------------------------------------------------------------------
# syntax       ScaleImageSizes($File,$hParams);
# $sample       ScaleImageSizes('C:\test\image.gif',
#                          { "SCALE_WIDTH_m" => 200,
#                            "SCALE_WIDTH_s" => 100,
#                            "SCALE_WIDTH_h" => 150,
#                            "SCALE_HEIGHT_h" => 150,
#                            "SCALE_NAME_m" => 'ImageMedium',
#                            "SCALE_NAME_s" => 'ImageSmall',
#                            "SCALE_NAME_h" => 'ImageHotDeal',
#                           });
#----------------------------------------------------------------------------------------
# description  scale an image by the given scale vars
#----------------------------------------------------------------------------------------
# input        $FileName | file  | string
# input        $hParams | params for processing | with following keys:
#               <ul>
#                   <li>SCALE_WIDTH_* - width of the image - int
#                   <li>SCALE_HEIGHT_* - height of the image - int
#                   <li>SCALE_NAME_* - attributename of the medium image - int
#                   <li>SCALE_QUALITY_* - quality for image  - int
#               </ul>|reference to hash
# return       $rahResult | with following keys: depends on the SCALE_NAME_*
#               <ul>
#                   <li>ImageMedium  - filename of meduim image - string
#                   <li>ImageSmall - filename of small image - string
#                   <li>ImageHotDeal - filename of hotdeal image - string
#               </ul>|reference to array of hashes
#========================================================================================
sub ScaleImageSizes {
    my ($Source, $hParams) = @_;

    my %AddVars;
    return \%AddVars unless ExistsFile($Source);

    # check if supported image format [EPG-10204]
    TestImageFormat( 'Source', $Source );

    # read image only once
    my $OrgImage = Image::Magick->new;
    my $ImageError = $OrgImage->Read($Source);
    Error('IMAGE_MAGICK', {'Function'=>'Read', 'File'=>$Source, 'Message'=>$ImageError}) if $ImageError;

    CorrectImage($Source, $OrgImage);

    my @Sizes = unique (
        map { s/^(?:SCALE_WIDTH|SCALE_HEIGHT|SCALE_QUALITY)_//x; $_ }
        grep { /^(?:SCALE_WIDTH|SCALE_HEIGHT|SCALE_QUALITY)_/x } keys %$hParams
    );

    my ($Path, $FName, $Extension) = fsplit($Source);

    # get image width and height
    my $ImageHeight = $OrgImage->Get('height');
    my $ImageWidth  = $OrgImage->Get('width');
    # possible image magick cant read width/height
    if (not defined $ImageWidth or not defined $ImageHeight) {
        my $ImageInfo = ImageInfo($Source);
        if (defined($ImageInfo->{'ImageHeight'}) && defined($ImageInfo->{'ImageWidth'})) {
            # try to determine image dimensions via exif
            $ImageHeight = $ImageInfo->{'ImageHeight'};
            $ImageWidth  = $ImageInfo->{'ImageWidth'};
        } else {
            Error('UNKNOWN_IMAGE_SIZE', {'File' => $Source, 'Width' => $ImageWidth, 'Height' => $ImageHeight });
        }
    }

    foreach my $Size (@Sizes) {
        my $Target = $FName.'_'.$Size.$Extension;

        # skip existing file?
        next if defined $hParams->{'donotoverwrite'} and ExistsFile($Path.'/'.$Target);

        # create memory copy of the image
        my $WorkImage = $OrgImage->Clone();
        my $Quality = $hParams->{'SCALE_QUALITY_'.$Size};

        # resize for particular size using the image object in $WorkImage
        ResizeImageUsingImageRef(
            $WorkImage,
            $Source,
            $Path.'/'.$Target,
            $ImageWidth,
            $ImageHeight,
            $hParams->{'SCALE_WIDTH_'.$Size},
            $hParams->{'SCALE_HEIGHT_'. $Size},
            $Quality
        );
        $AddVars{$hParams->{'SCALE_NAME_'.$Size}} = $Target if $hParams->{'SCALE_NAME_'.$Size};
        undef $WorkImage;
    }
    undef $OrgImage;

    # correct large image (without l)
    if(not defined $hParams->{'keeplarge'}) {
        MoveFile($Path.'/'.$FName.'_l'.$Extension, $Source) if ElementOf('l', @Sizes);
    }
    $AddVars{'ImageLarge'} = $FName.$Extension;
    return \%AddVars;
}



#========================================================================================
# function     TestUPCEAN
#----------------------------------------------------------------------------------------
# syntax       TestUPCEAN($Name, $EANUPC);
#----------------------------------------------------------------------------------------
# example      TestUPCEAN('EAN', '4004764007974');
# example      TestUPCEAN('UPC', '768268019959');
# example      TestUPCEAN('ISBN', '3-930673-48-7');
# example      TestUPCEAN('ISBN', '3 930673 48 7');
#----------------------------------------------------------------------------------------
# description  Tests if the parameter $EANUPC is a valid EAN-13/EAN-8
#               or a UPC Version A / UPV Version E or ISBN 10 format.
#----------------------------------------------------------------------------------------
# input        $Name | parameter name | string
# input        $EANUPC | parameter for test | string
#========================================================================================
sub TestUPCEAN {
    my ( $Name, $EANUPC) = @_;
    Error('PARAMETER_MISSING' , {'Parameter' => 'Name', 'ParameterName' => '(undef)'}) unless defined $Name;
    Error('PARAMETER_MISSING' , {'Parameter' => 'EANUPC', 'ParameterName' =>$Name}) if not defined $EANUPC;

    #remove spaces and - in the code
    $EANUPC =~ s/-//ig;
    $EANUPC =~ s/ //ig;

    my @CodeDigits = split(//,$EANUPC);
    my $codelength = scalar(@CodeDigits);

    my $rv;
    # test if we have 13 digits (EAN-13)
    $rv = CheckDigits('ean') if $codelength == DIGITS_EAN_13;
    # test if we have 12 digits (UPC Version A)
    $rv = CheckDigits('upc') if $codelength == DIGITS_UPC_VA;
    # test if we have 10 digits (ISBN 10)
    $rv = CheckDigits('isbn') if $codelength == DIGITS_ISBN_10;
    # test if we have 8 digits (EAN-8 or UPC Version E)
    $rv = CheckDigits('ean') if $codelength == DIGITS_EAN_8_o_UPC_VE;

    if (not defined $rv or not $rv->is_valid($EANUPC)) {
        Error('FORMAT_NOT_UPCEAN' , {'Parameter' => $Name, 'Value' => $EANUPC });
    }

    return;

}

#========================================================================================
# function     DeleteProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteProductImage($Product, $FileName);
#----------------------------------------------------------------------------------------
# description  Deletes an image file and unassigns it from the product, its super
#               product and all sub products.
#----------------------------------------------------------------------------------------
# input        $Product | product object (normal, super or sub product) | object
# input        $FileName | image name, relative to the product image directory | string
# input        $hOptions | with following keys:
#               <ul>
#                   <li>DontDeleteSubImages -
#                           changes default behavior, sub images are not deleted - boolean
#               </ul> | hash.ref
#========================================================================================
sub DeleteProductImage {
    my ($Product, $FileName, $hOptions) = @_;
    TestObject( 'Product', $Product );
    TestString( 'FileName', $FileName );
    TestHash( 'hOptions', $hOptions) if defined $hOptions;
    $hOptions //= {};
    my $DeleteSubImages = defined $hOptions->{DontDeleteSubImages} && $hOptions->{DontDeleteSubImages} ? 0 : 1;
    my $ShopPath = $Product->get('Site')->get('PublicPath');
    my $ProductPath = $Product->get('FileAlias');

    # is URL saved and contains //hostname
    my @DeletedFiles;
    if ($FileName =~ /\.url$/) {
        my ($hFileInfo) = grep { $_->{'URLFileName'} eq $FileName } @{$Product->get('PublicFiles')};
        # delete the file if it actually exists
        if (defined $hFileInfo) {
            DeleteFile( $hFileInfo->{'Name'} );
            push @DeletedFiles, $hFileInfo->{'FileName'};
        }
    } else {
        my @PublicFiles = map { $_->{'FileName'}} @{$Product->get('PublicFiles')};
        my (undef, $FName, $Extension) = fsplit($FileName);
        # delete file and views if it actually exists
        my $PublicPath = $Product->get('PublicPath');
        DeleteFile( $PublicPath.'/'.$FileName);
        push @DeletedFiles, $FileName;
        if ($DeleteSubImages) {
            foreach my $File (grep { ElementOf($_, @PublicFiles) } map { $FName.'_'.$_.$Extension } keys %{PRODUCT_IMAGES_SIZES()}) {
                push @DeletedFiles, $File;
                DeleteFile( $PublicPath.'/'.$File);
            }
        }

    }

    # unassign the image form this product, the super product and all sub products
    my $SuperProduct = $Product->get('SuperProduct') ? $Product->get('SuperProduct') : $Product;
    _UnassignProductImage( $SuperProduct, $FileName, \@DeletedFiles );
    _UnassignProductImage( $_, $FileName, \@DeletedFiles) foreach @{$SuperProduct->get('SubProducts')};

    # make sure nothing is left on file system
    # thumbnail is generated from "_s" image
    my (undef, $FName, $Extension) = fsplit($FileName);
    if ($FName =~ m/_s$/){
        $FName =~ s/_s$/_xs/;
        DeleteFile( $Product->get('PublicPath') . '/' . $FName . $Extension );
    }
    CleanPath($ShopPath, $ProductPath);

    return;
}

#========================================================================================
# function     _UnassignProductImage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UnassignProductImage($Product, $FileName, $aFileNames);
#----------------------------------------------------------------------------------------
# description  Unassigns the image from a single product and removes it from the
#               slideshow.
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $aFileNames | image name, relative to the product image directory | array.string
# input        $FileName | file name | string
#========================================================================================
sub _UnassignProductImage {
    my ($Product, $FileName, $aFileNames) = @_;

    # ImageLarge, ImageMediumLarge, ...
    my @AttributeNames = values %{PRODUCT_IMAGE_SIZE_NAMES()};

    # set Attribute to undef if file was deleted
    foreach my $ImageType (@AttributeNames) {
        my $Value = $Product->get($ImageType);
        $Product->set( { $ImageType => undef } ) if defined $Value && ElementOf($Value, @$aFileNames);
    }

    my $aCurrentSlideshowImages = $Product->get('ImagesSlideShow');
    if($aCurrentSlideshowImages) {
        # remove the file from the slideshow
        my @ImagesSlideShow =
            grep { $_ ne $FileName }
            map { $_->{'ImageName'} }
            @{$aCurrentSlideshowImages};

        # remove subimages from the slideshow
        foreach my $DeletedFileName (@{$aFileNames}) {
            @ImagesSlideShow =
                grep { $_ ne $DeletedFileName }
                @ImagesSlideShow;
        }
        my $ImagesSlideShowString = join ';', @ImagesSlideShow;

        # rebuild the Show...
        $Product->set( { 'ImagesSlideShowString' => $ImagesSlideShowString } );
    }
    return;
}

#========================================================================================
# function     RenameProductImageFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResult = RenameProductImageFile(
#                       $Product,
#                       $OldFileName,
#                       $NewFileName,
#                       $FileExt
#                    );
#----------------------------------------------------------------------------------------
# description  Renames the given old file associated to a product to the given new name.
#               Also renames all derivatives of the file, i.e. when the file is an image
#               rename the smaller images for hotdeal, thumbnail, etc.
#               If the image file was the current product image, update the product's
#               object attributes with the new filename as well.
#               If another file with the given new name already exists, an error is thrown.
#----------------------------------------------------------------------------------------
# input        $Product | product object instance | object
# input        $OldFileName | file name without extension of the current file | string
# input        $NewFileName | new file name without extension for the file | string
# input        $FileExt | file extension | string
# return       $hUpdateProductData | hash containing the changed product attributes
#               | reference to hash
#========================================================================================
sub RenameProductImageFile {
    my ($Product, $OldFileName, $NewFileName, $FileExt) = @_;

    # extension used from old file, won't change
    my $OldFullFileName = $OldFileName.$FileExt;
    my $NewFullFileName = $NewFileName.$FileExt;

    my $ProductFilesPath = $Product->get('PublicPath'); #what about: $Product->get('PublicFiles'); -> how do I update that attribute? or is it a generated one?
    my $hUpdateProductData;

    # check if other file is already there, if so add error
    if(ExistsFile($ProductFilesPath.'/'.$NewFullFileName)) {
        Error('ANOTHERFILEALREADYEXISTS', {
            'File' =>  $ProductFilesPath.'/'.$NewFullFileName,
        });
        return;
    }

    # move file itself (i.e. large image)
    MoveFile($ProductFilesPath.'/'.$OldFullFileName, $ProductFilesPath.'/'.$NewFullFileName);

    if (ElementOf(substr(lc($FileExt),1), IMAGE_FILE_TYPES())) {
        my %ProductImageSizes = %{PRODUCT_IMAGES_SIZES()};
        delete $ProductImageSizes{'l'}; # large is auto-renamed without the l
        # move image's derivatives
        foreach my $SizeType (keys %ProductImageSizes) {
            # derivatives may exists without the others when using the upload for the particular views
            next if !ExistsFile($ProductFilesPath.'/'.$OldFileName.'_'.$SizeType.$FileExt);
            MoveFile(
                $ProductFilesPath.'/'.$OldFileName.'_'.$SizeType.$FileExt,
                $ProductFilesPath.'/'.$NewFileName.'_'.$SizeType.$FileExt
            );
        }

        # the product image is also in the slideshow list, maybe this one has changed name? then update the product image!
        # image large and derivatives
        my %ReverseSizeNames = reverse %{PRODUCT_IMAGE_SIZE_NAMES()};

        my $hProductImages = $Product->get([keys %ReverseSizeNames]);
        my (undef, $LargeImageFileName, $LargeImageFileExt);
        (undef, $LargeImageFileName, $LargeImageFileExt) = fsplit($hProductImages->{'ImageLarge'}) if defined $hProductImages->{'ImageLarge'};

        foreach my $Attribute (keys %$hProductImages) {         #$Attribute is IMageLarge, ImageMedium, etc.
            next if !$hProductImages->{$Attribute};

            my $Value = $hProductImages->{$Attribute};          # current value on product

            # pure name or name with _s, _h etc.
            if($Value eq $OldFullFileName or $Value eq $OldFileName.'_'.$ReverseSizeNames{$Attribute}.$FileExt) {
                # update product attribute

                # same name as large one?
                my $UpdateName;

                if($LargeImageFileName and ($Value eq $LargeImageFileName.'_'.$ReverseSizeNames{$Attribute}.$LargeImageFileExt)) {
                    if($Attribute eq 'ImageLarge') {
                        # extra-Wurst for large
                        $UpdateName = $NewFileName.$FileExt;
                    } else {
                        $UpdateName = $NewFileName.'_'.$ReverseSizeNames{$Attribute}.$FileExt;
                    }
                } else {
                    $UpdateName = $NewFileName.$FileExt;
                }

                $hUpdateProductData->{$Attribute} = $UpdateName;
            }
        }
    }

    return $hUpdateProductData;
}

#========================================================================================
# function     RenameSubProductImageFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResult = RenameSubProductImageFile(
#                       $SubProduct,
#                       $OldFileName,
#                       $NewFileName,
#                       $FileExt
#                    );
#----------------------------------------------------------------------------------------
# description  Renames the given old file associated to a product to the given new name.
#               Also renames all derivatives of the file, i.e. when the file is an image
#               rename the smaller images for hotdeal, thumbnail, etc.
#               If the image file was the current product image, update the product's
#               object attributes with the new filename as well.
#               If another file with the given new name already exists, an error is thrown.
#----------------------------------------------------------------------------------------
# input        $SubProduct | subproduct object instance | object
# input        $OldFileName | file name without extension of the current file | string
# input        $NewFileName | new file name without extension for the file | string
# input        $FileExt | file extension | string
# return       $hUpdateSubProductData | hash containing the changed product attributes
#               | reference to hash
#========================================================================================
sub RenameSubProductImageFile {
    my ($SubProduct, $OldFileName, $NewFileName, $FileExt) = @_;

    # extension used from old file, won't change
    my $OldFullFileName = $OldFileName.$FileExt;
    my $NewFullFileName = $NewFileName.$FileExt;

    my %ReverseSizeNames = reverse %{PRODUCT_IMAGE_SIZE_NAMES()};
    my $hUpdateSubProductData;
    my $hSubProductImages = $SubProduct->get([keys %ReverseSizeNames]);
    my ($LargeImageFileName, $LargeImageFileExt);
    (undef, $LargeImageFileName, $LargeImageFileExt) = fsplit($hSubProductImages->{'ImageLarge'}) if defined $hSubProductImages->{'ImageLarge'};
    foreach my $ImageAttribute (keys %$hSubProductImages) {
        next if !$hSubProductImages->{$ImageAttribute};

        my $Value = $hSubProductImages->{$ImageAttribute};          # current value on product
        my $SubProductUpdateName;
        # pure name or name with _s, _h etc.
        if($Value eq $OldFullFileName or $Value eq $OldFileName.'_'.$ReverseSizeNames{$ImageAttribute}.$FileExt) {
            if($LargeImageFileName and ($Value eq $LargeImageFileName.'_'.$ReverseSizeNames{$ImageAttribute}.$LargeImageFileExt)) {
                if($ImageAttribute eq 'ImageLarge') {
                    # extra-Wurst for large
                    $SubProductUpdateName = $NewFullFileName;
                } else {
                    $SubProductUpdateName = $NewFileName.'_'.$ReverseSizeNames{$ImageAttribute}.$FileExt;
                }
            } else {
                $SubProductUpdateName = $NewFullFileName;
            }
            $hUpdateSubProductData->{$ImageAttribute} = $SubProductUpdateName;
        }
    }
    return $hUpdateSubProductData;
}

#========================================================================================
# function     SmartSearchProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProducts = SmartSearchProducts($Shop, $SearchString);
#----------------------------------------------------------------------------------------
# description  Returns the product objects for a search string (alias)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object instance | object
# input        $SearchString | an alias search string | string
# return       $aProducts | reference to array containing the found products
#               | reference to array of objects
#========================================================================================
sub SmartSearchProducts {
    my ($Shop, $SearchString) = @_;

    my $ProductFolder = $Shop->folder('Products');
    my $ProductFolderID = $ProductFolder->id;

    my $dbi = GetCurrentDBHandle();
    my @ProductIDs = @{DAL_GetSmartSearchProductIDs($ProductFolderID, $dbi->replaceLikeWildcards($SearchString).'%')};

    return [ map { LoadObject($_) } @ProductIDs ];
}

#========================================================================================
# function     GenerateCustomizedProductPDF
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPdfContent = GenerateCustomizedProductPDF($ProductLineItem);
#----------------------------------------------------------------------------------------
# description  Converts customized product details to pdf document
#----------------------------------------------------------------------------------------
# input        $ProductLineItem | Product Lineitem | object
# input        $MerchantLanguageID | LanguageID | integer
#----------------------------------------------------------------------------------------
# return       $hPdfContent | <ul>
#                               <li>pdf       - pdf document                  - binary
#                               <li>filename  - name of pdf file              - string
#                              </ul> | ref.hash
#========================================================================================
sub GenerateCustomizedProductPDF($$){
    my ($ProductLineItem, $MerchantLanguageID) = @_;

    TestObject('ProductLineItem', $ProductLineItem);

    my $Container = $ProductLineItem->get('LineItemContainer');

    my $PageType = LoadPageTypeByAlias('MBO-ExportCustomizedAsPDF');
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load($PageType,$ProductLineItem);

    my $LanguageCode = GetCodeByLanguageID($MerchantLanguageID);

    my $hVars = $ProductLineItem->tleHash;
    $hVars->{'INPUT'} = { 'Locale' => $Container->get('LocaleID') };
    $hVars->{'PDF'} = 1;
    $hVars->{'OSNAME'} = $Config{'osname'};

    my $HtmlContent;
    SetLogLevel('WARN', sub {
        $HtmlContent = ProcessPage($ProductLineItem, $ObjectPageType, $LanguageCode, $hVars);
    },'DE_EPAGES::Presentation::API::Template::INCLUDE');

    #build filename 1004_cg6563456.pdf
    my $OrderAlias = $Container->parent->alias;

    my $Product = $ProductLineItem->get('Product');
    my $ProductAlias = defined $Product ? $Product->alias : $ProductLineItem->get('Name');  ;

    my $PdfFileName = ConvertPathChars(join('_',$OrderAlias,$ProductAlias)) . '.pdf';

    return {
        'pdf' => DE_EPAGES::EPagesJ::API::PDF::PdfClient->create($ProductLineItem->getSite)->transform($HtmlContent),
        'filename' => $PdfFileName
    };
}

#========================================================================================
# function     SendCustomizationDocumentAsPDFMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendCustomizationDocumentAsPDFMail($LineItemProduct,$Servlet);
#----------------------------------------------------------------------------------------
# description  Send an customization document as pdf file mail to the given email .
#----------------------------------------------------------------------------------------
# input        $LineItemProduct | product lineitem | object
# input        $Servlet | the servlet | object
#========================================================================================
sub SendCustomizationDocumentAsPDFMail($$) {
    my ($LineItemProduct,$Servlet) = @_;

    my $Order = $LineItemProduct->parent->parent;
    my $Shop = $Order->getSite();
    my $From  = $Shop->get('MailFrom');

    unless( defined $From && $From =~ /\S/ ) {
        Warning('MISSING_FROM', {'ShopAlias' => $Shop->alias});
        return;
    }

    my $EmailTo  = $Servlet->params('To');
    my $EmailCc  = $Servlet->params('Cc');
    my $EmailBcc = $Servlet->params('Bcc');
    my $Subject  = $Servlet->params('Subject');
    my $EmailMsg = $Servlet->params('Msg');

    my $hPdfContent = GenerateCustomizedProductPDF($LineItemProduct, $Servlet->user->get('LanguageID'));


    # get all uploaded images and add as attachment to the mail
    my $ahAttachments = [];
    push @$ahAttachments, { 'filename' => $hPdfContent->{'filename'},
                            'data'     => $hPdfContent->{'pdf'},
                            'encoding' => 'base64'
                           };

    my $ahLineItemCustomizedFiles = $LineItemProduct->get('CustomizedFiles');
    foreach my $hCustomizedFile (@$ahLineItemCustomizedFiles) {
        my $FileName = $hCustomizedFile->get('PublicPath').'/'.$hCustomizedFile->get('FileName');
        push @$ahAttachments, { 'filename' => $FileName,
                              } if ExistsFile($FileName) ;
    }

    # create and send mail
    my %MailParams = (
        'from'        => $From,
        'to'          => $EmailTo,
        'cc'          => $EmailCc,
        'bcc'         => $EmailBcc,
        'subject'     => $Subject,
        'content'     => $EmailMsg,
        'attachments' => $ahAttachments,
    );

    SendTextMail(\%MailParams);
    return;
}

#========================================================================================
# function     GetVisibleProductIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProducts = GetVisibleProductIDs($Shop);
#----------------------------------------------------------------------------------------
# description  Returns all visible product IDs.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# return       $aProductIDs | product ids | ref.array.int
#========================================================================================
sub GetVisibleProductIDs {
    my ($Shop) = @_;
    my $ProductClass = LoadClassByAlias('Product');
    my $ProductFolder = $Shop->child('Products');
    return $ProductClass->findObjects('ByParentIDIsVisible', [$ProductFolder->id, 1]);
}

#========================================================================================
# function     GetVisibleSubProductIDs
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSubProductIDs = GetVisibleSubProductIDs($Product);
#----------------------------------------------------------------------------------------
# description  Returns all visible sub product IDs for a given (master) product.
#----------------------------------------------------------------------------------------
# input        $Product | master product | object
# return       $aSubProductIDs | sub product ids | ref.array.int
#========================================================================================
sub GetVisibleSubProductIDs {
    my ($Product) = @_;
    my $ProductClass = LoadClassByAlias('Product');
    my $SubProductFolder = $Product->child('SubProducts');
    return $ProductClass->findObjects('ByParentIDIsVisible', [$SubProductFolder->id, 1]);
}

#========================================================================================
# function     StoreProductImages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hResponse = StoreProductImages( $Product, $ImageName, $ImageBinary, $aSizes );
#----------------------------------------------------------------------------------------
# description  Stores a given image and generate smaller version of this image based on
#               parameter $aSize
#----------------------------------------------------------------------------------------
# input    $Product   | Product which will use the images | DE_EPAGES::Product::API::Object::Product
# input    $ImageName | name of image incl. extension | String
# input    $ImageBinary | Image file content | Blob
# input    $aSizes     | Image sizes you wan't to generate additionally to original file.
#                        For possible values see DE_EPAGES::Product::API::Constants PRODUCT_IMAGE_SIZE_NAMES
#                         and PRODUCT_IMAGES_SIZES
#                       | arra.ref.string
#========================================================================================
sub StoreProductImages($$$$) {
    my ($Product, $ImageName, $ImageBinary, $aSizes) = @_;

    TestObject('Product', $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestArray('aSizes', $aSizes);
    TestString('ImagePath', $ImageName,5);    # one letter + dot + file extension
    TestImageFormat('ImageName', $ImageName); # bassically check m/(jpg|bmp|gif|png)$/

    my $Path = $Product->get('PublicPath');
    CreateDirectory( $Path, {'recursive'=>1} );

    my $OrgImage = Image::Magick->new;
    my $ImageError = $OrgImage->BlobToImage($ImageBinary);
    Error('IMAGE_MAGICK', {'Function'=>'Read', 'File'=>$ImageName, 'Message'=>$ImageError})
        if $ImageError;

    my $hImageDimension = GetProductImageSizes($Product->getSite());

    my (undef, $FileName, $Extension) = fsplit($ImageName);

    # Genrate smaller versions of the uploaded image
    foreach my $Size (@$aSizes) {
        # Create new image name with postfix eg. picture_ml.png
        my $Target = $FileName .'_' . $Size . $Extension;
        my ($TargetWidth, $TargetHeight) = @{$hImageDimension->{$Size}};

        # Create memory copy of the image
        my $WorkImage = $OrgImage->Clone();

        my ($SourceHeight, $SourceWidth, $SourceQuality) = $WorkImage->Get('height','width', 'quality' );
        my $TargetQuality = PRODUCT_IMAGES_JPEG_QUALITIES->{$Size};

        # Throws error, if problems occur
        ResizeImageUsingImageRef(
            $WorkImage,
            $ImageName,
            $Path.'/'.$Target,
            $SourceWidth,
            $SourceHeight,
            $TargetWidth,
            $TargetHeight,
            $TargetQuality,
            $SourceQuality
        );

        # e.g. ImageMediumSmall
        my $AttributeName = PRODUCT_IMAGE_SIZE_NAMES->{$Size};

        # If image resizing works, delete old image and set attribute undef
        DeleteProductImageByAttribute($Product, [$AttributeName])
            unless ($Product->get($AttributeName)// '') eq $Target;
        $Product->set({ $AttributeName => $Target});
    }

    return;
}

#========================================================================================
# function     DeleteProductImageByAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteProductImageByAttribute( $Product, $aAttributes );
#----------------------------------------------------------------------------------------
# description  Delete images assigned to given attribute. This method calls <function DeleteProductImage>
#               to remove all assignments from slide show, subproducts and subproduct slide show.
#----------------------------------------------------------------------------------------
# input        $Product | product which reference the images | DE_EPAGES::Product::API::Object::Product
# input        $aAttributes | attributes which contain image pathes | array.ref.
# input        $hOptions | with following keys:
#               <ul>
#                   <li>DontDeleteSubImages -
#                           changes default behavior, sub images are not deleted - boolean
#               </ul> | hash.ref
#========================================================================================
sub DeleteProductImageByAttribute($$;$) {
    my ($Product, $aAttributes, $hOptions) = @_;
    TestObject('Product', $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestArray('aAttributes', $aAttributes);
    $hOptions //= {};


    foreach my $Attribute (@$aAttributes) {
        my $Value = $Product->get($Attribute);
        next unless isValid($Value);
        DeleteProductImage($Product, $Value, $hOptions);
    }

    return;
}

#========================================================================================
# function     GetScaledImages
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetScaledImages($FilePath, $Shop);
#----------------------------------------------------------------------------------------
# description  Resize an uploaded image to the sizes l,m,ms,ml,s,h,xs
#               using the method ScaleImageSizes and returns the related hash.ref
#----------------------------------------------------------------------------------------
# input        $FilePath | file path of the original image | string
# input        $Shop | shop | object
# input        $hParams (optional) | hash for overwriting original scale params  | hash.ref
# return       $hScaledImages | hash with all resized images | hash.ref
#========================================================================================
sub GetScaledImages {
    my ($FilePath, $Shop, $hParams) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestString('FilePath', $FilePath);


    my $hScaleParams = {
        'SCALE_WIDTH_l'   => $Shop->get('ProductImageLargeWidth'),
        'SCALE_HEIGHT_l'  => $Shop->get('ProductImageLargeHeight'),
        'SCALE_WIDTH_m'   => PRODUCT_IMAGES_SIZES->{'m'}->[0],
        'SCALE_HEIGHT_m'  => PRODUCT_IMAGES_SIZES->{'m'}->[1],
        'SCALE_WIDTH_ms'  => PRODUCT_IMAGES_SIZES->{'ms'}->[0],
        'SCALE_HEIGHT_ms' => PRODUCT_IMAGES_SIZES->{'ms'}->[1],
        'SCALE_WIDTH_ml'  => PRODUCT_IMAGES_SIZES->{'ml'}->[0],
        'SCALE_HEIGHT_ml' => PRODUCT_IMAGES_SIZES->{'ml'}->[1],
        'SCALE_WIDTH_s'   => PRODUCT_IMAGES_SIZES->{'s'}->[0],
        'SCALE_HEIGHT_s'  => PRODUCT_IMAGES_SIZES->{'s'}->[1],
        'SCALE_WIDTH_h'   => PRODUCT_IMAGES_SIZES->{'h'}->[0],
        'SCALE_HEIGHT_h'  => PRODUCT_IMAGES_SIZES->{'h'}->[1],
        'SCALE_WIDTH_xs'  => PRODUCT_IMAGES_SIZES->{'xs'}->[0],
        'SCALE_HEIGHT_xs' => PRODUCT_IMAGES_SIZES->{'xs'}->[1],
        'SCALE_NAME_l'    => PRODUCT_IMAGE_SIZE_NAMES->{'l'},
        'SCALE_NAME_m'    => PRODUCT_IMAGE_SIZE_NAMES->{'m'},
        'SCALE_NAME_ms'   => PRODUCT_IMAGE_SIZE_NAMES->{'ms'},
        'SCALE_NAME_ml'   => PRODUCT_IMAGE_SIZE_NAMES->{'ml'},
        'SCALE_NAME_s'    => PRODUCT_IMAGE_SIZE_NAMES->{'s'},
        'SCALE_NAME_h'    => PRODUCT_IMAGE_SIZE_NAMES->{'h'},
        'SCALE_NAME_xs'   => PRODUCT_IMAGE_SIZE_NAMES->{'xs'}
    };

    if ( defined $hParams and (ref $hParams eq 'HASH') ){
        $hScaleParams->{$_} = $hParams->{$_} foreach keys %$hParams;
    }

    return ScaleImageSizes($FilePath, $hScaleParams);
}

#========================================================================================
# function     StartUnityProgressMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       StartUnityProgressMonitor($Servlet, $Product, $aListItems);
#----------------------------------------------------------------------------------------
# description  Starts a progress Monitor in the Unity design.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# input        $Product | product | object
# input        $aListItems | list of items to process | array.ref
#========================================================================================
sub StartUnityProgressMonitor {
    my ($Servlet, $Product, $aListItems) = @_;

    my $Monitor = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
        'PageTypeName' => 'UnityCreateSubProductsMonitor',
        'Object'       => $Product,
    );
    $Servlet->{'Monitor'} = $Monitor;
    $Monitor->set({'Total'=> scalar @$aListItems});

    return;
}

#========================================================================================
# function     UpdateUnityProgressMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateUnityProgressMonitor($Servlet);
#----------------------------------------------------------------------------------------
# description  Updates a progress monitor by increasing the counter.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub UpdateUnityProgressMonitor {
    my ($Servlet) = @_;

    my $Monitor = $Servlet->{'Monitor'};

    if (defined $Monitor) {
        $Monitor->incCurrent;
        $Monitor->write;
    }

    return;
}

#========================================================================================
# function     GetUnityPrettyUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetUnityPrettyUrl( $Product, $ShopLanguageID, $LanguageID, $UseOnStoredPrettyUrl );
#----------------------------------------------------------------------------------------
# description  it gets pretty unity URL from database or from redirect request to unity service
#----------------------------------------------------------------------------------------
# input        $Product | product to whom we want to acquire the pretty URL | object
# input        $ShopLanguageID | shop language id number | integer
# input        $LanguageID | language id number for Url localization | integer
# input        $UseStoredPrettyUrl | rely or not on existing pretty url DB value | boolean
# return       $ProductPrettyUnityUrl | object's pretty unity URL string | string
#========================================================================================
sub GetUnityPrettyUrl {
    my ($Product, $ShopLanguageID, $LanguageID, $UseStoredPrettyUrl) = @_;

    my $Shop = $Product->getSite();
    Error('NO_UNITY_SHOP', {
        'ShopPath' => $Shop->pathString(),
    }) unless ($Shop->isUnity());

    my $ProductPrettyUnityUrl = $Product->get('ProductPrettyUnityUrl', $LanguageID);

    # optionally, return already existing pretty url (for the standard shop language) from database
    if ($UseStoredPrettyUrl && isValid($ProductPrettyUnityUrl)) {
        return $ProductPrettyUnityUrl;
    }

    my $WebUrlUnitySSL = $Product->get('WebUrlUnitySSL');
    $WebUrlUnitySSL = AddLanguageToURL($WebUrlUnitySSL, $LanguageID, $ShopLanguageID);
    $ProductPrettyUnityUrl //= $WebUrlUnitySSL;

    return _getAndSavePrettyUnityUrlViaRedirectRequest($Product, $LanguageID, $ProductPrettyUnityUrl, $WebUrlUnitySSL);
}

#========================================================================================
# function     _getAndSavePrettyUnityUrlViaRedirectRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getAndSavePrettyUnityUrlViaRedirectRequest( $Product, $LanguageID, $ProductPrettyUnityUrl, $WebUrlUnitySSL );
#----------------------------------------------------------------------------------------
# description  Get Pretty Unity URL from epages-ui web service through redirection
#----------------------------------------------------------------------------------------
# input        $Product | product object to process | object
# input        $LanguageID | product id language number | integer
# input        $ProductPrettyUnityUrl | initial Pretty Unity URL | string
# input        $WebUrlUnitySSL | original Unity URL | string
# return       $WebUrlPrettyUnitySSL | pretty unity URL gotten | string
#========================================================================================
sub _getAndSavePrettyUnityUrlViaRedirectRequest {
    my ($Product, $LanguageID, $ProductPrettyUnityUrl, $WebUrlUnitySSL) = @_;

    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->timeout(EPAGES_UI_WEBSERVICE_REQUEST_TIMEOUT);
    my $Response = $UserAgent->head($WebUrlUnitySSL);
    if ($Response->is_success()) {
        my $ResponseURL = $Response->base->as_string();
        if (isValid($ResponseURL)) {
            if ($ResponseURL ne $ProductPrettyUnityUrl) {
                $Product->set( { 'ProductPrettyUnityUrl' => $ResponseURL }, $LanguageID );
            }
            return $ResponseURL;
        }
    }

    return $ProductPrettyUnityUrl;
}

#========================================================================================
# function     GetProductPrettyUnityUrlsRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetProductPrettyUnityUrlsRequest( $aProductIDs, $LanguageID );
#----------------------------------------------------------------------------------------
# description  given an array of ProductIDs, it generates proper request to get pretty
#               unity URLs from epages-ui 'products-get-urls' end-point
#----------------------------------------------------------------------------------------
# input        $aProductIDs | list of product ids | ref.array
# input        $LanguageID | language to be used for the names | integer
# return       $PrettyUrlsRequest | array request data to get pretty unity URLs | ref.array
#========================================================================================
sub GetProductPrettyUnityUrlsRequest {
    my ($aProductIDs, $LanguageID) = @_;

    my @PrettyUrlsRequest = ();
    foreach my $ProductID (@$aProductIDs) {
        my $Product = LoadObject($ProductID);
        my $hProductRequest = { 'productId' => $Product->guid(), 'name' => $Product->get('NameOrAlias', $LanguageID) };

        if ($Product->isSubProduct()) {
            my $SuperProduct = $Product->superProduct();
            my $hInfoSuperProduct = $SuperProduct->get(['NameOrAlias', 'GUID'], $LanguageID);

            $hProductRequest->{'variationMaster'} = {
                'name'      => $hInfoSuperProduct->{'NameOrAlias'},
                'productId' => $hInfoSuperProduct->{'GUID'},
                'slug'      => ''
            };
            $hProductRequest->{'productVariationType'} = 'variation';
            $hProductRequest->{'productVariationSelection'} = GetVariationAttributes($Product, $LanguageID);
        }
        push(@PrettyUrlsRequest, $hProductRequest);
    }

    return \@PrettyUrlsRequest
}

#========================================================================================
# function     GetVariationAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductVariations = GetVariationAttributes( $Product, $LanguageID, $aVariationTypeValues );
#----------------------------------------------------------------------------------------
# description  get an array of variations
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $LanguageID | language id number | integer
# input        $aVariationTypeValues | array reference with variation type values | refarray
# return       $aProductVariations | array reference with product variations | refarray
#========================================================================================
sub GetVariationAttributes{
    my ($Product, $LanguageID, $aVariationTypeValues) = @_;

    $aVariationTypeValues //= $Product->getWithParent('VariationTypeValues', $LanguageID);

    if (scalar @$aVariationTypeValues == 0){
        return; # this is not a master product
    }
    my %VariationAttributeNamesAlias = map { $_->{'Attribute'}->alias() => $_->{'Attribute'}->get('NameOrAlias', $LanguageID)} @{$aVariationTypeValues};
    my @VariationAttributeNames = keys %VariationAttributeNamesAlias;
    my $hVariationAttributeValues = $Product->getWithParent(\@VariationAttributeNames);

    return [
            map {
                    {
                        'name' => $_,
                        'displayName' => $VariationAttributeNamesAlias{$_},
                        'value' => $hVariationAttributeValues->{$_}->alias(),
                        'displayValue' => $hVariationAttributeValues->{$_}->get('ValueOrAlias', $LanguageID),
                    }
            } grep { defined $hVariationAttributeValues->{$_} } @VariationAttributeNames
        ];
}

#========================================================================================
# function     SaveProductPrettyUrlsResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveProductPrettyUrlsResponse( $aProductPrettyUrlsResponse, $PortalLanguageID );
#----------------------------------------------------------------------------------------
# description  given the response array from epages-ui 'products-get-urls' end-point,
#               it saves product pretty urls into database
#----------------------------------------------------------------------------------------
# input        $aProductPrettyUrls | list of product ids | ref.array
# input        $PortalLanguageID | portal language to be used for the names | integer
#========================================================================================
sub SaveProductPrettyUrlsResponse {
    my ($aProductPrettyUrlsResponse, $PortalLanguageID) = @_;

    foreach my $ProductUrlResponse (@$aProductPrettyUrlsResponse) {
        my $Product = LoadObjectByGUID($ProductUrlResponse->{'productId'});
        $Product->set( { 'ProductPrettyUnityUrl' => $ProductUrlResponse->{'url'} }, $PortalLanguageID );
    }
}

#========================================================================================
# function     CropProductImages
# state        public
#----------------------------------------------------------------------------------------
# syntax       CropProductImages( $Product, $ahCropDataFromJSON, $hParams );
#----------------------------------------------------------------------------------------
# description  do croppping and rotation on the product image by the given $CropDataFromJSON
#               and returns the changed filenames
#----------------------------------------------------------------------------------------
# input        $Product | product object | object
# input        $ahCropDataFromJSON | crop data extracted from JSON | ref.array.hash
# input        $hParams | parameter hash | ref.hash
# return       $hChangedFiles | hash reference with changed filenames | ref.hash
#========================================================================================
sub CropProductImages {
    my ($Product, $ahCropDataFromJSON, $hParams) = @_;

    my $IsVariation = ((defined $hParams) and (exists $hParams->{'IsVariation'}) and ($hParams->{'IsVariation'})) ? 1 : 0 ;

    # Test if we have an array because we will loop over a list of cropData.
    # Build a hash oh arays of hashes with the name as key.
    # We have to loop over the cropdata array by filenames.
    my $ahCropData   = ref $ahCropDataFromJSON eq 'ARRAY' ? $ahCropDataFromJSON : [$ahCropDataFromJSON];
    my $hahCropData  = {};
    foreach my $hCropData (@$ahCropData){
        push @{$hahCropData->{$hCropData->{'name'}}} , $hCropData;
    }

    # Test if we have to crop variations.
    my $UseNewFileName = $IsVariation ? 1 : 0;
    my $hChangedFiles = {};

    # Loop over filenames which can have a lot of crop data.
    foreach my $Key (keys %$hahCropData) {
        my $ahCropDataByFile = $hahCropData->{$Key};
        my $CropDataCount    = 0;
        my $OriginalFileName;
        foreach my $hCropData (@$ahCropDataByFile){
            # Do image manipulation and get back new filename (same as sourcefilename for single products)
            # Use a new filename for the image only on the 1st crop action on the image
            my $hImageFileNames = _ManipulateImage($Product, $hCropData, $CropDataCount ? 0 : $UseNewFileName );

            if ( $IsVariation ) {
                # Change the name in crop data because we will not generate a new filename if we have
                # more than one crop action on the same image.
                foreach my $hCropData (@$ahCropDataByFile){
                    $hCropData->{'name'} = $hImageFileNames->{'NewFileName'};
                }
            }
            # we need to save the original filename only for the 1st crop action
            $OriginalFileName = $hImageFileNames->{'OriginalFileName'} unless $CropDataCount;
            # prepare hash for returning changed images
            $hChangedFiles->{$OriginalFileName} = $hImageFileNames->{'NewFileName'};
            $CropDataCount++;
        }
    }

    return $hChangedFiles;

}

sub _ManipulateImage {
    my ($Product, $hCropData, $UseNewFileName) = @_;

    my $INFileName = $hCropData->{'name'};
    # After uploading and cropping in the same step (without page reload) or for variation products
    # we have only the original filename in the cropdata.
    # We add here the _m extension to the filename.
    if ( ($hCropData->{'new'} == JSON::true) or ($hCropData->{'variation'} == JSON::true) ) {
        my (undef, $Name, $Extension) = fsplit($INFileName);
        $INFileName = $Name . '_m' . $Extension;
    }

    # get the filename from the _m size image (the image manipulation data are related the _m size images)
    my $hM_ImageInfo = GetImageInfo($Product->get('PublicPath') .'/'. $INFileName);
    my $M_Height     = $hM_ImageInfo->{'Height'};
    my $M_Width      = $hM_ImageInfo->{'Width'};

    # For non variation products we edit a _m size image in the cropper tool.
    # That means we get all crop dimenstions related to the _m size image.
    # For variation product image cropping we get the related dimensions directly from the
    # cropper tool in $hCropData->{'naturalHeight'} and $hCropData->{'naturalWidth'} for the original image with unknown dimensions.
    # That means we have no size relation to an image on disk and we have to calculate
    # the new crop data for the _m size image because all operations regarding dimensions are related to the _m size image!
    if (($hCropData->{'variation'} == JSON::true) and $hCropData->{'height'} and $hCropData->{'width'}) {
        my $ImageHeight = $hCropData->{'naturalHeight'};
        my $ImageWidth  = $hCropData->{'naturalWidth'};
        my $Factor      = ($M_Height + $M_Width) / ($ImageHeight + $ImageWidth);
        # change crop data by factor for having crop data related to the _m size image
        foreach my $Key (qw(width height x y)){
            $hCropData->{$Key} = $hCropData->{$Key} * $Factor;
        }
    }

    # Get the uploaded original image by the _m size image
    # (we crop the original image later)
    my $OriginalFileName = _FindOriginalImage($Product, $INFileName);

    # Do we have to use a new filename? e.g. if we crop images for variation products
    my $OUTFileName = $UseNewFileName ? _GenerateNewFileName($Product, $OriginalFileName) : $OriginalFileName;
    my $PathToProductOriginalFile = $Product->get('PublicPath') .'/'. $OriginalFileName;
    my $PathToProductOUTFile      = $Product->get('PublicPath') .'/'. $OUTFileName;

    # Get the access rights because we have to change the rights on the target file
    my $SourceAccessRights = GetAccessRights($PathToProductOriginalFile);

    # Read the original image from disk
    my $OriginalImage = Image::Magick->new;
    my $NumberOfImagesOrErrorMessage = $OriginalImage->Read($PathToProductOriginalFile);
    if ($NumberOfImagesOrErrorMessage != 1){
        Error( 'FileHandlerError(Read)', { 'Parameter' => $NumberOfImagesOrErrorMessage } );
    }
    # prevent problems if we crop png images by resetting the page
    $OriginalImage->Set(page=>'0x0');

    # Get height first because rotate destroys this info.
    # Get factor because we have the cropping information for the _m file
    # and do the crop operation always on the original file.
    my $ImageHeight = $OriginalImage->Get('height');
    my $ImageWidth  = $OriginalImage->Get('width');
    my $Factor      = ($ImageHeight + $ImageWidth) / ($M_Height + $M_Width);

    # Do the rotation first
    $OriginalImage->Rotate($hCropData->{'rotate'}) if defined $hCropData->{'rotate'};

    # If we have crop data we calculate the crop factor
    # and scale the cropped original image afterwards.
    my $hParams = {};
    if($hCropData->{'height'} and $hCropData->{'width'}){
        # add crop factor and crop the image
        my $hScaledCropData = {};
        foreach my $Key (qw(width height x y)){
            $hScaledCropData->{$Key} = $hCropData->{$Key} * $Factor;
        }
        my $Geometry = "$hScaledCropData->{'width'}x$hScaledCropData->{'height'}+$hScaledCropData->{'x'}+$hScaledCropData->{'y'}";
        $OriginalImage->Crop('geometry'=> $Geometry);

        # Now we can build the parameters for creating sub images for the original image.
        # With $hParams we can overwrite settings for image sizes etc.
        # We always scale the _m size image to the crop area size because
        # GetScaledImages() uses standard sizes for images.
        $hParams->{'SCALE_HEIGHT_m'} = $hCropData->{'height'};
        $hParams->{'SCALE_WIDTH_m'}  = $hCropData->{'width'};
    }

    # Save original image
    $NumberOfImagesOrErrorMessage = $OriginalImage->Write($PathToProductOUTFile);
    if ($NumberOfImagesOrErrorMessage != 1){
        Error( 'FileHandlerError(Write)', { 'Parameter' => $NumberOfImagesOrErrorMessage } );
    }

    # Set the correct rights because apache needs to read the file
    SetAccessRights( $PathToProductOUTFile, $SourceAccessRights );

    # Generate sub sizes for original image (replace old sub sizes)
    GetScaledImages($PathToProductOUTFile, $Product->getSite(), $hParams);

    return {
            'OriginalFileName' => $OriginalFileName ,
            'NewFileName'      => $OUTFileName
           };

}

sub _FindOriginalImage {
    my ($Product, $FileName) = @_;

    my (undef, $FName, $Extension) = fsplit($FileName);

    my $PublicPath = $Product->get('PublicPath');
    my $OriginalFileName = $FName;
    # test if we have a FileName with a suffix
    if ( $FName =~ /^.*?_m$/ ) {
        # test if we have a file without the suffix, if not the file with suffix is the original file
        $OriginalFileName =~ s/_m//;
        if (ExistsFile($PublicPath.'/'.$OriginalFileName.$Extension)){
            return $OriginalFileName.$Extension;
        } else {
            return $FileName;
        }
    }
    return $FileName;
}

sub _GenerateNewFileName {
    my ($Product, $NewFileName) = @_;

    my $PublicPath = $Product->get('PublicPath');
    my (undef, $FileName, $Extension) = fsplit($NewFileName);
    # file must exists
    return unless ExistsFile($PublicPath . '/' . $FileName . $Extension);

    my $Count = 1;
    # test if FileName (w.o. extension) already have an suffix like -1- and replace suffix if exists
    if ($FileName =~ /.*-\d{1,}-$/gx){
        while (ExistsFile($PublicPath.'/'.$FileName.$Extension)){
            $Count++;
            $FileName =~ s/(-)\d{1,}(-)$/$1$Count$2/gx;
        }
    } else {
        # we add a suffix e.g. -1- to the FileName
        while (ExistsFile($PublicPath.'/'.$FileName."-$Count-".$Extension)){
            $Count++;
        }
        $FileName .= "-$Count-";
    }

    # return the old or new filename with -x- before the file extension
    return $FileName.$Extension;
}

1;
