# package      DE_EPAGES::ProductPortal::API::REST::PortalExportDriver
# base         DE_EPAGES::Presentation::API::REST::MonitoredExportDriver;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a REST driver to export object portal products
#
# example      use DE_EPAGES::ProductPortal::API::REST::PortalExportDriver;
#
#               my $Driver = DE_EPAGES::ProductPortal::API::REST::PortalExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->($FileName, $hFileFormat, $aColumnNames, $ViewObject);
#========================================================================================
package DE_EPAGES::ProductPortal::API::REST::PortalExportDriver;
use base DE_EPAGES::Presentation::API::REST::MonitoredExportDriver;

use strict;
use DE_EPAGES::Core::API::Error                     qw (GetError ExistsError);
use DE_EPAGES::Core::API::Html                      qw (RemoveHtmlTags);
use DE_EPAGES::Core::API::Warning                   qw (Warning);
use DE_EPAGES::Object::API::Factory                 qw (LoadObject LoadObjectByPath);
use DE_EPAGES::ProductPortal::API::Export           qw (FormatPrice
                                                        GetImageUrl
                                                        IsProductValidPortal
                                                        IsProductValidPortalExt1
                                                        GetProductPrice
                                                        GetProductUrl
                                                        GetShippingPrice);
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);

#========================================================================================
# function     exportREST
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportREST($FileName, $hFileFormat, $aColumnNames, $ViewObject, $NoMonitor);
#----------------------------------------------------------------------------------------
# description  starts the REST export
#----------------------------------------------------------------------------------------
# input        $FileName | export to file | string
# input        $hFileFormat | file format <function DE_EPAGES::Presentation::API::REST::MonitoredExportDriver::exportREST> | ref.hash
# input        $aColumnNames | name of column | ref.array.string
# input        $ViewObject | export based on this object | PortalConfig
# input        $NoMonitor | deactivate monitor, eg. for scripts/scheduler | boolean
#========================================================================================
sub exportREST {
    my $self = shift;
    my ($aColumnNames, $ViewObject, $NoMonitor) = @_;

    my $hPortalConfig = $ViewObject->get([qw (
        PortalSite
        Add2Basket
        ImageAttribute
        DescriptionAttribute
        CurrencyID
        LanguageID
        ShippingMethod
    )]);

    my $PortalSite = $hPortalConfig->{'PortalSite'};
    $self->{$_} = $hPortalConfig->{$_} foreach keys %$hPortalConfig;

    $self->{'Object'} = $ViewObject;
    $self->{'TaxModel'} = $PortalSite->get('TaxModel');
    $self->{'PortalSite'} = $PortalSite;
    $self->{'CountryID'} = $PortalSite->get('Country')->{'CountryID'};
    $self->{'NoMonitor'} = $NoMonitor;

    $self->SUPER::exportREST($aColumnNames, $ViewObject,[]);
    return;
}

#========================================================================================
# function     dayUnitName
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $name = dayUnitName;
#----------------------------------------------------------------------------------------
# description  returns the name of the time unit day in the current language
#               the value is being cached
# return       $dayUnitName | localized name of the time unit day | string
#========================================================================================
sub dayUnitName {
    my $self = shift;

    if (!defined($self->{'DayUnitName'})) {
        $self->{'DayUnitName'} = LoadObjectByPath('/Units/second/day')->get('Name', $self->languageID);
    }
    return $self->{'DayUnitName'};
}

#========================================================================================
# function     deliveryPeriod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Period = deliveryPeriod($Product);
#----------------------------------------------------------------------------------------
# description  returns the delivery period followed by the localized name of the time unit day
#----------------------------------------------------------------------------------------
# input        $Product | product | object
# return       $DeliveryPeriod | deliveryperiod | string
#========================================================================================
sub deliveryPeriod {
    my $self = shift;
    my ($Product) = @_;

    my $DeliveryPeriod = $Product->getWithParent('DeliveryPeriod');
    return undef unless defined $DeliveryPeriod;

    return $DeliveryPeriod . ' ' . $self->dayUnitName;
}

#========================================================================================
# function     taxModel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxModel = $self->taxModel;
#----------------------------------------------------------------------------------------
# description  Tax model used for export.
#----------------------------------------------------------------------------------------
# return       $TaxModel | tax model | int
#========================================================================================
sub taxModel { return shift->{'TaxModel'}; }

#========================================================================================
# function     portalSite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalSite = $self->portalSite;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $PortalSite | Portal site | object
#========================================================================================
sub portalSite { return shift->{'PortalSite'}; }

#========================================================================================
# function     countryID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $countryID = $self->countryID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $countryID | country id | int
#========================================================================================
sub countryID { return shift->{'CountryID'}; }

#========================================================================================
# function     currencyID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CurrencyID = $self->currencyID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $CurrencyID | currency id (e.g. 'EUR') | char(3)
#========================================================================================
sub currencyID { return shift->{'CurrencyID'}; }

#========================================================================================
# function     languageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LanguageID = $self->languageID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $LanguageID | language | object
#========================================================================================
sub languageID { return shift->{'LanguageID'}; }

#========================================================================================
# function     shippingMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod = $self->shippingMethod;
#----------------------------------------------------------------------------------------
# description  used shipping method of portal config.
#----------------------------------------------------------------------------------------
# return       $ShippingMethod | shipping method | object
#========================================================================================
sub shippingMethod { return shift->{'ShippingMethod'}; }

#========================================================================================
# function     imageAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ImageAttribute = $self->imageAttribute;
#----------------------------------------------------------------------------------------
# description  used image attribute of portal config.
#----------------------------------------------------------------------------------------
# return       $ImageAttribute | image attribute | string
#========================================================================================
sub imageAttribute { return shift->{'ImageAttribute'}; }

#========================================================================================
# function     descriptionAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DescriptionAttribute = $self->descriptionAttribute;
#----------------------------------------------------------------------------------------
# description  used description attribute of portal config.
#----------------------------------------------------------------------------------------
# return       $DescriptionAttribute | description attribute | string
#========================================================================================
sub descriptionAttribute { return shift->{'DescriptionAttribute'}; }

#========================================================================================
# function     useAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UseAddToBasket = $self->useAddToBasket;
#----------------------------------------------------------------------------------------
# description  use add to basket link.
#----------------------------------------------------------------------------------------
# return       $UseAddToBasket | use add to basket link | boolean
#========================================================================================
sub useAddToBasket { return shift->{'Add2Basket'}; }

#========================================================================================
# function     startMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->startMonitor($PageTypeName);
#----------------------------------------------------------------------------------------
# description  starts the monitor (if plain "\rexport products (%d / %d)")
#----------------------------------------------------------------------------------------
# input        $PageTypeName | name of pagetype | string
#========================================================================================
sub startMonitor {
    my $self = shift;
    my ($PageTypeName) = @_;

    return if $self->{'NoMonitor'};
    return if exists $self->{'Monitor'};
    my $AppServer = GetAppServer();
    if (defined $AppServer) {
        my $Monitor = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
            'PageTypeName' => $PageTypeName,
            'Object'       => $self->object,
        );
        $self->{'Monitor'} = $Monitor;
    } else {
        my $Monitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
            'ContentTemplate' => "\rexport products (%d / %d)",
        );
        $self->{'Monitor'} = $Monitor;
    }
    return;
}

#========================================================================================
# function     allProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->allProducts;
#----------------------------------------------------------------------------------------
# description  Returns the product identifiers to be exported. If the option
#               "ExportAllVariation" is selected all SubProducts of a MasterProduct
#               are exported, else only default SubProducts are exported.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub allProducts {
    my $self = shift;

    return $self->object->allProducts;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->items;
#----------------------------------------------------------------------------------------
# description  Returns all product identifier of portal config, initialize monitor.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub items {
    my $self = shift;

    my $aProductIDs = $self->object->items;
    my $Monitor = $self->monitor;
    $Monitor->set({'Total'=> scalar @$aProductIDs}) if defined $Monitor;
    return $aProductIDs;
}

#========================================================================================
# function     getExportValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aValues = $self->getExportValues($Object);
#----------------------------------------------------------------------------------------
# description  helper method of <function exportLine> to convert column names to methods
#               names of this exporter and calls the method to get the values of the
#               exported object. The order of return values depends on order of column
#               names.
#----------------------------------------------------------------------------------------
# input        $Object | exported object | object
# return       $aValues | list of values | ref.array.string
#========================================================================================
sub getExportValues {
    my $self = shift;
    my ($Object) = @_;
    my @Values;

    foreach my $ColumnName (@{$self->columns}) {
        my $MethodName = $ColumnName;
        $MethodName =~ s/\W/_/g;
        my $Value = $self->$MethodName( $Object );
        push @Values, $Value;
    }
    return \@Values;
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->exportLine;
#----------------------------------------------------------------------------------------
# description  Export one line, increments monitor.
#========================================================================================
sub exportLine {
    my $self = shift;
    my $Object = LoadObject($self->topID('Item'));

    my $Monitor = $self->monitor;
    if (defined $Monitor) {
        $self->monitorIncrement;
        $self->monitorContent;
    }
    eval {
        $self->isProductValidPortal($Object);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars({'ProductNotExported' => 1});
        Warning($Error);
        return;
    }
    $self->writeLine($self->getExportValues($Object));
    return;
}

#========================================================================================
# function     isProductValidPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortal;
#----------------------------------------------------------------------------------------
# description  Throws error if not valid, (not visible).
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    IsProductValidPortal($Product);
}

#========================================================================================
# function     isProductValidPortalExt1
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortalExt1;
#----------------------------------------------------------------------------------------
# description  Throws error if not valid (not visible, no name or no price).
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    IsProductValidPortalExt1($self->object, $Product, $self->currencyID, $self->taxModel);
}

#========================================================================================
# function     productPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->productPrice($Product);
#----------------------------------------------------------------------------------------
# description  get price of product with vars of portal config.
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
# return       $Value | price | float
#========================================================================================
sub productPrice {
    my $self = shift;
    my ($Product) = @_;

    return GetProductPrice($Product, $self->currencyID, $self->taxModel, $self->countryID);
}

#========================================================================================
# function     shippingPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->shippingPrice($Product);
#----------------------------------------------------------------------------------------
# description  get shipping price for product with vars of portal config.
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
# return       $Value | price | float
#========================================================================================
sub shippingPrice {
    my $self = shift;
    my ($Product) = @_;

    return GetShippingPrice($self->object, $Product, $self->currencyID, $self->taxModel, $self->countryID);
}

#========================================================================================
# function     formatPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedValue = $self->formatPrice($Value);
# example      $FormattedShippingValue = $self->formatPrice($self->shippingPrice($Product));
#----------------------------------------------------------------------------------------
# description  formats prices (from shipping or products) to portal specific format.
#----------------------------------------------------------------------------------------
# input        $Value | price | float
# return       $FormattedValue | formatted price | string
#========================================================================================
sub formatPrice {
    my $self = shift;
    my ($Value) = @_;

    return FormatPrice($Value, $self->portalSite, $self->currencyID);
}

#========================================================================================
# function     productUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = $self->productUrl($Product);
#----------------------------------------------------------------------------------------
# description  Returns url to product.
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
# return       $URL | product url | string
#========================================================================================
sub productUrl {
    my $self = shift;
    my ($Product) = @_;

    return GetProductUrl($self->object, $Product, $self->useAddToBasket, $self->currencyID);
}

#========================================================================================
# function     imageUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = $self->imageUrl($Product, $ImageAttribute);
# example      $URL = $self->imageUrl($Product, $self->imageAttribute);
#----------------------------------------------------------------------------------------
# description  Returns url to product image.
#----------------------------------------------------------------------------------------
# input        $Product | exported product | DE_EPAGES::Product::API::Object::Product
# input        $ImageAttribute | image attributes | ref.hash
# return       $URL | image url | string
#========================================================================================
sub imageUrl {
    my $self = shift;
    my ($Product, $ImageAttribute) = @_;

    return GetImageUrl($Product, $ImageAttribute, $self->languageID);
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive($aObjects);
#----------------------------------------------------------------------------------------
# description  Overwrite standard drive methode to start monitor 'PortalMonitor'.
#----------------------------------------------------------------------------------------
# input        $aObjects | list of products to export | re.array.object
#========================================================================================
sub drive {
    my $self = shift;
    my ($aObjects) = @_;

    $self->startMonitor('PortalMonitor');
    $self->SUPER::drive($aObjects);
}

#========================================================================================
# function     removeHtmlTags
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $NoHtml = $self->removeHtmlTags($HTML);
#----------------------------------------------------------------------------------------
# description  Removes all html tags and consolidates spaces. Removes newlines after
#               converting.
#----------------------------------------------------------------------------------------
# input        $HTML | html string | string
# return       $NoHtml | string without html tags | string
#========================================================================================
sub removeHtmlTags {
    my $self = shift;
    my ($html) = @_;

    return '' unless defined $html;

    my $noHtml = RemoveHtmlTags($html);
    $noHtml =~ s/\s+/ /g;

    return $noHtml;
}

#========================================================================================
# function     formatedReferencePrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $referencePrice  = $self->formatedReferencePrice($Product);
#----------------------------------------------------------------------------------------
# description  EPG-23938 - Returns String with all data regarding reference price
#----------------------------------------------------------------------------------------
# input        $Product | product object | DE_EPAGES::Product::API::Object::Product
# return       $referencePrice | reference price | String
#========================================================================================
sub formatedReferencePrice {
    my $self = shift;
    my ( $Product ) = @_;
    my $Price     = $Product->getListPrice( $self->currencyID, $self->taxModel );
    my $RefPrice  = $Product->getReferencePrice( $self->currencyID, $Price );
    my $RefUnit   = $Product->getWithParent( 'RefUnit' );

    #ReferencePrice != #DisplayPrice same as in SF-Product.Content-Base-ImageUprightRight.html
    if ( defined $Price && defined $RefPrice
        && $Price != $RefPrice
        && defined $RefUnit
    ){
        my $RefAmount = $Product->getWithParent( 'RefAmount' );
        return sprintf ('%s %s=%s',
                        $self->portalSite->formatFloat($RefAmount),
                        $RefUnit->get( 'Abbreviation' ) // $RefUnit->get( 'NameOrAlias', $self->languageID ),
                        $self->portalSite->formatMoney( $RefPrice, $self->currencyID )
                      );
    }
    return '0';
}
1;
