#========================================================================================
# package      DE_EPAGES::Portal::API::XML::PortalExportDriver
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a xml driver to export object portal products
#
# example      use DE_EPAGES::ProductPortal::API::XML::PortalExportDriver;
#
#               my $Driver = DE_EPAGES::ProductPortal::API::XML::PortalExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->exportXML;
#========================================================================================
package DE_EPAGES::ProductPortal::API::XML::PortalExportDriver;
use base DE_EPAGES::Presentation::API::XML::MonitoredExportDriver;

use strict;
use DE_EPAGES::ProductPortal::API::Export qw (FormatPrice
    GetImageUrl
    IsProductValidPortal
    IsProductValidPortalExt1
    GetProductPrice
    GetProductUrl
    GetShippingPrice);
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);
use DE_EPAGES::Object::API::Table::Currency         qw (GetInfoCurrency);

#========================================================================================
# function     exportXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportXML($FileName, $hFileFormat, $aColumnNames, $ViewObject, $NoMonitor);
#----------------------------------------------------------------------------------------
# description  Exports objects into an XML file.
#----------------------------------------------------------------------------------------
# input        $FileName | name of export file | string
# input        $hFormat | file format specification
#                       <ul>
#                         <li>see: <function DE_EPAGES::XML::API::BaseExportDriver::exportXML></li>
#                         <li>Monitor - progress monitor (optional, see: <package DE_EPAGES::Presentation::API::Object::ProgressMonitor>)</li>
#                       </ul>
#                       | ref.hash
# input        $ViewObject | object to view monitor | object
# input        $aObjects | list of objects | ref.array.object
# input        $NoMonitor | true if output is to be monitored | boolean
#========================================================================================
sub exportXML {
    my $self = shift;
    my ( $FileName, $hFileFormat, $ViewObject, $NoMonitor ) = @_;

    my $hPortalConfig = $ViewObject->get([qw(
        PortalSite
        Add2Basket
        ImageAttribute
        DescriptionAttribute
        CurrencyID
        LanguageID
        ShippingMethod
    )]);

    my $PortalSite = $hPortalConfig->{'PortalSite'};
    $self->{$_} = $hPortalConfig->{$_} foreach keys %$hPortalConfig;

    $self->{'Object'}     = $ViewObject;
    $self->{'TaxModel'}   = $PortalSite->get('TaxModel');
    $self->{'PortalSite'} = $PortalSite;
    $self->{'CountryID'} = $PortalSite->get('Country')->{'CountryID'};
    $self->{'NoMonitor'}  = $NoMonitor;

    $self->SUPER::exportXML( $FileName, $hFileFormat, $ViewObject, [] );
    return;
}

#========================================================================================
# function     taxModel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxModel = $self->taxModel;
#----------------------------------------------------------------------------------------
# description  Tax model used for export.
#----------------------------------------------------------------------------------------
# return       $TaxModel | tax model | int
#========================================================================================
sub taxModel { return shift->{'TaxModel'}; }

#========================================================================================
# function     portalSite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PortalSite = $self->portalSite;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $PortalSite | Portal site | object
#========================================================================================
sub portalSite { return shift->{'PortalSite'}; }

#========================================================================================
# function     countryID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $countryID = $self->countryID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $countryID | country id | int
#========================================================================================
sub countryID { return shift->{'CountryID'}; }

#========================================================================================
# function     currencyID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CurrencyID = $self->currencyID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $CurrencyID | currency id (e.g. 'EUR') | char(3)
#========================================================================================
sub currencyID { return shift->{'CurrencyID'}; }

#========================================================================================
# function     languageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LanguageID = $self->languageID;
#----------------------------------------------------------------------------------------
# description  Exports to site of portal.
#----------------------------------------------------------------------------------------
# return       $LanguageID | language | object
#========================================================================================
sub languageID { return shift->{'LanguageID'}; }

#========================================================================================
# function     shippingMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingMethod = $self->shippingMethod;
#----------------------------------------------------------------------------------------
# description  used shipping method of portal config.
#----------------------------------------------------------------------------------------
# return       $ShippingMethod | shipping method | object
#========================================================================================
sub shippingMethod { return shift->{'ShippingMethod'}; }

#========================================================================================
# function     imageAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ImageAttribute = $self->imageAttribute;
#----------------------------------------------------------------------------------------
# description  used image attribute of portal config.
#----------------------------------------------------------------------------------------
# return       $ImageAttribute | image attribute | string
#========================================================================================
sub imageAttribute { return shift->{'ImageAttribute'}; }

#========================================================================================
# function     descriptionAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DescriptionAttribute = $self->descriptionAttribute;
#----------------------------------------------------------------------------------------
# description  used description attribute of portal config.
#----------------------------------------------------------------------------------------
# return       $DescriptionAttribute | description attribute | string
#========================================================================================
sub descriptionAttribute { return shift->{'DescriptionAttribute'}; }

#========================================================================================
# function     useAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UseAddToBasket = $self->useAddToBasket;
#----------------------------------------------------------------------------------------
# description  use add to basket link.
#----------------------------------------------------------------------------------------
# return       $UseAddToBasket | use add to basket link | boolean
#========================================================================================
sub useAddToBasket { return shift->{'Add2Basket'}; }

#========================================================================================
# function     startMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->startMonitor($PageTypeName);
#----------------------------------------------------------------------------------------
# description  starts the monitor
#----------------------------------------------------------------------------------------
# input        $PageTypeName | name of pagetype | string
#========================================================================================
sub startMonitor {
    my $self = shift;
    my ($PageTypeName) = @_;

    return if $self->{'NoMonitor'};
    return if exists $self->{'Monitor'};
    my $AppServer = GetAppServer();
    if ( defined $AppServer ) {
        my $Monitor = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
            'PageTypeName' => $PageTypeName,
            'Object'       => $self->object
        );
        $self->{'Monitor'} = $Monitor;
    }
    else {
        my $Monitor = DE_EPAGES::Core::API::Object::ProgressMonitor->new(
          'ContentTemplate' => "\rexport products (%d / %d)"
        );
        $self->{'Monitor'} = $Monitor;
    }
}

#========================================================================================
# function     allProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->allProducts;
#----------------------------------------------------------------------------------------
# description  Returns the product identifiers to be exported. If the option
#               "ExportAllVariation" is selected all SubProducts of a MasterProduct
#               are exported, else only default SubProducts are exported.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub allProducts {
    my $self = shift;

    return $self->object->allProducts;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductIDs = $self->items;
#----------------------------------------------------------------------------------------
# description  Returns all product identifier of portal config, initialize monitor.
#----------------------------------------------------------------------------------------
# return       $aProductIDs | product identifier | ref.array.int
#========================================================================================
sub items {
    my $self = shift;

    my $aProductIDs = $self->object->items;
    my $Monitor     = $self->monitor;
    $Monitor->set( { 'Total' => scalar @$aProductIDs } ) if defined $Monitor;
    return $aProductIDs;
}

#========================================================================================
# function     isProductValidPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortal;
#----------------------------------------------------------------------------------------
# description  Throws error if not valid, (not visible).
#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    return IsProductValidPortal($Product);
}

#========================================================================================
# function     isProductValidPortalExt1
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortalExt1;
#----------------------------------------------------------------------------------------
# description  Throws error if not valid (not visible, no name or no price).
#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    return IsProductValidPortalExt1( $self->object, $Product, $self->currencyID, $self->taxModel );
}

#========================================================================================
# function     productPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->productPrice($Product);
#----------------------------------------------------------------------------------------
# description  get price of product with vars of portal config.
#----------------------------------------------------------------------------------------
# return       $Value | price | float
#========================================================================================
sub productPrice {
    my $self = shift;
    my ($Product) = @_;

    return GetProductPrice( $Product, $self->currencyID, $self->taxModel, $self->countryID );
}

#========================================================================================
# function     shippingPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->shippingPrice($Product);
#----------------------------------------------------------------------------------------
# description  get shipping price for product with vars of portal config.
#----------------------------------------------------------------------------------------
# return       $Value | price | float
#========================================================================================
sub shippingPrice {
    my $self = shift;
    my ($Product) = @_;

    return GetShippingPrice( $self->object, $Product, $self->currencyID, $self->taxModel, $self->countryID );
}

#========================================================================================
# function     formatPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedValue = $self->formatPrice($Value);
# example      $FormattedShippingValue = $self->formatPrice($self->shippingPrice($Product));
#----------------------------------------------------------------------------------------
# description  formats prices (from shipping or products) to portal specific format.
#----------------------------------------------------------------------------------------
# input        $Value | price | float
# return       $FormattedValue | formatted price | string
#========================================================================================
sub formatPrice {
    my $self = shift;
    my ($Value) = @_;

    return FormatPrice( $Value, $self->portalSite, $self->currencyID );
}

#========================================================================================
# function     productUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = $self->productUrl($Product);
#----------------------------------------------------------------------------------------
# description  Returns url to product.
#----------------------------------------------------------------------------------------
# return       $URL | product url | string
#========================================================================================
sub productUrl {
    my $self = shift;
    my ($Product) = @_;

    return GetProductUrl( $self->object, $Product, $self->useAddToBasket, $self->currencyID );
}

#========================================================================================
# function     imageUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $URL = $self->imageUrl($Product, $ImageAttribute);
# example      $URL = $self->imageUrl($Product, $self->imageAttribute);
#----------------------------------------------------------------------------------------
# description  Returns url to product image.
#----------------------------------------------------------------------------------------
# return       $URL | image url | string
#========================================================================================
sub imageUrl {
    my $self = shift;
    my ( $Product, $ImageAttribute ) = @_;

    return GetImageUrl( $Product, $ImageAttribute, $self->languageID );
}

#========================================================================================
# function     formatedReferencePrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       $referencePrice = $self->formatedReferencePrice($Product);
#----------------------------------------------------------------------------------------
# description  EPG-23938 - Returns String with all data regarding reference price,
#               for example: 100g=1.77.
#               Returns '0' if there is no reference price.
#----------------------------------------------------------------------------------------
# input        $Product | product object | DE_EPAGES::Product::API::Object::Product
#----------------------------------------------------------------------------------------
# return       $referencePrice | reference price | String
#========================================================================================
sub formatedReferencePrice {
    my $self = shift;
    my ($Product) = @_;
    my $Price = $self->productPrice( $Product );
    my $RefPrice = $Product->getReferencePrice( $self->currencyID, $Price );

#ReferencePrice != #DisplayPrice same as in SF-Product.Content-Base-ImageUprightRight.html
    if (   defined $Price
        && defined $RefPrice
        && $Price != $RefPrice )
    {
        my $RefAmount = $Product->getWithParent('RefAmount');
        my $RefUnit   = $Product->getWithParent('RefUnit');
        return sprintf(
            '%d%s=%.2f%s',
            $RefAmount,
            $RefUnit->get('Abbreviation') // $RefUnit->get( 'NameOrAlias', $self->languageID ),
            $RefPrice,
            GetInfoCurrency( $self->currencyID )->{'Symbol'},
        );
    }

    return '0';
}

#========================================================================================
sub drive {
    my $self = shift;
    my ($aObjects) = @_;

    $self->startMonitor('PortalMonitor');
}

#========================================================================================
sub xml_decl_string { undef; }

#========================================================================================
sub xml_decl {
    my $self        = shift;
    my (@Parameter) = @_;
    my $String      = $self->xml_decl_string;
    return $self->SUPER::xml_decl(@Parameter) unless defined $String;

    # find the XML::SAX::Writer
    my $Handler = $self;
    my $Writer;
    do {
        $Writer = $Handler if exists $Handler->{'Consumer'};
        $Handler = $Handler->can('get_handler')
            ? $Handler->get_handler
            : $Handler->{'Handler'};
    } while ( !defined $Writer && defined $Handler );

    die 'Consumer not found in filter chain' unless defined $Writer;

    # write the desired string directly to the output
    $Writer->{Consumer}->output($String);
}

1;
