#========================================================================================
# package      DE_EPAGES::SEO::API::XMLSitemap
# state        public
# base         Exporter
# description  This package provides shop functions
#========================================================================================
package DE_EPAGES::SEO::API::XMLSitemap;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    ExportXMLSitemap
);

use DE_EPAGES::Content::API::Table::ShortURL qw ( GetInfoShortURLsBySiteID );
use DE_EPAGES::Core::API::Error              qw ( Error ExistsError GetError VarsOnError );
use DE_EPAGES::Core::API::File               qw ( DeleteFile ExistsFile WriteFile GetInfoDirectory CreateDirectory ExistsDirectory SetAccessRights );
use DE_EPAGES::Object::API::Factory          qw ( LoadClassByAlias );
use DE_EPAGES::SEO::API::Constants           qw ( @BLACKLIST_MOBILECONTENT_CLASSES $SITEMAP_NAME $SITEMAP_NAME_MOBILE $SITEMAP_NAME_EXT $SITEMAP_EXPORT_FOLDER );
use DE_EPAGES::SEO::API::XML::SitemapExportDriver;
use DE_EPAGES::Trigger::API::Trigger         qw ( TriggerHook );
use DE_EPAGES::Database::API::Connection     qw ( GetCurrentDBHandle );

#-- The maximum number of entries we will export across ALL sub sitemap files
use constant MAX_ENTRIES => 10000000;
#-- The maximum number we will export per sub sitemap file
use constant MAX_PER_FILE => 50000;

#========================================================================================
# function     ExportXMLSitemap
# state        public
#----------------------------------------------------------------------------------------
# syntax       ExportXMLSitemap($Shop);
#----------------------------------------------------------------------------------------
# description  calls export of sitemap xml files
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#========================================================================================
sub ExportXMLSitemap {
    my ($Shop) = @_;
    #export as xml-file
    my $SitemapFileName = $Shop->get('SitemapPathFileName');
    my $TempSitemapFileName = "$SitemapFileName.tmp";
    my $MobileSitemapFileName = $Shop->get('MobileSitemapPathFileName');
    my $UseMobileSitemap = $Shop->featureMaxValue('MobileView') && $Shop->get('UseMobileSF');
    my $TempMobileSitemapFileName = "$MobileSitemapFileName.tmp";
    eval {
        #-- Create the .tmp file first, as a marker that the export is currntly running
        WriteFile($TempSitemapFileName, \'', { 'create_directory' => 1 }) unless ExistsFile($TempSitemapFileName);
        DeleteFile($TempMobileSitemapFileName) if (!$UseMobileSitemap && ExistsFile($TempMobileSitemapFileName));
        _exportXMLSitemap( $Shop, $TempSitemapFileName, $UseMobileSitemap ? $TempMobileSitemapFileName : undef );
        rename $TempSitemapFileName, $SitemapFileName or Error('FileRenameFails', {
            'Source' => $TempSitemapFileName,
            'Target' => $SitemapFileName,
            'Code' => $!
        });
        if($UseMobileSitemap) {
            rename $TempMobileSitemapFileName, $MobileSitemapFileName or Error('FileRenameFails', {
                'Source' => $TempMobileSitemapFileName,
                'Target' => $MobileSitemapFileName,
                'Code' => $!
            });
        }
    };
    if (ExistsError()) {
        my $Error = GetError;
        DeleteFile($TempSitemapFileName) if ExistsFile($TempSitemapFileName);
        DeleteFile($TempMobileSitemapFileName) if ExistsFile($TempMobileSitemapFileName);
        Error($Error);
    }
}




#========================================================================================
# function     _exportSubSitemapXML
# state        private
#----------------------------------------------------------------------------------------
# description  Exports and writes a sub sitemap file, replaces SitemapExportDriver
#========================================================================================
sub _exportSubSitemapXML {
    my ($PathFileName, $MobilePathFileName, $Shop, $ahData, $aMobileBlackListedClassIDs, $hClassesMap, $hContentObjectsRestricted) = @_;

    my $SitemapContent = '<?xml version="1.0" encoding="UTF-8"?><urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
    my $MobileSitemapContent = '<?xml version="1.0" encoding="UTF-8"?><urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:mobile="http://www.google.com/schemas/sitemap-mobile/1.0">';

    my $hShopInfo = $Shop->get(['HasSSLCertificate', 'WebServerScriptUrl', 'WebServerScriptUrlMobileSSL', 'ProtocolAndServer', 'ProtocolAndServerSSL', 'WebServerScriptUrlMobile']);

    my $hExcludedForMobile;
    $hExcludedForMobile->{$_} = 1 foreach @$aMobileBlackListedClassIDs;

    #-- Main loop, export all content objects
    foreach my $hInfo ( @$ahData ) {
      my $ObjectID = $hInfo->{'ObjectID'};

      #-- If this object id has an entry in the permission table, exclude it as it means it is restricted
      if ( exists $hContentObjectsRestricted->{$ObjectID} ) {
        next;
      }

      #-- REALLY slow, DON'T use it!
      #next unless EveryoneCanDoIt( LoadObject($hInfo->{'ObjectID'}), 'View');

      my $Url = $hInfo->{'URL'};
      my $MobileUrl = $Url;

      #-- Export an explicit priority tag for certain classes
      my $ClassID = $hInfo->{'ClassID'};
      my $Priority;
      if ($ClassID == $hClassesMap->{'StartPage'}){
        $Priority = 1;
      } elsif ($ClassID == $hClassesMap->{'Imprint'}){
        $Priority = 0;
      } elsif ($ClassID == $hClassesMap->{'CustomerInformation'}){
        $Priority = 0;
      } elsif ($ClassID == $hClassesMap->{'PrivacyPolicy'}){
        $Priority = 0;
      } elsif ($ClassID == $hClassesMap->{'TermsAndConditions'}){
        $Priority = 0;
      }

      $Priority = "<priority>$Priority</priority>" if defined $Priority;

      if ($hShopInfo->{'HasSSLCertificate'}) {
        # change servlet in URLs to .mobile and https
        $MobileUrl =~ s/\Q$hShopInfo->{'WebServerScriptUrl'}\E/$hShopInfo->{'WebServerScriptUrlMobileSSL'}/;

        # change URLs that don't contain /epages/<ShopName>.{mobile|sf} to https
        $Url =~ s/\Q$hShopInfo->{'ProtocolAndServer'}\E/$hShopInfo->{'ProtocolAndServerSSL'}/;
      } else {
        # change servlet in URLs to .mobile
        $MobileUrl =~ s/\Q$hShopInfo->{'WebServerScriptUrl'}\E/$hShopInfo->{'WebServerScriptUrlMobile'}/;
      }

      $Url = _escapeForSitemap($Url);
      $MobileUrl = _escapeForSitemap($MobileUrl);

      $SitemapContent .= "\n<url><loc>" . $Url . "</loc>$Priority</url>";
      if ( ! exists $hExcludedForMobile->{$ClassID} ) {
        $MobileSitemapContent .= "\n<url><loc>" . $MobileUrl . "</loc>$Priority<mobile:mobile /></url>";
      }
    }

    $SitemapContent .= "\n</urlset>";
    $MobileSitemapContent .= "\n</urlset>";

    WriteFile($PathFileName, \$SitemapContent, { 'create_directory' => 1 });

    VarsOnError({ 'PathFileName' => $PathFileName });
    SetAccessRights($PathFileName);

    if($MobilePathFileName) {
        WriteFile($MobilePathFileName, \$MobileSitemapContent, { 'create_directory' => 1 });

        VarsOnError({ 'MobilePathFileName' => $MobilePathFileName });
        SetAccessRights($MobilePathFileName);
    }

    return;
}

sub _escapeForSitemap {
  my ($String) = @_;

  my $Escaped = $String;

  $Escaped =~ s/\&/\&amp;/g;
  $Escaped =~ s/\'/\&apos;/g;
  $Escaped =~ s/\"/\&quot;/g;
  $Escaped =~ s/\</\&lt;/g;
  $Escaped =~ s/\>/\&gt;/g;

  return $Escaped;
}

#========================================================================================
# function     _exportXMLSitemap
# state        private
#----------------------------------------------------------------------------------------
# syntax       _exportXMLSitemap($Shop);
#----------------------------------------------------------------------------------------
# description  Exports sitemap xml
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $SitemapFileName | private path to sitemap.xml | string
# input        $MobileSitemapFileName | private path to mobilesitemap.xml | string
#========================================================================================
sub _exportXMLSitemap {
    my ($Shop, $SitemapFileName, $MobileSitemapFileName) = @_;

    my $aContentObjectsData = GetObjects($Shop);

    my $hClassesMap;
    foreach my $ClassAlias ( qw (StartPage Imprint CustomerInformation PrivacyPolicy TermsAndConditions)) {
        $hClassesMap->{$ClassAlias} = LoadClassByAlias($ClassAlias)->id;
    }
    my @MobileBlackListedClassIDs = map { LoadClassByAlias($_)->id } @BLACKLIST_MOBILECONTENT_CLASSES;

    #-- Base folder under shop /WebRoot, where we export the sub sitemap files
    #-- e.g. /WebRoot/Store/Shops/DemoShop/Sitemaps
    my $BaseExportPath = $Shop->get('PublicPath') . "/$SITEMAP_EXPORT_FOLDER/";
    CreateDirectory($BaseExportPath) if ! ExistsDirectory($BaseExportPath);
    my $hInfo = GetInfoDirectory( $BaseExportPath , { 'recursive' => 0, 'relative' => 1 } );

    #-- All existing sub sitemap files in the folder, we use this to delete unused files later
    my $hExistingFiles;
    foreach ( @{$hInfo->{'Files'}} ) {
      #-- Match only files that we would generate
      next unless $_ =~ /^(?:mobile)?sitemap-[0-9]+\.xml(?:\.tmp)?$/;

      $hExistingFiles->{$_} = 1;
    }

    #-- Build a hash of all content object ids, with an entry in table 'permission'.
    #-- The assumption is that any entry here means that it is restricted.
    #-- For speedy lookups
    my $Sql = 'select distinct contentobjectid from contentobject co join permission p on co.contentobjectid=p.objectid where co.siteid = ?';
    my $dbh = GetCurrentDBHandle();
    my $aaResults = $dbh->execute(
        $Sql,
        [$Shop->id],
    );
    my $hContentObjectsRestricted;
    $hContentObjectsRestricted->{$_->[0]} = 1 foreach @$aaResults;

    my $aIndexes = [];
    my $Index = 1;

    #-- The idea is to extract MAX_PER_FILE elements per run and export all these elements to a sub sitemap file
    while ( scalar @$aContentObjectsData > 0 ) {
      #-- Really slow to reuse the same instance, seems the objects from the previous run are still there somewhere, need to 'reset' it somehow
      #-- Really slow in general to use this parser here, can investigate further when we have lust/time
      #my $Driver = DE_EPAGES::SEO::API::XML::SitemapExportDriver->new($hClassesMap);
      #$Driver->exportXML($FileNameTemp, $MobileFileNameTemp, $Shop, \@Objects, \@MobileBlackListedClassIDs);

      #-- Each sub sitemap file is named the same as the sitemap index, with a number after it "-n"
      #-- e.g. sitemap-1.xml, sitemap-2.xml, mobilesitemap-1.xml, mobilesitemap-2.xml
      my $FileNameFinal = $BaseExportPath . _getSitemapSubFileName(0, $Index);
      my $MobileFileNameFinal = $MobileSitemapFileName ? $BaseExportPath . _getSitemapSubFileName(1, $Index) : undef;

      #-- We write to a .tmp file so that the existing files remain accessible, then rename them later
      my $FileNameTemp = "$FileNameFinal.tmp";
      my $MobileFileNameTemp = $MobileSitemapFileName ? "$MobileFileNameFinal.tmp" : undef;

      push @$aIndexes, $Index;

      #-- Export to the .tmp file
      #-- Extract MAX_PER_FILE elements, the maximum that we can write to per file
      my @Objects = splice(@$aContentObjectsData,0,MAX_PER_FILE);

      _exportSubSitemapXML($FileNameTemp, $MobileFileNameTemp, $Shop, \@Objects, \@MobileBlackListedClassIDs, $hClassesMap, $hContentObjectsRestricted);

      $Index++;
    }

    #-- Sitemap index file content
    my $SiteMapContent =<<END;
<?xml version="1.0" encoding="UTF-8"?>
<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
END
    my $MobileSiteMapContent = $SiteMapContent;

    #-- Rename each sub sitemap .tmp file
    #-- Generate the main sitemap index file
    foreach my $Index ( @$aIndexes ) {
      my $FileNameIndex = _getSitemapSubFileName(0, $Index);
      my $MobileFileNameIndex = _getSitemapSubFileName(1, $Index);

      my $FileName = $BaseExportPath . $FileNameIndex;
      my $MobileFileName = $MobileSitemapFileName ? $BaseExportPath . $MobileFileNameIndex : undef;

      eval {
          rename "$FileName.tmp", $FileName or Error('FileRenameFails', {
              'Source' => "$FileName.tmp",
              'Target' => $FileName,
              'Code' => $!
          });
          if($MobileSitemapFileName) {
              rename "$MobileFileName.tmp", $MobileFileName or Error('FileRenameFails', {
                  'Source' => "$MobileFileName.tmp",
                  'Target' => $MobileFileName,
                  'Code' => $!
              });
          }
      };
      if (ExistsError()) {
          my $Error = GetError;
          DeleteFile("$FileName.tmp") if ExistsFile("$FileName.tmp");
          DeleteFile("$MobileFileName.tmp") if ExistsFile("$MobileFileName.tmp");
          Error($Error);
      }

      #-- Remove the renamed file, any remaining entries in this hash will be deleted
      delete $hExistingFiles->{$FileNameIndex};
      delete $hExistingFiles->{$MobileFileNameIndex} if $MobileFileName;

      #-- Include the webroot path for this sub sitemap file in the sitemap index
      my $WebPath = $Shop->get($Shop->get('HasSSLCertificate')?'ProtocolAndServerSSL':'ProtocolAndServer') . $Shop->get('WebPath') . "/$SITEMAP_EXPORT_FOLDER/" . $FileNameIndex;
      my $MobileWebPath = $Shop->get($Shop->get('HasSSLCertificate')?'ProtocolAndServerSSL':'ProtocolAndServer') . $Shop->get('WebPath') . "/$SITEMAP_EXPORT_FOLDER/" . $MobileFileNameIndex;

      $SiteMapContent .= "<sitemap><loc>$WebPath</loc></sitemap>";
      $MobileSiteMapContent .= "<sitemap><loc>$MobileWebPath</loc></sitemap>";
    }

    #-- Delete any unused sub sitemap file
    DeleteFile("$BaseExportPath$_") foreach ( keys %$hExistingFiles );

    #-- Now we can write the main sitemap index file
    $SiteMapContent .= '</sitemapindex>';
    $MobileSiteMapContent .= '</sitemapindex>';
    WriteFile($SitemapFileName, \$SiteMapContent, { 'create_directory' => 1 });
    WriteFile($MobileSitemapFileName, \$MobileSiteMapContent, { 'create_directory' => 1 }) if $MobileSitemapFileName;

    return;
}

#========================================================================================
# function     _getSitemapSubFileName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $FileName = _getSitemapSubFileName($IsMobile, $Index);
#----------------------------------------------------------------------------------------
# description  Returns the filename to use for a sub sitemap file, according to $Index
#----------------------------------------------------------------------------------------
# input        $IsMobile | Want the mobile filename? | boolean
# input        $Index | A number | Integer
#----------------------------------------------------------------------------------------
# return        $FileName | The generated filename, e.g. sitemap-1.xml/mobilesitemap-1.xml | string
#========================================================================================
sub _getSitemapSubFileName {
  my ($IsMobile, $Index) = @_;

  return $IsMobile ? "$SITEMAP_NAME_MOBILE-$Index.$SITEMAP_NAME_EXT" : "$SITEMAP_NAME-$Index.$SITEMAP_NAME_EXT";
}

#========================================================================================
# function     GetObjects
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aContentObjectData = GetObjects($Shop);
#----------------------------------------------------------------------------------------
# description  Returns all visible, enabled content objects
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# return       $ContentObjectsData | content object data | ref.array.hash
#========================================================================================
sub GetObjects {
    my ($Shop) = @_;

    my @ContentObjectHashs;
    my $maxEntries = MAX_ENTRIES;
    my $CountLocales = 0;

    my $hShopData = $Shop->get([ qw ( SiteLocales ProtocolAndServer WebServerScriptUrl DomainName LanguageID ) ]);

    # add the locale used for canonical url of these languages (first one by sort order) to a locale array
    my @LanguageData;
    my %ActiveLanguages;

    foreach my $Locale (@{$hShopData->{'SiteLocales'}}) {
        my $Language = $Locale->get('Language');
        my $LanguageID = $Language->{'LanguageID'};

        next if defined $ActiveLanguages{$LanguageID}; # e.g. iteration already did de_de and now does de_ch => only add once!
        next unless defined $Shop->activeLocaleByLanguage($LanguageID); #needed??

        $ActiveLanguages{$LanguageID} = 1;

        push(@LanguageData, {
            'LanguageID' => $LanguageID,
            'LocaleID' => $Locale->get('LocaleID'),
        });
        $CountLocales++
    }
    
    #-- Retrieve all content objects that should be exported for the sitemap
    #-- Products are excluded, handled by the hook below
    #-- Speed is important
    my $SqlParams = [
      $Shop->id,
      LoadClassByAlias('StartPage')->id,
      LoadClassByAlias('Article')->id,
      LoadClassByAlias('BlogPost')->id,
      LoadClassByAlias('BusinessHours')->id,
      LoadClassByAlias('Category')->id,
      LoadClassByAlias('CookiePolicy')->id,
      LoadClassByAlias('CustomForm')->id,
      LoadClassByAlias('CustomerInformation')->id,
      LoadClassByAlias('Forum')->id,
      LoadClassByAlias('ImageGallery')->id,
      LoadClassByAlias('Imprint')->id,
      LoadClassByAlias('PrivacyPolicy')->id,
      LoadClassByAlias('ShippingAndPaymentInformation')->id,
      LoadClassByAlias('SiteMap')->id,
      LoadClassByAlias('TermsAndConditions')->id,
    ];

    my $Sql = 'select contentobjectid, classid from contentobject co join object o on co.contentobjectid=o.objectid where co.siteid = ? and isvisible = 1 and classid in (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)';
    my $dbh = GetCurrentDBHandle();
    my $aaResults = $dbh->execute(
        $Sql,
        $SqlParams,
    );

    push(@ContentObjectHashs, {
      'ObjectID' => $_->[0],
      'ClassID' => $_->[1],
    }) foreach @$aaResults;
    #--------------
    #-------------------------------------------

    TriggerHook('API_GetObjectsForXMLSitemap', {
        'Object'             => $Shop,
        'ContentObjectHashs' => \@ContentObjectHashs,
        'MaxEntries'         => \$maxEntries,
        'ShopData'           => $hShopData,
        'CountLocales'       => $CountLocales,
    });

    #-- Get ALL short URLs for this shop, use it as a lookup table (FAST)
    my $ahAllResults = GetInfoShortURLsBySiteID($Shop->id);
    my $hURILookup;
    foreach my $hInfo ( @$ahAllResults ) {
      $hURILookup->{$hInfo->{LanguageID}}->{$hInfo->{ContentObjectID}} = $hInfo->{Uri};
    }

    # get only needed data (URLs) of the given objectids
    my @ContentObjectsData;
    my $StartPageClassID = LoadClassByAlias('StartPage')->id;

    foreach my $hContentObjectInfo (@ContentObjectHashs) {
        foreach my $hLang (@LanguageData) {
            my $Object = {
                %$hContentObjectInfo,
                'URL' => _getURLFromObjectData(
                    $hContentObjectInfo,
                    $hLang->{'LanguageID'},
                    $hShopData->{'DomainName'},
                    $hShopData->{'LanguageID'},
                    $hLang->{'LocaleID'},
                    $hShopData->{'ProtocolAndServer'},
                    $hShopData->{'WebServerScriptUrl'},
                    $StartPageClassID,
                    $hURILookup,
                 )
            };
            push(@ContentObjectsData, $Object);
        }
    }

    my @URLs = ();
    TriggerHook('API_GetURLsForXMLSitemap', {
        'Object'             => $Shop,
        'URLs'               => \@URLs,
        'LanguageIDs'        => $Shop->languages
    });

    push(@ContentObjectsData, $_) foreach (@URLs);

    return \@ContentObjectsData;
}

sub _getURLFromObjectData {
    my ($hContentObjectInfo, $LanguageID, $ShopDomainName, $ShopLanguageID, $LocaleString,
        $ProtocolAndServer, $WebServerScriptUrl, $StartPageClassID, $hURILookup) = @_;

    my $ContentObjectID = $hContentObjectInfo->{'ObjectID'};
    my $Uri;
    if (defined $ShopDomainName) {
        if ($hContentObjectInfo->{'ClassID'} == $StartPageClassID) {
            if ($ShopLanguageID != $LanguageID) {
                $Uri = $LocaleString;
            } else {
                $Uri = '';
            }
        } elsif (defined $LanguageID) {
          $Uri = $hURILookup->{$LanguageID}->{$ContentObjectID} if $hURILookup->{$LanguageID}->{$ContentObjectID};
        }
    }
    if (defined $Uri) {
        my $CanonicalURL = $ProtocolAndServer . '/';   # EPG-18932 : ending slash for shop domain
        $CanonicalURL = $CanonicalURL . $Uri if $Uri ne '';
        return $CanonicalURL;
    } else {
        return $WebServerScriptUrl . '/' . $LocaleString . '/?ObjectID=' . $ContentObjectID;
    }
}

1;
