#========================================================================================
# package      DE_EPAGES::ShippingTypes::API::Object::ShippingMethodProductsInBasket
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerShippingMethod
#----------------------------------------------------------------------------------------
# description  object interface for ShippingMethodProductsInBasket.
#========================================================================================
package DE_EPAGES::ShippingTypes::API::Object::ShippingMethodProductsInBasket;
use base DE_EPAGES::Order::API::Object::ContainerShippingMethod;

use strict;
use DE_EPAGES::Core::API::Error         qw ( Error );
use DE_EPAGES::Core::API::Log           qw ( LogDebug );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString TestArray );

sub ErrorDefinition { return {
    'ShippingMethodNoPrice'    => 'shipping method ( #ShippingMethodID ) has no price.',
}}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hValues) = @_;

    #--- get sum of quantities of products in shipping elements
    my $ShippingElements       = $hValues->{'ShippingElements'};
    my $Quantity = 0;
    foreach my $ShippingElement ( @$ShippingElements ) {
        my $Element = $ShippingElement->{'Element'};
        next unless $Element->instanceOf('Product');
        $Quantity += $ShippingElement->{'Quantity'};
    }

    return $self->_price($hValues->{'CurrencyID'}, $hValues->{'TaxModel'}, $Quantity, $hValues->{'TaxMatrix'});
}

#========================================================================================
# function     calculateShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->calculateShippingPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub calculateShippingPrice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    #--- get sum of product quantities
    my @ProductLineItems = grep {$_->instanceOf('LineItemProduct')} @{$LineItemContainer->get('LineItems')};
    my $Quantity;
    foreach my $ProductLineItem ( @ProductLineItems ) {
        $Quantity += $ProductLineItem->get('Quantity') unless $ProductLineItem->get('IsLineItemDownloadProduct');
    }

    # test if the shipping method has a free shipping price value
    my $hInfo = $LineItemContainer->get([qw (CurrencyID TaxModel TaxMatrix LineItemsSum)]);

    my $hShippingValues = $self->getShippingValues($hInfo->{'CurrencyID'}, $hInfo->{'TaxModel'});

    return 0 if defined($hShippingValues->{'FreeShippingPrice'}) &&
        $hShippingValues->{'FreeShippingPrice'} <= $hInfo->{'LineItemsSum'};

    return $self->_price($hInfo->{'CurrencyID'}, $hInfo->{'TaxModel'}, $Quantity, $hInfo->{'TaxMatrix'});
}

#========================================================================================
# function     _price
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->_price($CurrencyID, $TaxModel, $Quantity, $TaxMatrix);
#----------------------------------------------------------------------------------------
# description  Selects the corresponding shipping level and calculates the shipping
#               price for the given number of products.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency id | string(3)
# input        $TaxModel | tax model (0=net, 1=gross) | int
# input        $Quantity | number of products | float
# input        $TaxMatrix | tax matrix | object
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub _price {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $Quantity, $TaxMatrix) = @_;

    my $ahLevels = $self->getLevels($CurrencyID, $TaxModel, {'TaxMatrix' => $TaxMatrix});
    my $ShippingPrice;
    foreach my $hLevel (@$ahLevels) {
        if ($hLevel->{'LowerBound'} <= $Quantity && $Quantity < $hLevel->{'UpperBound'}) {
            $ShippingPrice = $hLevel->{'Multiplier'} * $Quantity + $hLevel->{'BaseValue'};
            last;
        }
    }

    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless defined $ShippingPrice;

    # shipping price should never be negative
    $ShippingPrice = 0 if $ShippingPrice < 0;
    return $ShippingPrice;
}

#========================================================================================
# function     getLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, $hParams);
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, {'ExtraPrecision' => 1});
#----------------------------------------------------------------------------------------
# description  Return the shipping levels for the given currency in the requested tax
#               model.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $hParams | (optional) <ul>
#                   <li>ExtraPrecision  | (optional, default=0) not used | boolean
#                   <li>TaxMatrix      - tax matrix - object
#               </ul> | ref.hash
# return       $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier, ShippingLevelID
#                           | ref.array.hash
#========================================================================================
sub getLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $hParams) = @_;

    my $Levels = $self->SUPER::getLevels(@_);

    LogDebug('getLevels = ',$Levels);

    my $TaxMatrix = $hParams->{'TaxMatrix'} // $self->getSite()->get('TaxMatrix');

    my $TaxClass = $self->get('TaxClass') // $TaxMatrix->defaultTaxClass;
    $TaxClass =  $TaxMatrix->getTaxClassByAlias($TaxClass->alias);

    foreach my $hLevel (@$Levels) {
        $hLevel->{'Multiplier'} = $TaxMatrix->net2gross($CurrencyID, $hLevel->{'Multiplier'}, {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxClass,
            'Precision' => 4, # round to 4 digits like in 5.04 to avoid extra digits (0.999984 instead of 1.0)
        });
    }
    return $Levels;
}

#========================================================================================
# function     setLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setLevels($CurrencyID, $TaxModel, $ahLevels);
#----------------------------------------------------------------------------------------
# description  Sets all shipping levels for the given currency.
#               Deletes shipping levels that are not included in the list.
#               Calculates net prices from gross prices if $TaxModel is TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier
#                           | ref.array.hash
#========================================================================================
sub setLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ahLevels) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    TestArray('ahLevels', $ahLevels);

    # convert net prices to gross
    my $TaxMatrix = $self->getSite()->get('TaxMatrix');
    my $TaxClass = $self->get('TaxClass');

    foreach my $hLevel (@$ahLevels) {
        $hLevel->{'Multiplier'} = $TaxMatrix->gross2net($CurrencyID, $hLevel->{'Multiplier'}, {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxClass,
            'Precision' => 9,
        });
    }

    $self->SUPER::setLevels(@_);
}

1;
