#========================================================================================
# package      DE_EPAGES::ShippingTypes::API::Object::ShippingMethodWeightOfBasket
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerShippingMethod
#----------------------------------------------------------------------------------------
# description  object interface for ShippingMethodWeightOfBasket.
#========================================================================================
package DE_EPAGES::ShippingTypes::API::Object::ShippingMethodWeightOfBasket;
use base DE_EPAGES::Order::API::Object::ContainerShippingMethod;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString TestArray );

sub ErrorDefinition {
    return {
        'ShippingMethodNoPrice'    => 'shipping method ( #ShippingMethodID ) has no price.',
    }
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Can = $ShippingMethod->canAddToBasket($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a shipping method can be added to the basket with the given currency.
#               Validates if the shipping methode has a weight limit. If yes, the weight of
#               the basket shouldn't exceed the limit.
#----------------------------------------------------------------------------------------
# input        $hVars | vars of container and basket | ref.hash
# return       $Can | can added to the basket | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return 0 unless $self->SUPER::canAddToBasket($hVars);

    my $LineItemContainer = $hVars->{'LineItemContainer'};

    # test if the shipping method has a weight limit
    if (defined $LineItemContainer) {
        my $ContainerCurrencyID = $LineItemContainer->get('CurrencyID');
        my $ContainerTaxModel = $LineItemContainer->get('TaxModel');
        my $hShippingValues = $self->getShippingValues($ContainerCurrencyID, $ContainerTaxModel);

        return 0 if defined($hShippingValues->{'MaximumWeight'})
            && $hShippingValues->{'MaximumWeight'} < $self->_weight($LineItemContainer);
    }

    return 1;
}

#========================================================================================
# function     price
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingPrice = $Object->price( $hVars );
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#                   <li>ShippingElements
#                       <ul>
#                          <li>Element   - e.g. Products       - object
#                          <li>Quantity  - quantity of element - float
#                      </ul> | ref.array.hash
#               </ul> | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub price {
    my $self = shift;
    my ($hValues) = @_;

    #--- get shipping expenses
    my $ShippingElements = $hValues->{'ShippingElements'};
    my $SumOfProductWeights = 0;
    foreach my $ShippingElement ( @$ShippingElements ) {
        my $Element = $ShippingElement->{'Element'};
        next unless $Element->instanceOf('Product');
        my $ProductWeight     = $Element->getWithParent('Weight');
        my $ProductWeightUnit = $Element->getWithParent('WeightUnit');
        my $PriceQuantity     = $Element->getWithParent('PriceQuantity');
        $PriceQuantity = 1 unless defined $PriceQuantity;
        if (defined $ProductWeight and defined $ProductWeightUnit) {
            #--- convert weights to gram and add weights for all line items
            if ($ProductWeightUnit->alias ne 'gram') {
                $ProductWeight *= $ProductWeightUnit->get('Factor');
            }
            $SumOfProductWeights += $ProductWeight * $ShippingElement->{'Quantity'} / $PriceQuantity;

        }
    }

    #--- convert weights into weight unit chosen by merchant
    my $ReferenceWeightUnit = $self->get('WeightUnit');
    if (defined $ReferenceWeightUnit and $ReferenceWeightUnit->alias ne 'gram') {
        $SumOfProductWeights /= $ReferenceWeightUnit->get('Factor');
    }

    return $self->_price($hValues->{'CurrencyID'}, $hValues->{'TaxModel'}, $SumOfProductWeights, $hValues->{'TaxMatrix'});
}

#========================================================================================
# function     calculateShippingPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->calculateShippingPrice($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping price by the given requirements.
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | ref.hash
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub calculateShippingPrice {
    my $self = shift;
    my ($LineItemContainer) = @_;

    # test if the shipping method has a free shipping price value
    my $hInfoContainer = $LineItemContainer->get([qw (CurrencyID TaxModel TaxMatrix LineItemsSum)]);
    my $hShippingValues = $self->getShippingValues($hInfoContainer->{'CurrencyID'}, $hInfoContainer->{'TaxModel'});

    return 0 if defined($hShippingValues->{'FreeShippingPrice'}) &&
        $hShippingValues->{'FreeShippingPrice'} <= $hInfoContainer->{'LineItemsSum'};

    return $self->_price(
        $hInfoContainer->{'CurrencyID'},$hInfoContainer->{'TaxModel'},
        $self->_weight($LineItemContainer),$hInfoContainer->{'TaxMatrix'}
    );;
}

#========================================================================================
# function     _price
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingPrice = $ShippingMethod->_price($CurrencyID, $TaxModel, $Weight);
#----------------------------------------------------------------------------------------
# description  Selects the corresponding shipping level and calculates the shipping
#               price for the given total weight.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency id | string(3)
# input        $TaxModel | tax model (0=net, 1=gross) | int
# input        $Weight | total weight of all products in the selected unit | float
# return       $ShippingPrice | price of shipping | money
#========================================================================================
sub _price {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $Weight, $TaxMatrix) = @_;

    my $ahLevels = $self->getLevels($CurrencyID, $TaxModel, {'TaxMatrix' => $TaxMatrix}  );
    my $ShippingPrice;

    foreach my $hLevel (@$ahLevels) {
        if ($hLevel->{'LowerBound'} <= $Weight && $Weight < $hLevel->{'UpperBound'}) {
            $ShippingPrice = $hLevel->{'Multiplier'} * $Weight + $hLevel->{'BaseValue'};
            last;
        }
    }

    Error('ShippingMethodNoPrice', { 'ShippingMethodID'=>$self->id }) unless defined $ShippingPrice;

    # shipping price should never be negative
    $ShippingPrice = 0 if $ShippingPrice < 0;
    return $ShippingPrice;
}

#========================================================================================
# function     _weight
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ShippingWeight = $ShippingMethod->_weight($Container);
#----------------------------------------------------------------------------------------
# description  Returns the shipping weight by the given requirements.
#----------------------------------------------------------------------------------------
# input        $Container | LineItemContainer | ref.hash
# return       $ShippingWeight | weight of shipping in gram | float
#========================================================================================
sub _weight {
    my $self = shift;
    my ($Container) = @_;

    my @ProductLineItems = grep {$_->instanceOf('LineItemProduct')} @{$Container->get('LineItems')};
    my $SumOfProductWeights = 0;
    foreach my $LineItemProduct (@ProductLineItems) {
        my $ProductWeight     = $LineItemProduct->get('Weight');
        my $ProductWeightUnit = $LineItemProduct->get('WeightUnit');
        my $WeightQuantity    = $LineItemProduct->get('WeightQuantity');

        # set the WeightQuantity to PriceQuantity if not defied (EPG-13158)
        unless ($WeightQuantity) {
            # get the value from original Product
            my $Product = $LineItemProduct->get('Product');
            $WeightQuantity = $Product->getWithParent('PriceQuantity') if defined $Product;
            $WeightQuantity = 1 unless $WeightQuantity;
            $LineItemProduct->set({'WeightQuantity' => $WeightQuantity });
        }

        if (defined $ProductWeight and defined $ProductWeightUnit) {
            #--- convert weights to gram and add weights for all line items
            if ($ProductWeightUnit->alias ne 'gram') {
                $ProductWeight *= $ProductWeightUnit->get('Factor');
            }
            $SumOfProductWeights += $ProductWeight * $LineItemProduct->get('Quantity') / $WeightQuantity;

        }
    }

    #--- convert weights into weight unit chosen by merchant
    my $ReferenceWeightUnit = $self->get('WeightUnit');
    if ($ReferenceWeightUnit->alias ne 'gram') {
        $SumOfProductWeights /= $ReferenceWeightUnit->get('Factor');
    }

    return $SumOfProductWeights;
}

#========================================================================================
# function     getLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, $hParams);
# syntax       $ahLevels = $Object->getLevels($CurrencyID, $TaxModel, {'ExtraPrecision' => 1});
#----------------------------------------------------------------------------------------
# description  Return the shipping levels for the given currency in the requested tax
#               model.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $hParams | (optional) <ul>
#                   <li>ExtraPrecision  | (optional, default=0) not used | boolean
#                   <li>TaxMatrix      - tax matrix - object
#               </ul> | ref.hash
# return       $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier, ShippingLevelID
#                           | int
#========================================================================================
sub getLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $hParams) = @_;

    my $Levels = $self->SUPER::getLevels($CurrencyID, $TaxModel, $hParams);

    LogDebug('getLevels = ',$Levels);

    my $TaxMatrix = $hParams->{'TaxMatrix'} // $self->getSite()->get('TaxMatrix');

    my $TaxClass = $self->get('TaxClass') // $TaxMatrix->defaultTaxClass;
    $TaxClass =  $TaxMatrix->getTaxClassByAlias($TaxClass->alias);

    foreach my $hLevel (@$Levels) {
        $hLevel->{'Multiplier'} = $TaxMatrix->net2gross($CurrencyID, $hLevel->{'Multiplier'}, {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxClass,
            'Precision' => 4, # round to 4 digits like in 5.04 to avoid extra digits (0.999984 instead of 1.0)
        });
    }
    return $Levels;
}

#========================================================================================
# function     setLevels
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->setLevels($CurrencyID, $TaxModel, $ahLevels);
#----------------------------------------------------------------------------------------
# description  Sets all shipping levels for the given currency.
#               Deletes shipping levels that are not included in the list.
#               Calculates net prices from gross prices if $TaxModel is TAX_MODEL_GROSS.
#----------------------------------------------------------------------------------------
# input        $CurrencyID | currency  id | char(3)
# input        $TaxModel | tax model (1=gross/including tax, 0=net/without tax) | int
# input        $ahLevels | list of hashes, keys:
#                           BaseValue, LowerBound, UpperBound, Multiplier
#                           | int
#========================================================================================
sub setLevels {
    my $self = shift;
    my ($CurrencyID, $TaxModel, $ahLevels) = @_;
    TestString('CurrencyID', $CurrencyID);
    TestInteger('TaxModel', $TaxModel);
    TestArray('ahLevels', $ahLevels);

    # convert net prices to gross
    my $TaxMatrix = $self->getSite()->get('TaxMatrix');
    my $TaxClass = $self->get('TaxClass');

    foreach my $hLevel (@$ahLevels) {
        $hLevel->{'Multiplier'} = $TaxMatrix->gross2net($CurrencyID, $hLevel->{'Multiplier'}, {
            'TaxModel' => $TaxModel,
            'TaxClass' => $TaxClass,
            'Precision' => 9,
        });
    }

    $self->SUPER::setLevels($CurrencyID, $TaxModel, $ahLevels);
}

1;
