#========================================================================================
# §package      DE_EPAGES::Shop::Hooks::ShopNotification
# §state        public
# §description  hook functionality for events on a ShopNotification
#========================================================================================
package DE_EPAGES::Shop::Hooks::ShopNotification;

use strict;

use DateTime::Duration;
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Shop::API::Constants qw ( $FEEDBACK_BANNER_DISPLAY_PARAMETERS );

#========================================================================================
# §function     OnBeforeUpdateShopNotification
#----------------------------------------------------------------------------------------
# §syntax       OnBeforeUpdateShopNotification( $hParams );
#----------------------------------------------------------------------------------------
# §description  special FeedBack banner handling for displaying the banner again
#               dependent of the current date
#               (Hook OBJ_BeforeUpdateShopNotification)
#----------------------------------------------------------------------------------------
# §input        $hParams | hook params
#               <ul>
#                 <li>Object - ShopNotification - object
#                 <li>ShopNotificationID        - object id - int
#                 <li>Values - new values for object - ref.hash
#               </ul>
#========================================================================================
sub OnBeforeUpdateShopNotification {
    my ($hParams) = @_;

    my $ShopNotification = $hParams->{'Object'};
    my $hValuesToUpdate = $hParams->{'Values'};
    
    if($ShopNotification->alias eq 'FeedbackBanner'){
        my $Shop = $ShopNotification->getSite();
        return unless $Shop->featureMaxValue('FeedbackBanner');
        # we close the dialog...
        if ((exists $hValuesToUpdate->{'IsActive'}) and ($hValuesToUpdate->{'IsActive'} == 0) ){
            # set attributes for FeedbackBanner dependent on the activation date of the shop
            # act here only if we press the close button and try to set the attribute IsActive to 0
            my $ShopCreationNormDay = $Shop->get('CreationDate')->clone->truncate( to => 'day' );
            my $Duration = DateTime::Duration->new( days  => $FEEDBACK_BANNER_DISPLAY_PARAMETERS->{'BANNER_CLOSE_THRESHOLD'} );
            my $NotificationReleaseDate = $ShopCreationNormDay + $Duration;
            my $NowNormDay = GetCurrentDBHandle()->currentDateTime->clone->truncate( to => 'day' );
            my $AttrIsTempInActiveDate = $ShopNotification->get('IsTemporaryInActiveOn');
            my $TempInActiveDate;

            if($AttrIsTempInActiveDate){
                # closed permanently: IsTemporaryInActive= 1 ? and IsActive = 0 then IsTemporaryInActive = 0
                $TempInActiveDate = undef;
            } else {
                # shop register date <= 20d then IsTemporaryInActiveOn= CurrentDateTime (display again after 10d)
                # shop register date > 20d then IsTemporaryInActiveOn= undef  (never display again)
                $TempInActiveDate = $NowNormDay >= $NotificationReleaseDate ? undef : GetCurrentDBHandle()->currentDateTime;
            }

            $ShopNotification->set({'IsTemporaryInActiveOn' => $TempInActiveDate});
        }
    }

    return;
}

#========================================================================================
# §function     OnMBOViewMBO
#----------------------------------------------------------------------------------------
# §syntax       OnMBOViewMBO( $hParams );
#----------------------------------------------------------------------------------------
# §description  Triggered by the hook UI_MBOViewMBO.
#               Check for Unity shops if a ShopNotification can be displayed
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet object | object</li>
#                   <li>Shop - shop object | object</li>
#                   <li>Form - form object | object</li>
#                   <li>Values - value hash | hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnMBOViewMBO {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Shop'};
    if ($Shop->get('IsUnityRegistered') and
        $Shop->featureMaxValue('FeedbackBanner') and
        $Shop->folder('ShopNotifications')->existsChild('FeedbackBanner')) {
        my $ShopNotification = $Shop->folder('ShopNotifications')->child('FeedbackBanner');
        # only if not already active
        my $IsTempInactiveDate = $ShopNotification->get('IsTemporaryInActiveOn');
        if ((!$ShopNotification->get('IsActive')) and $IsTempInactiveDate){
            # test if we can show the FeedbackBanner again
            # dependent on the attribute IsTemporaryInActiveOn and the creation date of the shop
            # (switch to active)
            my $IsTempInactiveDateNormDay = $IsTempInactiveDate->clone->truncate( to => 'day' );
            my $Duration = DateTime::Duration->new( days  => $FEEDBACK_BANNER_DISPLAY_PARAMETERS->{'BANNER_SHOW_AGAIN_DAYS'} );
            my $NotificationShowDate = $IsTempInactiveDateNormDay + $Duration;
            my $NowNormDay = GetCurrentDBHandle()->currentDateTime->clone->truncate( to => 'day' );

            if($NowNormDay >= $NotificationShowDate){
                $ShopNotification->set({'IsActive'=>1});
            }
        }
    }
}

1;
