#========================================================================================
# package      DE_EPAGES::Tax::API::Cartridge
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Tax::API::Cartridge;
use base DE_EPAGES::Presentation::API::PresentationInstaller;

use strict;
use DE_EPAGES::Core::API::Error qw(GetError ExistsError);
use DE_EPAGES::Core::API::File qw( RunOnGlob );
use DE_EPAGES::Core::API::Log qw(LogError GetLog);
use DE_EPAGES::Address::API::Address qw(CompareZipCodes);
use DE_EPAGES::Object::API::Factory qw( Transaction LoadObject LoadObjectByPath LoadRootObject LoadClassByAlias);
use DE_EPAGES::Object::API::Table::ReferenceAttribute qw ( GetPKeyReferenceAttributesByReferencedObjectID );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw (DeleteObjects);
use DE_EPAGES::XML::API::TestSchema qw ( TestSchema );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Tax::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Tax.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                   cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw(ProhibitLongLines)
sub new {
    my ($class, %options) = @_;
    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory'=> 'DE_EPAGES/Tax',
        'Version'            => '7.94.0',
        'Patches'           => ['7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0','7.85.0','7.86.0','7.87.0','7.88.0','7.89.0','7.90.0','7.91.0','7.92.0','7.93.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;
    $self->startInstall;
    $self->SUPER::install;

    Transaction( sub {
        my $XMLDir = $self->xmlDirectory;
        RunOnGlob($XMLDir, 'Taxes*.xml', sub { ImportObjects( $_, '/' ) });
        ImportObjects( "$XMLDir/TaxMatrixDigital.xml" );
        ImportObjects( "$XMLDir/DefaultTaxMatrix.xml" );
        ImportObjects( "$XMLDir/DefaultTaxMatrixDigital.xml" );
        ImportObjects( "$XMLDir/TaxTypes.xml" );
        ImportTranslation( "$XMLDir/Translation.TaxAreasDigital" );
    });

    $self->finishInstall;
    return;
}

#========================================================================================
# function     test
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->test;
#----------------------------------------------------------------------------------------
# description  runs xml schema validation for Taxes
#========================================================================================
sub test {
    my $self = shift;
    $self->SUPER::test();
    my $XMLDir = $self->xmlDirectory;
    my $XSDDir = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Tax/Data/XSD/';
    TestSchema($XMLDir, 'Taxes*.xml', [$XSDDir.'Taxes.xsd', $XSDDir.'v6/Taxes.xsd'] );
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;
    $self->startUninstall;

    my $XMLDir = $self->xmlDirectory;
    RunOnGlob($XMLDir, 'Taxes*.xml', sub { DeleteObjects( $_ ) }, {reverse=>1, force=>$self->force});
    DeleteObjects("$XMLDir/TaxTypes.xml");

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patch_7_54_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patch_7_54_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge  in active store (database connection)
#               from version '7.54.0' to version '7.55.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '7.55.0' | string
#========================================================================================
sub patch_7_54_0 {
    my $self = shift;

    _setupTaxMatrixForStayingNIinTheEU();
    _setupTaxMatrixGreatBritain();
    $self->importPatchDir('Patch_7_54_0', 'Patch');
    return '7.55.0';
}

#========================================================================================
# function         _setupTaxMatrixForStayingNIinTheEU
# state            private
#----------------------------------------------------------------------------------------
# syntax           _setupTaxMatrixForStayingNIinTheEU();
#----------------------------------------------------------------------------------------
# description      Change Alias and CountryID for DigitalTaxMatrix UnitedKingdom to
#                   UnitedKingdomEU (Alias) and 000 (CountryID).
#========================================================================================
sub _setupTaxMatrixForStayingNIinTheEU {

    GetLog()->info( '_setupTaxMatrixForStayingNIinTheEU' );
    my $System = LoadRootObject();
    my $Cartridge = DE_EPAGES::Tax::API::Cartridge->new;

    # import new Matrix TaxMatrixNorthernIreland (NI) if not already exists
    unless ($System->existsChild('TaxMatrixNorthernIreland')){
        ImportObjects($Cartridge->xmlDirectory().'/TaxesNorthernIreland.xml');
    }

    my $TaxMatrixAttributeID = LoadClassByAlias('Shop')->attribute('TaxMatrix')->id;
    my $NITaxMatrix = $System->child('TaxMatrixNorthernIreland');
    my $UKTaxMatrixID = $System->child('TaxMatrixGreatBritain')->id;
    my $UKCountryID = '826';
    my $ZipFrom = 'BT01';
    my $ZipTo   = 'BT94';

    #get all shops with UK system taxmatrix
    my $ahReferenceAttributesObjects = GetPKeyReferenceAttributesByReferencedObjectID($UKTaxMatrixID);

    # assign TaxMatrixNorthernIreland to NI shops (TaxMatrixGreatBritain and having zip code ranges BT01 - BT99)
    eval {
        foreach my $hInfo (@$ahReferenceAttributesObjects) {
            next if $hInfo->{'AttributeID'} != $TaxMatrixAttributeID;

            my $Shop = LoadObject($hInfo->{'ObjectID'});
            # next if shop has not the UK tax matrix from the system
            # (ignore adapted tax matrix and other)
            next unless $Shop->get('TaxMatrix')->id eq $UKTaxMatrixID;

            # find out if we have an NI shop with the correct countryid ant the zip code range
            my $ShopAddress = $Shop->get('Address');
            next unless defined $ShopAddress;

            next unless $ShopAddress->get('CountryID') eq $UKCountryID;
            my $ZipCode = $ShopAddress->get('Zipcode');

            next unless $ZipCode;

            if ((CompareZipCodes($ZipFrom, $ZipCode) != 1) and
                (CompareZipCodes($ZipTo, $ZipCode) != -1)){
                GetLog()->info( 'set Shop: ' . $Shop->alias . ' to new TaxMatrix: TaxMatrixNorthernIreland');
                $Shop->set({'TaxMatrix' => $NITaxMatrix});
            }
        }
    };
    LogError(GetError()) if ExistsError();
    GetLog()->info( '_setupTaxMatrixForStayingNIinTheEU done' );
    return;
}


sub _setupTaxMatrixGreatBritain {

    my $FilesDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Tax/Scripts/Brexit_files";

    _setupTaxMatrixDigitalForBrexit($FilesDir);
    _setupTaxMatrixGreatBritainForBrexit($FilesDir);
}

# Change Alias and CountryID for DigitalTaxMatrix UnitedKingdom to
# UnitedKingdomEU (Alias) and 000 (CountryID).
sub _setupTaxMatrixDigitalForBrexit {
    my ($FilesDir) = @_;

    GetLog()->info( '_setupTaxMatrixDigitalForBrexit' );
    my $TaxMatrixUK = LoadObjectByPath('/TaxMatrixDigital');

    if ($TaxMatrixUK->existsChild('UnitedKingdomEU')) {
        GetLog()->info( 'TaxMatrixDigital UnitedKingdomEU already exists' );
        return;
    }

    eval {
        GetLog()->info( 'Rename TaxMatrixDigital/UnitedKingdom to TaxMatrixDigital/UnitedKingdomEU and set new Country' );
        ImportObjects($FilesDir.'/changeTaxMatrixDigitalUK.xml');
    };
    LogError(GetError()) if ExistsError();
    GetLog()->info( '_setupTaxMatrixDigitalForBrexit done' );

    return;
}

# Change Alias for EU and non-EU Taxareas for TaxMatrixGreatBritain to
# United Kingdom and abroad.
sub _setupTaxMatrixGreatBritainForBrexit {
    my ($FilesDir) = @_;

    GetLog()->info( '_setupTaxMatrixGreatBritainForBrexit' );
    eval {
        my $TaxMatrixUK = LoadObjectByPath('/TaxMatrixGreatBritain');

        if ($TaxMatrixUK->existsChild('abroad')) {
            GetLog()->info( 'TaxArea abroad already exists for TaxMatrixGreatBritain' );
            return;
        }
        GetLog()->info( 'Rename TaxArea EU to United Kingdom and non-EU to abroad' );
        ImportObjects($FilesDir.'/renameTaxAreasGreatBritain.xml');

        GetLog()->info( 'remove EU countries from United Kingdom TaxArea in TaxMatrixGreatBritain' );
        DeleteObjects($FilesDir.'/removeGreatBritainfromEU.xml');

        GetLog()->info( 'Import EU exceptions for Northern Ireland' );
        ImportObjects($FilesDir.'/importTaxAreaExeptionsNorthIreland.xml');

    };
    LogError(GetError()) if ExistsError();
    GetLog()->info( '_setupTaxMatrixGreatBritainForBrexit done' );

    return;
}

1;
