#========================================================================================
# package      DE_EPAGES::Zettle::API::ImageHelper
# state        public
#----------------------------------------------------------------------------------------
# description  class that implements methods related to image handling
#========================================================================================
package DE_EPAGES::Zettle::API::ImageHelper;
use base qw ( Exporter );

use strict;

our @EXPORT_OK = qw (
    GetGuidImageMap
    GetImageUploadResults
    GetImageUrls
);

use DE_EPAGES::Zettle::API::Object::ImageUploadResult;

use URI;

# Message to be shown when processing image upload results for image URLs which failed to be uploaded to Zettle
# (those URLs retrieved from 'invalidUploads' field from BulkImageUploadResponseDTO objects)
use constant MESSAGE_INVALID_IMAGE => 'Zettle Image API rejected URL';


#========================================================================================
# function     GetGuidImageMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hImageUrls = GetGuidImageMap($aProducts, $ImageFormat);
#----------------------------------------------------------------------------------------
# description  Get all image URLs for given products.
#               See <DE_EPAGES::Zettle::API::WebService:ImageService>.
#----------------------------------------------------------------------------------------
# input        $aProducts | Products to be exported | ref.array.object
# input        $ImageFormat | optional. name of the image format to be processed,
#                              any of: ImageLarge, ImageMedium, ImageSmall, ImageThumbnail
#                              If argument not present, all product images will be exported | string
#----------------------------------------------------------------------------------------
# return       $hImageUrls | a map of image URLs indexed by product GUID | ref.hash.array
#========================================================================================
sub GetGuidImageMap {
    my ($aProducts, $ImageFormat) = @_;

    my $hImageUrls = {};
    foreach my $Product (@{$aProducts}){
        # Variation images are not exported to Zettle
        next if defined $Product->superProduct();

        my $ProductGuid = $Product->guid;
        my $aImageUrls = GetImageUrls($Product, $ImageFormat);
        $hImageUrls->{$ProductGuid} = $aImageUrls if defined $aImageUrls && scalar @{$aImageUrls};
    }

    return $hImageUrls;
}

#========================================================================================
# function     GetImageUploadResults
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hImageUploadResults = GetImageUploadResults($BulkImageUploadResponseDTO, $hImageUrls);
#----------------------------------------------------------------------------------------
# description  Processes $BulkImageUploadResponseDTO and $hImageUrls to generate a structure containing image upload results.
#----------------------------------------------------------------------------------------
# input        $BulkImageUploadResponseDTO | the response retrieved after a bulk upload of images.
#                                             Image lookupKeys for products to be exported will be searched here.
#                                           | DE_EPAGES::Zettle::API::WebService::DTO::BulkImageUploadResponseDTO
# input        $hImageUrls | Map of image URLs indexed by product GUID | ref.hash
#----------------------------------------------------------------------------------------
# return       $hImageUploadResults | a map of ImageUploadResult objects indexed by product GUID.
#                                      See <DE_EPAGES::Zettle::API::Object::ImageUploadResult> | ref.hash.array.object
#========================================================================================
sub GetImageUploadResults {
    my ($BulkImageUploadResponseDTO, $hImageUrls) = @_;

    # We reverse hImageUrls so that keys are ImageUrls and values are product GUIDs to perform a URL-based search below
    my $hReverseImageUrls = _ReverseImageUrls($hImageUrls);

    my $hImageUploadResults = {};
    my $ImageUploadResult;

    # STEP 1 - Process images succesfully uploaded to Zettle
    # CAUTION: beware that some Zettle URLs might still fail because Zettle creates lookupKeys before actually uploading images to Zettle site
    my $aUploadResults = $BulkImageUploadResponseDTO->imageUploadResults();
    foreach my $BulkImageUploadItemDTO (@{$aUploadResults}){
        my $LookupKey = $BulkImageUploadItemDTO->imageLookupKey();
        # Zettle documentation states: "use the first URL provided in the imageUrls array"
        # (see https://github.com/iZettle/api-documentation/blob/master/image.md)
        my $ZettleUrl = @{$BulkImageUploadItemDTO->imageUrls()}[0];
        my $ShopProductImageUrl = $BulkImageUploadItemDTO->source();
        my $ProductGuid = $hReverseImageUrls->{$ShopProductImageUrl};

        $ImageUploadResult = DE_EPAGES::Zettle::API::Object::ImageUploadResult->new();
        $ImageUploadResult->epagesUrl($ShopProductImageUrl);
        $ImageUploadResult->lookupKey($LookupKey);
        $ImageUploadResult->zettleUrl($ZettleUrl);
        $hImageUploadResults->{$ProductGuid} = () unless defined $hImageUploadResults->{$ProductGuid};
        push(@{ $hImageUploadResults->{$ProductGuid} }, $ImageUploadResult);
    }

    # STEP 2 - Process images that Zettle failed to upload
    my $aInvalidUploads = $BulkImageUploadResponseDTO->invalidUploads();
    foreach my $InvalidProductUrl (@{$aInvalidUploads}){
        my $ProductGuid = $hReverseImageUrls->{$InvalidProductUrl};

        $ImageUploadResult = DE_EPAGES::Zettle::API::Object::ImageUploadResult->new();
        $ImageUploadResult->epagesUrl($InvalidProductUrl);
        $ImageUploadResult->error(MESSAGE_INVALID_IMAGE);
        $hImageUploadResults->{$ProductGuid} = () unless defined $hImageUploadResults->{$ProductGuid};
        push(@{ $hImageUploadResults->{$ProductGuid} }, $ImageUploadResult);
    }

    return $hImageUploadResults;
}

#========================================================================================
# function     GetImageUrls
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aImageUrls = GetImageUrls($Product, $ImageFormat);
#----------------------------------------------------------------------------------------
# description  Returns a list of image URLs of the product.
#               If ImageFormat is provided a single-element array is generated.
#               If ImageFormat is not provided a list containing URLs of all product images is generated.
#----------------------------------------------------------------------------------------
# input        $Product | product | DE_EPAGES::Product::API::Object::Product
# input        $ImageFormat | optional. name of the image format to be processed,
#                              any of: ImageLarge, ImageMedium, ImageSmall, ImageThumbnail
#                              If argument not present, all product images will be exported | string
#----------------------------------------------------------------------------------------
# return       $aImageUrls | a list of image URLs | ref.array.string
#========================================================================================
sub GetImageUrls {
    my ($Product, $ImageFormat) = @_;

    my @aImageUrls = ();
    if(defined $ImageFormat){
        my $ImageName = $Product->get($ImageFormat);
        if(defined $ImageName){
            push(@aImageUrls, _BuildImageUrl($Product, $ImageName));
        }
    }else{
        foreach my $ImageName ( map { $_->{'ImageName'} } @{$Product->get('ImagesSlideShow')} ){
            push(@aImageUrls, _BuildImageUrl($Product, $ImageName));
        }
    }

    return \@aImageUrls;
}

#========================================================================================
# function     _BuildImageUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ImageUrl = _BuildImageUrl($ProductUrlBase, $ImageName);
#----------------------------------------------------------------------------------------
# description  Build url to image
#               also handles case where image is already url, in that case also
#               urlencodes the parts of the url as that is what zettle wants
#----------------------------------------------------------------------------------------
# input        $Product | string | ProductUrlBase (PublicPath)
# input        $ImageName | string | Image name (asdf.jpg)
#----------------------------------------------------------------------------------------
# return       $UrlToImage | url to image
#========================================================================================
sub _BuildImageUrl {
    my ($Product, $ImageName) = @_;

    return $ImageName if(!$ImageName);

    my $Shop = $Product->getSite();

    # if Image is url just fix it and pass back
    if($ImageName =~ /^\w+\:\/\//) {
        my $uri = URI->new($ImageName);
        return $uri->as_string();
    }

    my $Path = join('/', $Product->get('WebPath'), $ImageName);
    my $uri = URI->new($Shop->get('ProtocolAndServerSSL'));
    $uri->path($Path);
    return $uri->as_string();
}


#========================================================================================
# function     _ReverseImageUrls
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hReverseImageUrls = _ReverseImageUrls($hImageUrls);
#----------------------------------------------------------------------------------------
# description  Returns a "reverse" map.
#----------------------------------------------------------------------------------------
# input        $hImageUrls | map of image URLs where keys are product GUIDs and values are lists of URLs | ref.hash.array.string
#----------------------------------------------------------------------------------------
# return       $hReverseImageUrls | map of image URLs where keys are image URLs and values are product GUIDs | ref.hash.string
#========================================================================================
sub _ReverseImageUrls {
    my ($hImageUrls) = @_;

    my $hReverseImageUrls = {};
    foreach my $ProductGuid (keys %{$hImageUrls}){
        foreach my $ImageUrl ( @{ $hImageUrls->{$ProductGuid} } ){
            $hReverseImageUrls->{$ImageUrl} = $ProductGuid;
        }
    }

    return $hReverseImageUrls;
}

1;
