#========================================================================================
# package      DE_EPAGES::Zettle::API::Logger
# state        public
#----------------------------------------------------------------------------------------
# description  Wrapper of LogContext DE_EPAGES::Zettle::API::Logger
#========================================================================================
package DE_EPAGES::Zettle::API::Logger;

use strict;

use Log::Log4perl::NDC;

use DE_EPAGES::Core::API::Log qw ( LogContext LogCommunication );
use DE_EPAGES::Core::API::PerlTools qw ( IsA );
use DE_EPAGES::Core::API::TestParameter  qw( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Zettle::API::Constants qw( INTERNAL_CARTRIDGE_NAME ZETTLE_OBJECT_ALIAS );

# Method used to filter sensitive information out of the logs
my $Filter = sub {
    my $LogString = shift;
    # filter client_secret
    $$LogString =~s/(?<='client_secret' => ')[^']*?(?=')/---replaced---/g;
    $$LogString =~s/(?<=client_secret=)[-a-zA-Z0-9]*?(?=(&|$|'|[[:space:]]))/---replaced---/g;
    # filter refresh_token
    $$LogString =~s/(?<='refresh_token' => ')[^']*?(?=')/---replaced---/g;
    $$LogString =~s/(?<=refresh_token=)[-a-zA-Z0-9]*?(?=(&|$|[[:space:]]))/---replaced---/g;
    $$LogString =~s/(?<="refresh_token":")[^"]*?(?=")/---replaced---/g;
};

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::Logger->new($Shop);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::Logger
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop)  = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $self  = bless {
        'Shop' => $Shop
    }, $class;

    return $self;
}

#========================================================================================
# function     logCommunication
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->logCommunication($Name, $Structure);
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $Structure to the log file if logging is activated.
#               See <function DE_EPAGES::Core::API::Log::LogCommunication>.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $Structure | the object | scalar (usually hash or array ref)
#========================================================================================
sub logCommunication {
    my $self = shift;
    my ($Name, $Structure) = @_;

    my $Category = INTERNAL_CARTRIDGE_NAME;
    my $CallerDepth = 1;
    my $DoLogging = 0;
    my $MaxDepth;
    
    if($self->_isInitialExportCall()){
        if($self->_isInitialExportLoggingActivated()){
            $DoLogging = 1;
            # Do not limit dumper depth for initial export logging (otherwise outbound export request is not entirely logged)
            $MaxDepth = 0;
        }
    }
    elsif($self->_isLoggingActivated()){
        $DoLogging = 1;
    }

    return unless $DoLogging;

    my $PrettyStructure = $self->_prettify($Structure);

    if ($self->_isLogContextEmpty()) {
        # Add logging context if current context is empty to avoid:
        # Context : [undef]
        my $Shop = $self->{'Shop'};
        my $EnvironmentId = $Shop->existsChild(ZETTLE_OBJECT_ALIAS) ? $Shop->child(ZETTLE_OBJECT_ALIAS)->id : '[undef]';
        LogContext(
            {
                'Zettle Environment ID' => $EnvironmentId,
                'ShopID'     => $Shop->id,
                'ShopAlias'  => $Shop->alias,
            },
            sub {
                LogCommunication($Name, $PrettyStructure, $Category, $Filter, $CallerDepth, $MaxDepth);
            }
        );
    } else {
        LogCommunication($Name, $PrettyStructure, $Category, $Filter, $CallerDepth, $MaxDepth);
    }

    return;
}


#========================================================================================


#========================================================================================
sub _isLoggingActivated {
    my $self = shift;
    return LoadRootObject()->get('Zettle_Logging');
}

#========================================================================================
sub _isInitialExportLoggingActivated {
    my $self = shift;
    return LoadRootObject()->get('Zettle_LoggingInitialExport');
}

#========================================================================================
sub _prettify {
    my $self = shift;
    my ($Structure) = @_;

    my $PrettyStructure;
    if( IsA($Structure, 'HTTP::Response') ) {
        $PrettyStructure = $self->_prettyResponse($Structure);
    } elsif (ref $Structure eq 'HASH'
            && exists($Structure->{'Request'})
            && exists($Structure->{'Response'})) {
        # Clone hash (shallow copy) to avoid messing with original structure
        $PrettyStructure = { %$Structure };
        # Remove request data from response error (not required since CorrelationID is present)
        delete($PrettyStructure->{'Request'});
    } else {
        $PrettyStructure = $Structure;
    }

    return $PrettyStructure;
}

#========================================================================================
sub _prettyResponse {
    my $self = shift;
    my ($Response) = @_;
    # Reference: https://metacpan.org/pod/HTTP::Message#$mess-%3Edump(-%opt-)
    return sprintf ("\n%s \n", $Response->dump( maxlength => 0 ));
}

#========================================================================================
# function     _isInitialExportCall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isInitialExportCall()
#----------------------------------------------------------------------------------------
# description  Search a call to ZettleService::initialExport in stacktrace.
#----------------------------------------------------------------------------------------
# return        TRUE if initialExport() call is found in stacktrace, false otherwise
#========================================================================================
sub _isInitialExportCall {
    my $self = shift;

    my $Depth = 0;
    my ($package, $filename, $line, $subroutine) = caller($Depth);
    while(defined $subroutine){
        # Check for: subroutine=DE_EPAGES::Zettle::API::ZettleService::initialExport
        return 1 if $subroutine =~ /ZettleService::initialExport/;

        # Stop if we get to a stack entry outside of Zettle package
        return 0 if $package !~ /DE_EPAGES::Zettle/;

        $Depth += 1;
        ($package, $filename, $line, $subroutine) = caller($Depth);
    }
    return 0;
}

#========================================================================================
# function     _isLogContextEmpty
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsContextEmpty = $self->_isLogContextEmpty()
#----------------------------------------------------------------------------------------
# description  Verify whether log context is empty.
#----------------------------------------------------------------------------------------
# return       $IsContextEmpty | TRUE if log context is empty | bool
#========================================================================================
sub _isLogContextEmpty {
    # Reference: https://metacpan.org/pod/Log::Log4perl::NDC#my-$text-=-Log::Log4perl::NDC-%3Eget();
    return Log::Log4perl::NDC->get() eq '[undef]';
}

1;
