#========================================================================================
# package      DE_EPAGES::Zettle::API::Mail
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides mail functions for Zettle.
#========================================================================================
package DE_EPAGES::Zettle::API::Mail;
use base Exporter;

use strict;

our @EXPORT_OK = qw ( NotifyRefreshTokenIssue );

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use constant MAILTYPE_ZETTLE_TOKEN_EXPIRED => 'ZettleTokenExpiration';

#========================================================================================
# function     _GetNotificationEmail
# state        private
#----------------------------------------------------------------------------------------
# syntax       $NotificationEmail = _GetNotificationEmail($Environment);
#----------------------------------------------------------------------------------------
# description  Retrieve email where Zettle-related notifications will be sent out to.
#               Use shop admin email, fallback to shop email or, as a last resort,
#               fallback to merchant email registered in Zettle.
#----------------------------------------------------------------------------------------
# return       $NotificationEmail | Email address where notifications will be sent out to | string
#========================================================================================
sub _GetNotificationEmail {
    my ($Environment) = @_;

    my $Shop = $Environment->getSite();

    my $Admin = $Shop->get('SuperMerchant');

    return $Admin->get('EMail') // $Shop->get('MailFrom') // $Environment->getMerchantEmail();
}

#========================================================================================
# function     _GetRenewTokenLink
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RenewTokenLink = _GetRenewTokenLink($Environment);
#----------------------------------------------------------------------------------------
# description  Retrieve shop url to Zettle page in MBO where refresh token can be "renewed".
#----------------------------------------------------------------------------------------
# return       $RenewTokenLink | Shop URL to Zettle page | string
#========================================================================================
sub _GetRenewTokenLink {
    my ($Environment) = @_;

    my $Shop = $Environment->getSite();
    my $ViewAction = 'MBO-ViewZettleGeneral';
    if ($Shop->isUnity()) {
        $ViewAction = 'Unity'.$ViewAction;
    }

    my $RenewTokenLink = BuildShopUrl($Environment, {'ViewAction' => $ViewAction }, {'Type' => 'admin' , 'UseSSL' => 1 });

    return $RenewTokenLink;
}

#========================================================================================
# function     NotifyRefreshTokenIssue
# state        public
#----------------------------------------------------------------------------------------
# syntax       NotifyRefreshTokenIssue($Environment, $DaysToExpiration);
#----------------------------------------------------------------------------------------
# description  Send off email to merchant notifying an issue with refresh token.
#               Destination email address is obtained with <function _GetMerchantMailForNotification>.
#               Shop language is used for email subject and contents.
#----------------------------------------------------------------------------------------
# input        $Environment | ZettleEnvironment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
# input        $DaysToExpiration | Number of days till token expiration (message is based on this value) | int
#----------------------------------------------------------------------------------------
# return       $hNotificationInfo | Notification information with following keys:
#               <ul>
#                   <li>IsNotified - TRUE if notification was sent to merchant, FALSE otherwise - boolean
#                   <li>NotificationEmail - Optional. Email address where notification was sent to - string
#               </ul> | ref.hash
#========================================================================================
sub NotifyRefreshTokenIssue {
    my ($Environment, $DaysToExpiration) = @_;
    
    TestObject('ZettleEnvironment', $Environment, 'DE_EPAGES::Zettle::API::Object::ZettleEnvironment');
    TestInteger('DaysToExpiration', $DaysToExpiration);

    my $Shop = $Environment->getSite();
    GetLog->debug('WARNING: refresh token issue in shop '.$Shop->alias.': DaysToExpiration='.$DaysToExpiration);

    my $System = LoadRootObject();
    my $MailType = $System->folder('MailTypes')->child(MAILTYPE_ZETTLE_TOKEN_EXPIRED);
    return unless $MailType->get('IsActive');

    # Retrieve email where notification will be sent to
    my $NotificationEmail = _GetNotificationEmail($Environment);
    # Apply same logic as UNITY-6650: Use site email address as sender to get through spam filters
    # if site email address is not valid fallback to notification email and hope it gets through
    my $FromEmail = $Shop->getSiteEmailAddress() // $NotificationEmail;
    my $ErrorCode = $DaysToExpiration <= 0 ? 'TokenExpired' : 'TokenWarning';
    # Retrieve URL where merchant can "renew" Zettle connection
    my $RenewTokenLink = _GetRenewTokenLink($Environment);

    #TODO: better off with super-merchant language?
    my $LocaleID = $Shop->get('LocaleID');
    my $LanguageID = GetInfoLocale($LocaleID)->{LanguageID};
    my %RegionalParams = (
        Locale   => $LocaleID,
        Language => GetCodeByLanguageID($LanguageID),
    );

    my %MailParams = (
        'HTMLAllowed' => 1,
        # 'HTMLAllowed' => $Shop->get('SuperMerchant')->get('IsHtmlEMailAllowed'),
        'FROM'        => $FromEmail,
        'TO'          => $NotificationEmail,
        'Subject'     => $MailType->get('Subject', $LanguageID),
    );

    my %TleVars = (
        'Shop'        => $Shop->tleHash,
        'Environment' => $Environment->tleHash,
        'ErrorCode'   => $ErrorCode,
        'RenewTokenLink' => $RenewTokenLink
    );

    my $hNotificationInfo = {
        'IsNotified'        => 0,
        'NotificationEmail' => $NotificationEmail
    };
    eval {
        $MailType->send( $Shop, \%MailParams, \%RegionalParams, \%TleVars );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
    } else {
        $hNotificationInfo->{'IsNotified'} = 1;
    }

    return $hNotificationInfo;
}

1;
