#========================================================================================
# package      DE_EPAGES::Zettle::API::Object::TokenHelper
# state        public
#----------------------------------------------------------------------------------------
# description  handles token-related (access token/refresh token) behaviour
#========================================================================================
package DE_EPAGES::Zettle::API::Object::TokenHelper;

use strict;

use DateTime;
use DateTime::Duration;
use Readonly;

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter  qw ( TestObject );
use DE_EPAGES::Zettle::API::Constants qw (
    EXPORT_STATUS
    REFRESH_TOKEN_EXPIRATION_DAYS_WARNING1
    REFRESH_TOKEN_EXPIRATION_DAYS_WARNING2
    REFRESH_TOKEN_EXPIRATION_DAYS_WARNING3
    REFRESH_TOKEN_VALIDITY_PERIOD_DAYS
    %CONNECTION_STATUS
);
use DE_EPAGES::Zettle::API::ZettleService;
use DE_EPAGES::Zettle::API::Mail qw ( NotifyRefreshTokenIssue );

# Names for token attributes in ZettleEnvironment object
Readonly my $REFRESH_TOKEN_INVALID_DATETIME_ATTR => 'InvalidRefreshTokenAcknowledgementDate';


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::Object::TokenHelper->new($Environment);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Environment | the Zettle environment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::Object::TokenHelper
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment)  = @_;
    TestObject('ZettleEnvironment', $Environment, 'DE_EPAGES::Zettle::API::Object::ZettleEnvironment');

    my $self  = bless {
        'Environment' => $Environment,
    }, $class;

    return $self;
}

#========================================================================================
# function     daysToRefreshTokenExpiration
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DaysToExpiration = $self->daysToRefreshTokenExpiration();
#----------------------------------------------------------------------------------------
# description  Calculate the number of days until refresh token expires.
#               A value of 0 or a negative value means token is expired.
#----------------------------------------------------------------------------------------
# return       $DaysToExpiration | Number of days till token expiration | int
#========================================================================================
sub daysToRefreshTokenExpiration {
    my $self = shift;

    my $Environment = $self->_getEnvironment();
    my $AccessTokenExpirationDT = $Environment->accessTokenExpiration();
    return 0 unless defined $AccessTokenExpirationDT;

    # If refresh token was previously detected as invalid, no further check is necessary
    return 0 if defined $Environment->get($REFRESH_TOKEN_INVALID_DATETIME_ATTR);

    # We use access token expiration date as reference for refresh token expiration.
    # From: https://developer.zettle.com/docs/api/oauth/user-guides/set-up-app-authorisation/set-up-authorisation-code-grant
    # > When a new access token is generated, a new refresh token is also generated
    # > access token is valid for 7200 seconds
    # > refresh token is valid for 180 days
    # Since we discard time part, the 2 hours difference (7200 seconds) won't really matter
    my $RefreshTokenExpiration = $AccessTokenExpirationDT + DateTime::Duration->new(days => REFRESH_TOKEN_VALIDITY_PERIOD_DAYS);

    # Calculate absolute days difference (discarding time difference) between current datetime and RefreshTokenExpiration
    # See https://metacpan.org/pod/DateTime#$dt-%3Edelta_days($datetime)
    # We only take into account date part ignoring time part to avoid errors when handling hours/minutes/seconds
    # time is not relevant for required calculations
    my $Now = GetCurrentDateTime();
    my $DaysToExpiration = ($RefreshTokenExpiration - $Now)->{'days'};

    if ($DaysToExpiration > 0) {
        # Perform the check unless we're in "warning period" (notification will be sent out anyway in that case)
        if ($DaysToExpiration > REFRESH_TOKEN_EXPIRATION_DAYS_WARNING1) {
            # Refresh token may have expired or being revoked on Zettle side
            # Use ZettleService to check whether refresh token is still valid on their side
            my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
            my $ConnectionStatus = $ZettleService->checkConnectionStatus();
            if ($ConnectionStatus < 0 && $ConnectionStatus == $CONNECTION_STATUS{INVALID_REFRESH_TOKEN}) {
                $DaysToExpiration = 0;
            }
        }
    }

    return $DaysToExpiration;
}

#========================================================================================
# function     verifyRefreshTokenAndNotifyMerchant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsNotificationTriggered = $self->verifyRefreshTokenAndNotifyMerchant();
#----------------------------------------------------------------------------------------
# description  Ensure environment is ready, verify refresh token and, if necessary, notify merchant
#               that a direct action (re-login) is required.
#               Conditions for notification (see <function _refreshTokenRequiresNotification> for details):
#               <ul>
#                   <li>Refresh token is about to expire:
#                       merchant is urged to act before expiration in order to avoid service disruption</li>
#                   <li>Refresh token is expired:
#                       merchant is informed that integration is not working anymore and direct action is required</li>
#               </ul>
#               See <function DE_EPAGES::Zettle::API::Mail::NotifyRefreshTokenIssue>.
#----------------------------------------------------------------------------------------
# return       $hNotificationInfo | Notification information with following keys:
#               <ul>
#                   <li>IsNotified - TRUE if notification was sent to merchant, FALSE otherwise - boolean
#                   <li>NotificationEmail - Optional. Email address where notification was sent to - string
#               </ul> | ref.hash
#----------------------------------------------------------------------------------------
# notes        This implementation is based on the assumption that the method will be executed
#               once a day (from a scheduled script), meaning we'll generate one notification per day
#               at most. If there's a change in the flow we'll have to revisit current implementation.
#========================================================================================
sub verifyRefreshTokenAndNotifyMerchant {
    my $self = shift;

    return 0 unless $self->_tokenValidationRequired();

    my $DaysToExpiration = $self->daysToRefreshTokenExpiration();

    return 0 unless $self->_refreshTokenRequiresNotification($DaysToExpiration);

    my $Environment = $self->_getEnvironment();
    my $hNotificationInfo = NotifyRefreshTokenIssue($Environment, $DaysToExpiration);

    return $hNotificationInfo;
}


#========================================================================================

#========================================================================================
# function     _tokenValidationRequired
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RequiresValidation = $self->_tokenValidationRequired();
#----------------------------------------------------------------------------------------
# description  Verify whether a refresh token needs to be validated.
#               Conditions to meet:
#               <ul>
#                   <li>Merchant enabled scheduled export</li>
#                   <li>Initial export is complete</li>
#                   <li>Merchant did not disconnect integration</li>
#               </ul>
#----------------------------------------------------------------------------------------
# return       $RequiresValidation | TRUE if validation is required, FALSE otherwise | boolean
#========================================================================================
sub _tokenValidationRequired {
    my $self = shift;

    my $Environment = $self->_getEnvironment();

    # No notice required unless scheduled export is enabled
    # Merchant using manual export mode would get a warning in MBO (see https://epages.atlassian.net/browse/UD-52)
    return 0 unless $Environment->exportScheduleInterval() > 0;

    # No notice required unless initial export is completed
    return 0 unless $Environment->initialExportStatus() == EXPORT_STATUS->{COMPLETED};

    # No notice required if merchant disconnected from Zettle
    return 0 if $Environment->requiresCleanup();

    return 1;
}

#========================================================================================
# function     _refreshTokenRequiresNotification
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RequiresNotification = $self->_refreshTokenRequiresNotification($DaysToExpiration);
#----------------------------------------------------------------------------------------
# description  Verify whether a notification needs to be sent to merchant.
#               Conditions for notification:
#               <ul>
#                   <li>$DaysToExpiration is equal to REFRESH_TOKEN_EXPIRATION_DAYS_WARNING1</li>
#                   <li>$DaysToExpiration is equal to REFRESH_TOKEN_EXPIRATION_DAYS_WARNING2</li>
#                   <li>$DaysToExpiration is equal to REFRESH_TOKEN_EXPIRATION_DAYS_WARNING3</li>
#                   <li>$DaysToExpiration is less than or equal to 0</li>
#               </ul>
#               For constant values refer to <package DE_EPAGES::Zettle::API::Constants>.
#----------------------------------------------------------------------------------------
# input        $DaysToExpiration | Number of days till token expiration | int
#----------------------------------------------------------------------------------------
# return       $RequiresNotification | TRUE if notification is required to be sent, FALSE otherwise | boolean
#========================================================================================
sub _refreshTokenRequiresNotification {
    my $self = shift;

    my ($DaysToExpiration) = @_;

    return ($DaysToExpiration == REFRESH_TOKEN_EXPIRATION_DAYS_WARNING1)
            || ($DaysToExpiration == REFRESH_TOKEN_EXPIRATION_DAYS_WARNING2)
            || ($DaysToExpiration == REFRESH_TOKEN_EXPIRATION_DAYS_WARNING3)
            || ($DaysToExpiration <= 0);
}

#========================================================================================
# function     _getEnvironment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Environment = $self->_getEnvironment();
#----------------------------------------------------------------------------------------
# description  Returns the Zettle Environment.
#----------------------------------------------------------------------------------------
# return       $Environment | Zettle Environment | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#========================================================================================
sub _getEnvironment {
    return shift->{'Environment'};
}

1;