#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle incoming messages triggered by Zettle events associated to webhooks subscriptions
#               See https://github.com/iZettle/api-documentation/blob/master/pusher-api/concept/event-payloads.md
#----------------------------------------------------------------------------------------
# notes        It's a generic structure which contains the field 'payload' whose contents vary depending on the type of event.
#               <ul>
#                   <li> organizationUuid - organization uuid -  string
#                   <li> messageUuid - UUID v1 based on timestamp and messageId - string
#                   <li> eventName - name of the event that triggered the message - string
#                   <li> messageId - unique UUID of the message - string
#                   <li> payload - specific content which depends on the type of event - hash
#                   <li> timestamp - event timestamp in ISO-8601 format - string
#               </ul>
#----------------------------------------------------------------------------------------
# example      {
#                   "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
#                   "messageUuid": "146719b0-0379-11ec-9482-a528d1b6a8b0",
#                   "eventName": "InventoryBalanceChanged",
#                   "messageId": "6785fcb4-4cc0-5f8b-9482-a528d1b6a8b0",
#                   "payload": "{ ... }",
#                   "timestamp": "2021-08-22T18:45:10.987Z"
#               }
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO;

use strict;

use DE_EPAGES::Zettle::API::Tools qw ( ParseDateTime );

# Name of event used for test message sent from Zettle to shop callback URL when a subscription is created
use constant SUBSCRIPTION_TEST_EVENT_NAME => 'TestMessage';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO->new();
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

# input        $OrganizationUuid | Identifies the organization | string
sub organizationUuid {
    my $self = shift;
    my ($OrganizationUuid) = @_;
    return $self->{'organizationUuid'} unless defined($OrganizationUuid);
    $self->{'organizationUuid'} = $OrganizationUuid;
}

# input        $MessageUuid | UUID v1 based on timestamp and messageId | string
sub messageUuid {
    my $self = shift;
    my ($MessageUuid) = @_;
    return $self->{'messageUuid'} unless defined($MessageUuid);
    $self->{'messageUuid'} = $MessageUuid;
}

# input        $EventName | Name of event | string
# See https://github.com/iZettle/api-documentation/blob/master/pusher-api/api-reference.md#supported-events
# See also Models > CreateSubscription > eventNames in https://pusher.izettle.com/swagger#/subscriptions/createSubscription
sub eventName {
    my $self = shift;
    my ($EventName) = @_;
    return $self->{'eventName'} unless defined($EventName);
    $self->{'eventName'} = $EventName;
}

# input        $MessageId | Unique UUID of the message | String
sub messageId {
    my $self = shift;
    my ($MessageId) = @_;
    return $self->{'messageId'} unless defined($MessageId);
    $self->{'messageId'} = $MessageId;
}

# input        $Payload | Specific payload associated to the event with real time information (JSON encoded) | string
sub payload {
    my $self = shift;
    my ($Payload) = @_;
    return $self->{'payload'} unless defined($Payload);
    $self->{'payload'} = $Payload;
}

# input        $Timestamp | Event timestamp | Datetime
sub timestamp {
    my $self = shift;
    my ($Timestamp) = @_;
    return $self->{'timestamp'} unless defined($Timestamp);
    $self->{'timestamp'} = $Timestamp;
}

# return       TRUE if current event is a test event, false otherwise
sub isTestEvent {
    my $self = shift;
    # Sample test request:
    # {
    #     "organizationUuid": "1a52e2a8-824d-11eb-acc8-aafa2648b59f",
    #     "messageUuid": "634a56c8-036f-11ec-8696-b989a6520ced",
    #     "eventName": "TestMessage",
    #     "messageId": "634a56dc-036f-11ec-aed9-f7a88aa6d3aa",
    #     "payload": "{\"data\":\"payload\"}",
    #     "timestamp": "2021-08-22T17:35:48.371226Z"
    # }
    return $self->eventName() eq SUBSCRIPTION_TEST_EVENT_NAME;
}

#========================================================================================
# function     fromHash
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WebhookEventDTO = DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO->fromHash($hWebhookEvent);
#----------------------------------------------------------------------------------------
# description  parse JSON contents from incoming event produced by a webhook subscription.
#               See <function DE_EPAGES::Zettle::API::PusherService::createSubscription>.
#----------------------------------------------------------------------------------------
# input        $hWebhookEvent | Hash parsed from JSON structure retrieved from incoming event (triggerd by webhook).
#                                See https://github.com/iZettle/api-documentation/blob/master/pusher-api/user-guides/subscriptions.md#understand-how-events-work | ref.hash
#----------------------------------------------------------------------------------------
# return       $WebhookEventDTO | a WebhookEventDTO object | DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO
#========================================================================================
sub fromHash {
    my $class = shift;
    my ($hWebhookEvent) = @_;

    my $WebhookEventDTO = $class->new();
    $WebhookEventDTO->organizationUuid($hWebhookEvent->{'organizationUuid'}) if defined $hWebhookEvent->{'organizationUuid'};
    $WebhookEventDTO->messageUuid($hWebhookEvent->{'messageUuid'}) if defined $hWebhookEvent->{'messageUuid'};
    $WebhookEventDTO->eventName($hWebhookEvent->{'eventName'}) if defined $hWebhookEvent->{'eventName'};
    $WebhookEventDTO->messageId($hWebhookEvent->{'messageId'}) if defined $hWebhookEvent->{'messageId'};
    $WebhookEventDTO->payload($hWebhookEvent->{'payload'}) if defined $hWebhookEvent->{'payload'};
    my $Timestamp = ParseDateTime($hWebhookEvent->{'timestamp'});
    $WebhookEventDTO->timestamp($hWebhookEvent->{'timestamp'}) if defined $Timestamp;

    return $WebhookEventDTO;
}

1;
