#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::EndpointHandler
# state        private
#----------------------------------------------------------------------------------------
# description  This module provides all the methods to retrieve Zettle API endpoint URLs
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::EndpointHandler;
use base qw ( Exporter );

use strict;

my @AuthServiceEndpoints = qw (
    GetAuthApiUrl
    GetDisconnectApiUrl
    GetTokenApiUrl
    GetUserInfoApiUrl
);
my @MerchantServiceEndpoints = qw (
    GetAccountInfoApiUrl
);
my @ImageServiceEndpoints = qw (
    GetBulkUploadImageApiUrl
    GetDeleteImageApiUrl
    GetUploadImageApiUrl
);
my @ProductServiceEndpoints = qw (
    GetAllProductsApiUrl
    GetBulkExportApiUrl
    GetBulkExportStatusApiUrl
    GetDeleteProductApiUrl
    GetDeleteProductListApiUrl
    GetProductApiUrl
    GetProductCountApiUrl
    GetProductUpdateApiUrl
);
my @InventoryServiceEndpoints = qw (
    GetAllLocationsApiUrl
    GetBulkUpdateProductTrackingApiUrl
    GetLocationInventoryApiUrl
    GetProductInventoryApiUrl
    GetProductListInventoryApiUrl
    GetUpdateInventoryBalanceApiUrl
    GetTrackedProductsApiUrl
    GetTrackingStatusApiUrl
);
my @PusherServiceEndpoints = qw (
    CreateSubcriptionApiUrl
    DeleteSubscriptionApiUrl
    GetAllSubscriptionsApiUrl
);

our @EXPORT_OK = (
    @AuthServiceEndpoints,
    @MerchantServiceEndpoints,
    @ImageServiceEndpoints,
    @ProductServiceEndpoints,
    @InventoryServiceEndpoints,
    @PusherServiceEndpoints,
    'GetAPICallRetryDelay'
);

use DE_EPAGES::Object::API::Factory qw( LoadRootObject );

# Method URLs relative to OAuth API endpoint
my $OAuthApiMethodUrl = {
    'auth'       => '/authorize',
    'disconnect' => '/application-connections/self',
    'token'      => '/token',
    'userInfo'   => '/users/self',
};

# Method URLs relative to Secure API endpoint
my $SecureApiMethodUrl = {
    'accountInfo' => '/api/resources/organizations/self',
};

# Method URLs relative to Image API endpoint
my $ImageApiMethodUrl = {
    'bulkUploadImages' => '/v2/images/organizations/self/products/bulk',
    'deleteImage'      => '/v2/images/organizations/self/products/#ImageLookupKey',
    'uploadImage'      => '/v2/images/organizations/self/products',
};

# Method URLs relative to Product API endpoint
my $ProductApiMethodUrl = {
    'allProducts'       => '/organizations/self/products/v2',
    'bulkExport'        => '/organizations/self/import/v2',
    'bulkExportStatus'  => '/organizations/self/import/status/#ExportUUID',
    'deleteProduct'     => '/organizations/self/products/#ProductUUID',
    'deleteProductList' => '/organizations/self/products',
    'getProduct'        => '/organizations/self/products/#ProductUUID',
    'productCount'      => '/organizations/self/products/v2/count',
    'productUpdate'     => '/organizations/self/products/v2/#ProductUUID',
};

# Method URLs relative to Inventory API endpoint
my $InventoryApiMethodUrl = {
    'allLocations'              => '/inventories',
    'bulkUpdateProductTracking' => '/products',
    'locationInventory'         => '/stock/#LocationUUID',
    'productInventory'          => '/stock/#LocationUUID/products/#ProductUUID',
    'productListInventory'      => '/stock/#LocationUUID/products',
    'updateInventoryBalance'    => '/movements',
    'trackedProducts'           => '/products',
    'trackingStatus'            => '/products/status',
};

# Method URLs relative to Pusher API endpoint
my $PusherApiMethodUrl = {
    'createSubscription'     => '/organizations/self/subscriptions',
    'getAllSubscriptions'    => '/organizations/self/subscriptions',
    'deleteSubscription'     => '/organizations/self/subscriptions/#SubscriptionUUID',
};


# Zettle recommends to not exceed 15 requests per second, that is 1 request each 67ms, so we round up to 75ms.
# See https://developer.zettle.com/docs/api/product-library/concepts/product-library-concept#recommendations-and-limits
use constant DEFAULT_API_RETRY_WAIT_TIME_MS => 75;

# Endpoint-specific delay when rate-limit is reached.
# Consists in a map of <URL, Delay> pairs where:
# - URL is the base endpoint URL.
# - Delay is the initial wait time (milliseconds to wait on first 429 error).
my $UrlThrottling = {
    # 4 requests per second (see https://developer.zettle.com/docs/api/inventory/overview#rate-limit)
    'inventory.izettle.com/v3' => 250
};

#========================================================================================
# function     GetAPICallRetryDelay
# state        public
#========================================================================================
# syntax       $DelayMS = $self->GetAPICallRetryDelay($RequestUrl, $RetryCount);
# syntax       $DelayMS = $self->GetAPICallRetryDelay($RequestUrl, $RetryCount, $RetryBaseWaitTime);
#----------------------------------------------------------------------------------------
# description  Calculate the time to wait before retrying an API call that reached rate limits.
#               Delay will be adjusted with a linear increment based on $RetryCount.
#----------------------------------------------------------------------------------------
# input        $RequestUrl | URL the API request was sent to (used to apply endpoint-specific settings) | string
# input        $RetryCount | Indicates the number of retry of the current request (used to adjust delay) | int
# input        $RetryBaseWaitTime | optional. Force a specific base wait time (milliseconds).
#                                    See <function _GetEndpointRetryBaseWaitTime> | int
#----------------------------------------------------------------------------------------
# return       $DelayMS | time (in milliseconds) to wait before retrying API call | int
#========================================================================================
sub GetAPICallRetryDelay {
    my ($RequestUrl, $RetryCount, $RetryBaseWaitTime) = @_;

    $RetryBaseWaitTime //= _GetEndpointRetryBaseWaitTime($RequestUrl);

    # Linear increment based on RetryCount
    my $DelayMS = $RetryCount*$RetryBaseWaitTime;

    return $DelayMS;
}

#================================ OAuth endpoint related methods =================================

sub GetAuthApiUrl {
    return _GetOAuthEndpoint() . $OAuthApiMethodUrl->{'auth'};
}

sub GetDisconnectApiUrl {
    return _GetOAuthEndpoint() . $OAuthApiMethodUrl->{'disconnect'};
}

sub GetTokenApiUrl {
    return _GetOAuthEndpoint() . $OAuthApiMethodUrl->{'token'};
}

sub GetUserInfoApiUrl {
    return _GetOAuthEndpoint() . $OAuthApiMethodUrl->{'userInfo'};
}

#================================ Secure endpoint related methods =================================

sub GetAccountInfoApiUrl {
    return _GetSecureEndpoint() . $SecureApiMethodUrl->{'accountInfo'};
}

#================================ Image endpoint related methods =================================

sub GetBulkUploadImageApiUrl {
    return _GetImageEndpoint() . $ImageApiMethodUrl->{'bulkUploadImages'};
}

sub GetDeleteImageApiUrl {
    my ($ImageLookupKey) = @_;

    my $ParameterizedMethodUrl = $ImageApiMethodUrl->{'deleteImage'};
    $ParameterizedMethodUrl =~ s/#ImageLookupKey/$ImageLookupKey/;

    return _GetImageEndpoint() . $ParameterizedMethodUrl;
}

sub GetUploadImageApiUrl {
    return _GetImageEndpoint() . $ImageApiMethodUrl->{'uploadImage'};
}

#================================ Product endpoint related methods =================================

sub GetAllProductsApiUrl {
    return _GetProductEndpoint() . $ProductApiMethodUrl->{'allProducts'};
}

sub GetBulkExportApiUrl {
    return _GetProductEndpoint() . $ProductApiMethodUrl->{'bulkExport'};
}

#========================================================================================
# function     GetBulkExportStatusApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetBulkExportStatusApiUrl($ExportUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to query bulk export status
#               See: https://products.izettle.com/swagger#/import/getStatusByUuid
#               See: https://products.izettle.com/swagger#/import/getLatestImportStatus
#----------------------------------------------------------------------------------------
# input        $ExportUUID | optional. Export UUID (default: query last bulk export status) | string
#----------------------------------------------------------------------------------------
# return       $StatusURL | Export status URL | string
#========================================================================================
sub GetBulkExportStatusApiUrl {
    my ($ExportUUID) = @_;
    $ExportUUID //= '';

    my $ParameterizedMethodUrl = $ProductApiMethodUrl->{'bulkExportStatus'};
    $ParameterizedMethodUrl =~ s/#ExportUUID/$ExportUUID/;

    return _GetProductEndpoint() . $ParameterizedMethodUrl;
}

#========================================================================================
# function     GetDeleteProductApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetDeleteProductApiUrl($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to delete a single product
#               See: https://products.izettle.com/swagger#/products/deleteProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $DeleteURL | URL required for DELETE request | string
#========================================================================================
sub GetDeleteProductApiUrl {
    my ($ProductUUID) = @_;

    my $ParameterizedMethodUrl = $ProductApiMethodUrl->{'deleteProduct'};
    $ParameterizedMethodUrl =~ s/#ProductUUID/$ProductUUID/;

    return _GetProductEndpoint() . $ParameterizedMethodUrl;
}

#========================================================================================
# function     GetDeleteProductListApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetDeleteProductListApiUrl();
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to delete a list of product
#               See: https://products.izettle.com/swagger#/products/deleteProducts
#----------------------------------------------------------------------------------------
# return       $DeleteURL | URL required for DELETE request | string
#========================================================================================
sub GetDeleteProductListApiUrl {
    my ($ProductUUID) = @_;
    return _GetProductEndpoint() . $ProductApiMethodUrl->{'deleteProductList'};
}

#========================================================================================
# function     GetProductApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetProductApiUrl($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to retrieve a single product
#               See: https://products.izettle.com/swagger#/products/getProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $GetProductURL | URL required for GET request | string
#========================================================================================
sub GetProductApiUrl {
    my ($ProductUUID) = @_;

    my $ParameterizedMethodUrl = $ProductApiMethodUrl->{'getProduct'};
    $ParameterizedMethodUrl =~ s/#ProductUUID/$ProductUUID/;

    return _GetProductEndpoint() . $ParameterizedMethodUrl;
}

sub GetProductCountApiUrl {
    return _GetProductEndpoint() . $ProductApiMethodUrl->{'productCount'};
}

#========================================================================================
# function     GetProductUpdateApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetProductUpdateApiUrl($ProductUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to update a single product
#               See: https://products.izettle.com/swagger#/products/updateProduct
#----------------------------------------------------------------------------------------
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $UpdateURL | URL required for PUT request | string
#========================================================================================
sub GetProductUpdateApiUrl {
    my ($ProductUUID) = @_;

    my $ParameterizedMethodUrl = $ProductApiMethodUrl->{'productUpdate'};
    $ParameterizedMethodUrl =~ s/#ProductUUID/$ProductUUID/;

    return _GetProductEndpoint() . $ParameterizedMethodUrl;
}

#================================ Inventory endpoint related methods =================================

#========================================================================================
# function     GetAllLocationsApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetAllLocationsApiUrl();
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve all locations (inventories according to APIv3 terminology).
#----------------------------------------------------------------------------------------
# return       $GetURL | URL required for GET request | string
#========================================================================================
sub GetAllLocationsApiUrl { 
    return _GetInventoryEndpoint() . $InventoryApiMethodUrl->{'allLocations'};
}

#========================================================================================
# function     GetBulkUpdateProductTrackingApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetBulkUpdateProductTrackingApiUrl();
#----------------------------------------------------------------------------------------
# description  Retrieve URL used to disable update product tracking.
#----------------------------------------------------------------------------------------
# return       $UpdateTrackingURL | URL required for POST request | string
#========================================================================================
sub GetBulkUpdateProductTrackingApiUrl {
    return _GetInventoryEndpoint() . $InventoryApiMethodUrl->{'bulkUpdateProductTracking'};
}

#========================================================================================
# function     GetLocationInventoryApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetLocationInventoryApiUrl($LocationUUID);
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve inventory for given location.
#----------------------------------------------------------------------------------------
# input        $LocationUUID | Location UUID | string
#----------------------------------------------------------------------------------------
# return       $GetURL | URL required for GET request | string
#========================================================================================
sub GetLocationInventoryApiUrl {
    my ($LocationUUID) = @_;

    my $ParameterizedMethodUrl = $InventoryApiMethodUrl->{'locationInventory'};
    $ParameterizedMethodUrl =~ s/#LocationUUID/$LocationUUID/;

    return _GetInventoryEndpoint() . $ParameterizedMethodUrl;
}

#========================================================================================
# function     GetProductInventoryApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetProductInventoryApiUrl($LocationUUID, $ProductUUID);
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve inventory for given product in specified location.
#----------------------------------------------------------------------------------------
# input        $LocationUUID | Location UUID | string
# input        $ProductUUID | Product UUID | string
#----------------------------------------------------------------------------------------
# return       $GetURL | URL required for GET request | string
#========================================================================================
sub GetProductInventoryApiUrl {
    my ($LocationUUID, $ProductUUID) = @_;

    my $ParameterizedMethodUrl = $InventoryApiMethodUrl->{'productInventory'};
    $ParameterizedMethodUrl =~ s/#LocationUUID/$LocationUUID/;
    $ParameterizedMethodUrl =~ s/#ProductUUID/$ProductUUID/;

    return _GetInventoryEndpoint() . $ParameterizedMethodUrl;
}

#========================================================================================
# function     GetProductListInventoryApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetProductListInventoryApiUrl($LocationUUID);
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve stock levels for products in specified location.
#----------------------------------------------------------------------------------------
# input        $LocationUUID | Location UUID | string
#----------------------------------------------------------------------------------------
# return       $GetURL | URL required for POST request | string
#========================================================================================
sub GetProductListInventoryApiUrl {
    my ($LocationUUID) = @_;

    my $ParameterizedMethodUrl = $InventoryApiMethodUrl->{'productListInventory'};
    $ParameterizedMethodUrl =~ s/#LocationUUID/$LocationUUID/;

    return _GetInventoryEndpoint() . $ParameterizedMethodUrl;
}

#========================================================================================
# function     GetUpdateInventoryBalanceApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ResourceURL = GetUpdateInventoryBalanceApiUrl();
#----------------------------------------------------------------------------------------
# description  Generate URL used to update stock levels for products.
#----------------------------------------------------------------------------------------
# return       $ResourceURL | URL required for POST request | string
#========================================================================================
sub GetUpdateInventoryBalanceApiUrl {
    return _GetInventoryEndpoint() . $InventoryApiMethodUrl->{'updateInventoryBalance'};
}

#========================================================================================
# function     GetTrackedProductsApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $GetURL = GetTrackedProductsApiUrl();
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve tracked products.
#----------------------------------------------------------------------------------------
# return       $GetURL | URL required for GET request | string
#========================================================================================
sub GetTrackedProductsApiUrl {
    return _GetInventoryEndpoint() . $InventoryApiMethodUrl->{'trackedProducts'};
}

#========================================================================================
# function     GetTrackingStatusApiUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = GetTrackingStatusApiUrl();
#----------------------------------------------------------------------------------------
# description  Generate URL used to retrieve product tracking status.
#----------------------------------------------------------------------------------------
# return       $Url | URL required for POST request | string
#========================================================================================
sub GetTrackingStatusApiUrl {
    return _GetInventoryEndpoint() . $InventoryApiMethodUrl->{'trackingStatus'};
}

#================================ Pusher endpoint related methods =================================

sub CreateSubcriptionApiUrl {
    return _GetPusherEndpoint() . $PusherApiMethodUrl->{'createSubscription'};
}

sub DeleteSubscriptionApiUrl {
    my ($SubscriptionUUID) = @_;

    my $ParameterizedMethodUrl = $PusherApiMethodUrl->{'deleteSubscription'};
    $ParameterizedMethodUrl =~ s/#SubscriptionUUID/$SubscriptionUUID/;

    return _GetPusherEndpoint() . $ParameterizedMethodUrl;
}

sub GetAllSubscriptionsApiUrl {
    return _GetPusherEndpoint() . $PusherApiMethodUrl->{'getAllSubscriptions'};
}
 
#=================================== Private methods ====================================

# Retrieve OAuth endpoint
# See https://developer.zettle.com/docs/api/oauth/overview
# See https://github.com/iZettle/api-documentation/blob/master/authorization.md (deprecated)
sub _GetOAuthEndpoint {
    return _LoadSystemAttribute('Zettle_OAuthEndpoint');
}

# Retrieve Image endpoint
# See https://developer.zettle.com/docs/api/image/overview
# See https://github.com/iZettle/api-documentation/blob/master/image.md (deprecated)
sub _GetImageEndpoint {
    return _LoadSystemAttribute('Zettle_ImageEndpoint');
}

# Retrieve Product endpoint
# See https://developer.zettle.com/docs/api/product-library/overview
# See https://github.com/iZettle/api-documentation/blob/master/product-library.adoc (deprecated)
sub _GetProductEndpoint {
    return _LoadSystemAttribute('Zettle_ProductEndpoint');
}

# Retrieve Inventory endpoint
# See https://developer.zettle.com/docs/api/inventory/overview
# See https://github.com/iZettle/api-documentation/tree/master/inventory-api (deprecated)
sub _GetInventoryEndpoint {
    return _LoadSystemAttribute('Zettle_InventoryEndpoint');
}

# Retrieve Secure endpoint
# See DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.4.2. Retrieve a Merchant's Account Information
sub _GetSecureEndpoint {
    return 'https://secure.izettle.com';
}

# Retrieve Pusher endpoint
# https://developer.zettle.com/docs/api/pusher/overview
# See https://github.com/iZettle/api-documentation/blob/master/pusher-api/api-reference.md (deprecated)
sub _GetPusherEndpoint {
    return 'https://pusher.izettle.com';
}

# Return the value of the given system attribute
sub _LoadSystemAttribute {
    my ($AttributeName) = @_;

    my $System = LoadRootObject();
    my $AttributeValue = $System->get($AttributeName);

    return $AttributeValue;
}

#========================================================================================
# function     _GetEndpointRetryBaseWaitTime
# state        private
#========================================================================================
# syntax       $BaseRetryWaitTime = _GetEndpointRetryBaseWaitTime($RequestUrl);
#----------------------------------------------------------------------------------------
# description  Retrieve the initial time to wait before retrying an API call that reached API rate limits for the first time.
#----------------------------------------------------------------------------------------
# input        $RequestUrl | URL of the API request that exceeded quota | string
#----------------------------------------------------------------------------------------
# return       $BaseRetryWaitTime | time (in milliseconds) to wait before retrying API call | int
#========================================================================================
sub _GetEndpointRetryBaseWaitTime {
    my ($RequestUrl) = @_;

    my $BaseRetryWaitTime;
    for my $BaseUrl (keys %{$UrlThrottling}) {
        # Adjust throttling for specific endpoints
        # The '\Q' and '\E' operators prevent the dot character '.' to be interpreted as a wildcard
        if ($RequestUrl =~ /\Q$BaseUrl\E/) {            
            $BaseRetryWaitTime = $UrlThrottling->{$BaseUrl};
            last;
        }
    }

    # If no BaseRetryWaitTime is configured we use 75 milliseconds (see constant DEFAULT_API_RETRY_WAIT_TIME_MS).
    $BaseRetryWaitTime //= DEFAULT_API_RETRY_WAIT_TIME_MS;

    return $BaseRetryWaitTime;
}

1;
