#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::InventoryService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around inventory-related methods (aka stock levels) from Zettle Inventory API
#               See https://developer.zettle.com/docs/api/inventory/overview
#               See https://github.com/iZettle/api-documentation/blob/master/inventory-api/overview.md
#               See https://inventory.izettle.com/swagger#/
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::InventoryService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use HTTP::Status qw( HTTP_NOT_FOUND );

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Zettle::API::WebService::DTO::LocationDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductTrackingDTO;
##no critic qw(RequireCorrectImports)
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw (
    GetAllLocationsApiUrl
    GetBulkUpdateProductTrackingApiUrl
    GetLocationInventoryApiUrl
    GetProductInventoryApiUrl
    GetProductListInventoryApiUrl
    GetUpdateInventoryBalanceApiUrl
    GetTrackedProductsApiUrl
    GetTrackingStatusApiUrl
);
##use critic qw(RequireCorrectImports)

# List of default stores (locations) in Zettle
use constant DEFAULT_STORES => qw / Supplier Store Sold Bin /;

# Maximum number of items to send in a single bulk request.
# Consists in map of <MethodName, MaxItems> pairs where:
# - MethodName is the name of the method in current module.
# - MaxItems is the maximum number of items to include in one request.
my $BatchSizeByMethod = {
    # Reference: Request body > Schema in https://inventory.izettle.com/swagger#/default/trackProducts
    'bulkUpdateProductTracking' => 500,
    # Reference: Request body > Schema in https://inventory.izettle.com/swagger#/inventory/bulkUpdate
    # We keep old v2 limit since v3 doesn't specify it: https://inventory.izettle.com/swagger#/default/moveInventory
    # For v2 limit refer to Request body > Schema in https://inventory.izettle.com/swagger#/inventory/updateBalanceForProduct ## no critic qw(ProhibitLongLines)
    'bulkUpdateProductBalance' => 500
};

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Environment | a Zettle environment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::InventoryService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     getAllLocations
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aLocationDTO = $self->getAllLocations();
#----------------------------------------------------------------------------------------
# description  Retrieve all locations from Zettle.
#               See DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md,
#                   section 4.7.1. Inventory Lifecycle and Locations
#               See https://developer.zettle.com/docs/api/inventory/concepts/how-inventories-work#inventory-flow
#               See: https://inventory.izettle.com/swagger#/default/getInventories
#----------------------------------------------------------------------------------------
# return       $aLocationDTO | List of locations.
#                               See DE_EPAGES::Zettle::API::WebService::DTO::LocationDTO | ref.array.object
#========================================================================================
sub getAllLocations {
    my $self = shift;

    my $LocationsUrl = GetAllLocationsApiUrl();

    my $HttpResponse = $self->sendGetRequest($LocationsUrl);
    my $ahLocation = $self->decodeResponseContent($HttpResponse);

    my $aLocationDTO = [];
    foreach my $hLocation (@{$ahLocation}){
        my $LocationDTO = DE_EPAGES::Zettle::API::WebService::DTO::LocationDTO->fromHash($hLocation);
        push(@{$aLocationDTO}, $LocationDTO);
    }

    return $aLocationDTO;
}

#========================================================================================
# function     enableProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Enabled = $self->enableProductTracking($ProductUuid);
#----------------------------------------------------------------------------------------
# description  Start inventory tracking for given product.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-tracking/enable-disable-tracking#enable-tracking ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#trackProducts
#               See: https://inventory.izettle.com/swagger#/default/trackProducts
#----------------------------------------------------------------------------------------
# return       $Enabled | TRUE if product tracking was enabled | boolean
#----------------------------------------------------------------------------------------
# throws       WebServiceError | If an error occurred while enabling tracking.
#========================================================================================
sub enableProductTracking {
    my $self = shift;

    my ($ProductUuid) = @_;

    return $self->bulkUpdateProductTracking([ $ProductUuid ], 'enable');
}

#========================================================================================
# function     disableProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Disabled = $self->disableProductTracking($ProductUuid);
#----------------------------------------------------------------------------------------
# description  Stop inventory tracking for given product.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-tracking/enable-disable-tracking#disable-tracking ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#trackProducts
#               See: https://inventory.izettle.com/swagger#/default/trackProducts
#----------------------------------------------------------------------------------------
# return       $Disabled | TRUE if product tracking was disabled | boolean
#----------------------------------------------------------------------------------------
# throws       WebServiceError | If an error occurred while disabling tracking.
#========================================================================================
sub disableProductTracking {
    my $self = shift;

    my ($ProductUuid) = @_;

    return $self->bulkUpdateProductTracking([ $ProductUuid ], 'disable');
}

#========================================================================================
# function     bulkUpdateProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Updated = $self->bulkUpdateProductTracking($aProductUuid, $TrackingStatus);
#----------------------------------------------------------------------------------------
# description  Update inventory information for given products.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-tracking/enable-disable-tracking ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#trackProducts
#               See: https://inventory.izettle.com/swagger#/default/trackProducts
#----------------------------------------------------------------------------------------
# input        $aProductUuid | List of product UUIDs | ref.array.string
# input        $TrackingStatus | The new tracking status to set for updated products, any of: enable, disable | string
#----------------------------------------------------------------------------------------
# return       $Updated | TRUE if product tracking was successfully updated | boolean
#========================================================================================
# throws       WebServiceError - If $TrackingStatus is 'disable' and some of the UUIDs in $aProductUuid
#               are not present in Zettle the request will fail on the first missing UUID and not update
#               the rest of the products. The error is:
#               WebserviceError: Response={"developerMessage":"Product not tracked. UUID=61180610-1f42-1948-2520-b976395f1ee2",
#                 "errorType":"PRODUCT_NOT_TRACKED","violations":[]}; Status=404; StatusCode=404 Not Found
#               To complete pending updates caller will have to invoke the method again passing the rest of the products.
#========================================================================================
sub bulkUpdateProductTracking {
    my $self = shift;
    my ($aProductUuid, $TrackingStatus) = @_;

    Error("Unexpected value '$TrackingStatus' for parameter 'TrackingStatus' (values: 'enable', 'disable')")
        unless (ElementOf($TrackingStatus, qw / enable disable /));

    my $InventoryUrl = GetBulkUpdateProductTrackingApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };

    # Make batches according to API specification
    my $BatchSize = $BatchSizeByMethod->{'bulkUpdateProductTracking'};
    my $ProductCount = scalar @{$aProductUuid};
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=$BatchSize){
        my $EndIndex = ($StartIndex+$BatchSize >= $ProductCount) ? $ProductCount-1 : $StartIndex+$BatchSize-1;
        my @aBatchProductUuid = @{$aProductUuid}[$StartIndex..$EndIndex];
        my @aContent;
        foreach my $ProductUuid (@aBatchProductUuid){
            push(@aContent,
                    {
                        'productUuid' => $ProductUuid,
                        'tracking' => $TrackingStatus
                    });
        }

        eval {
            # API responds with 204 (NO CONTENT) if successful
            $self->sendPostRequest($InventoryUrl, $hHeadersParams, \@aContent);
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ($Error->vars()->{'Status'} != HTTP_NOT_FOUND) {
                Error($Error); #forward all other Errors
            }
        }
    }

    return 1;
}

#========================================================================================
# function     getLocationTypeInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductInventoryDTO = $self->getLocationTypeInventory($LocationType);
#----------------------------------------------------------------------------------------
# description  Retrieve all product inventory information for given type of location.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/fetch-inventory-balance#fetch-inventory-balance-of-a-store-inventory ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getStockForInventory
#               See: https://inventory.izettle.com/swagger#/default/getStockForInventory
#----------------------------------------------------------------------------------------
# input        $LocationType | Type of location, any of: Supplier, Store, Sold, Bin | string
#----------------------------------------------------------------------------------------
# return       $aProductInventoryDTO | a list of ProductInventoryDTO objects each indicating the balance
#                                       of every variation | ref.array.DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO
#========================================================================================
sub getLocationTypeInventory {
    my $self = shift;
    my ($LocationType) = @_;

    Error("Unexpected value '$LocationType' for parameter 'LocationType' (expected: Supplier, Store, Sold, Bin)")
            unless (ElementOf($LocationType, DEFAULT_STORES));

    my $Environment = $self->getEnvironment();
    # Retrieve UUID for given location from Zettle Environment
    my $LocationUUID = $Environment->get("Location${LocationType}UUID");

    return $self->getLocationInventory($LocationUUID);
}

#========================================================================================
# function     getLocationInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductInventoryDTO = $self->getLocationInventory();
# syntax       $aProductInventoryDTO = $self->getLocationInventory($LocationUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve all product inventory information for given location.
#               Inventory for 'Store' location will be retrieved if no location is provided.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/fetch-inventory-balance#fetch-inventory-balance-of-a-store-inventory ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getStockForInventory
#               See: https://inventory.izettle.com/swagger#/default/getStockForInventory
#----------------------------------------------------------------------------------------
# input        $LocationUUID | optional. Location UUID (default: 'Store' location UUID) | string
#----------------------------------------------------------------------------------------
# return       $aProductInventoryDTO | a list of ProductInventoryDTO objects each indicating the balance
#                                       of every variation
#                                     | ref.array.DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO
#========================================================================================
sub getLocationInventory {
    my $self = shift;
    my ($LocationUUID) = @_;

    $LocationUUID //= $self->getEnvironment()->locationStoreUuid();

    my $Url = GetLocationInventoryApiUrl($LocationUUID);

    my $HttpResponse = $self->sendGetRequest($Url);
    my $ahProductInventory = $self->decodeResponseContent($HttpResponse);

    my $aProductInventoryDTO = [];
    for my $hProductInventory (@{$ahProductInventory}) {
        my $ProductInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO->fromHash($hProductInventory);
        push (@{$aProductInventoryDTO}, $ProductInventoryDTO);
    }

    return $aProductInventoryDTO;
}

#========================================================================================
# function     getProductInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductInventoryDTO = $self->getProductInventory($ProductUuid);
# syntax       $ProductInventoryDTO = $self->getProductInventory($ProductUuid, $LocationUuid);
#----------------------------------------------------------------------------------------
# description  Retrieve inventory for product in given location.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/fetch-inventory-balance#fetch-inventory-balance-of-a-product-in-a-store-inventory ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getStockForProduct
#               See: https://inventory.izettle.com/swagger#/default/getStockForProduct
#----------------------------------------------------------------------------------------
# input        $ProductUuid | a superproduct Zettle identifier (ZettleUuid) | string
# input        $LocationUUID | optional. Location UUID (default: location UUID for Store) | string
#----------------------------------------------------------------------------------------
# return       $aProductInventoryDTO | a list of ProductInventoryDTO objects each indicating the balance
#                                       of every variation of the given superproduct UUID
#                                     | ref.array.DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO
#----------------------------------------------------------------------------------------
# notes        Response will contain variation balances if superproduct were found in Zettle, even if tracking is disabled.
#               An empty list will be returned if not found.
#========================================================================================
sub getProductInventory {
    my $self = shift;
    my ($ProductUuid, $LocationUuid) = @_;

    $LocationUuid //= $self->getEnvironment()->locationStoreUuid();

    my $Url = GetProductInventoryApiUrl($LocationUuid, $ProductUuid);

    my $HttpResponse = $self->sendGetRequest($Url);
    my $ahProductInventory = $self->decodeResponseContent($HttpResponse);

    my $aProductInventoryDTO = [];
    for my $hProductInventory (@{$ahProductInventory}) {
        my $ProductInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO->fromHash($hProductInventory);
        push (@{$aProductInventoryDTO}, $ProductInventoryDTO);
    }

    return $aProductInventoryDTO;
}

#========================================================================================
# function     getProductListInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductInventoryDTO = $self->getProductListInventory($aProductUuid);
# syntax       $aProductInventoryDTO = $self->getProductListInventory($aProductUuid, $LocationUuid);
#----------------------------------------------------------------------------------------
# description  Retrieve inventory for a list of products.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/fetch-inventory-balance#fetch-balances-of-a-few-products-in-an-inventory ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getStockForProducts
#               See: https://inventory.izettle.com/swagger#/default/getStockForProducts
#----------------------------------------------------------------------------------------
# input        $aProductUuid | list of superproduct Zettle identifiers (ZettleUuid)  | ref.array.string
# input        $LocationUuid | optional. Location UUID (default: location UUID for Store) | string
#----------------------------------------------------------------------------------------
# return       $aProductInventoryDTO | a list of ProductInventoryDTO object.
#                                       See DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO | ref.array.object
#----------------------------------------------------------------------------------------
# notes        Response will contain variation balances for superproducts found in Zettle, even if tracking is disabled.
#               Superproduct UUIDs not found in Zettle will be silently ignored.
#========================================================================================
sub getProductListInventory {
    my $self = shift;

    my ($aProductUuid, $LocationUuid) = @_;

    $LocationUuid //= $self->getEnvironment()->locationStoreUuid();

    my $InventoryUrl = GetProductListInventoryApiUrl($LocationUuid);
    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };

    my $HttpResponse = $self->sendPostRequest($InventoryUrl, $hHeadersParams, $aProductUuid);
    my $ahProductInventory = $self->decodeResponseContent($HttpResponse);

    my $aProductInventoryDTO = [];
    for my $hProductInventory (@{$ahProductInventory}) {
        my $ProductInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO->fromHash($hProductInventory);
        push (@{$aProductInventoryDTO}, $ProductInventoryDTO);
    }

    return $aProductInventoryDTO;
}

#========================================================================================
# function     updateProductBalance
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductBalanceDTO = $self->updateProductBalance($ProductUuid, $VariantUuid, $BalanceChange);
#----------------------------------------------------------------------------------------
# description  Update balance for a single product variation.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/update-inventory-balance#update-inventory-balance ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#moveInventory
#               See: https://inventory.izettle.com/swagger#/default/moveInventory
#----------------------------------------------------------------------------------------
# input        $ProductUuid | Product UUID | string
# input        $VariantUuid | Variation UUID | string
# input        $BalanceChange | Change in product stock level | integer
#----------------------------------------------------------------------------------------
# return       $Updated | True if balance was successfully updated in Zettle, false otherwise | boolean
#========================================================================================
sub updateProductBalance {
    my $self = shift;
    my ($ProductUuid, $VariantUuid, $BalanceChange) = @_;

    my $hBalanceUpdate = {
                            $ProductUuid => {
                                $VariantUuid => $BalanceChange
                            }
                         };
    my $hUpdateResult = $self->bulkUpdateProductBalance($hBalanceUpdate);

    return ElementOf($VariantUuid, $hUpdateResult->{'updated'});
}

#========================================================================================
# function     bulkUpdateProductBalance
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductBalanceBulkDTO = $self->bulkUpdateProductBalance($hBalanceUpdate);
#----------------------------------------------------------------------------------------
# description  Update balance for multiple products.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-balances/update-inventory-balance#update-inventory-balance ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#moveInventory
#               See: https://inventory.izettle.com/swagger#/default/moveInventory
#----------------------------------------------------------------------------------------
# input        $hBalanceUpdate | Map of balance updates indexed by ProductUUID
#                                 where values are a map of variation stock changes indexed by VariantUUID | ref.hash.hash
#----------------------------------------------------------------------------------------
# return       $hUpdateResult | List of updated variations and list of failed updates, with following keys:
#               <ul>
#                   <li>updated - list of ZettleUUIDs of variations successfully updated in Zettle - ref.array.string
#                   <li>failed - list of ZettleUUIDs of variations that failed to be updated - ref.array.string
#               </ul>| ref.hash
#========================================================================================
sub bulkUpdateProductBalance {
    my $self = shift;
    my ($hBalanceUpdate) = @_;

    my $BulkUpdateBalanceUrl = GetUpdateInventoryBalanceApiUrl();

    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };

    # Make batches since call is limited to 500 items
    my $hUpdateResult = {
        'updated' => [],
        'failed' => []
    };
    my @aProductUuid = keys %{$hBalanceUpdate};
    my $ProductCount = scalar @aProductUuid;
    # Make batches according to API specification
    my $BatchSize = $BatchSizeByMethod->{'bulkUpdateProductBalance'};
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=$BatchSize) {
        my $EndIndex = ($StartIndex+$BatchSize >= $ProductCount) ? $ProductCount-1 : $StartIndex+$BatchSize-1;
        my @aBatchProductUuid = @aProductUuid[$StartIndex..$EndIndex];
        # Use slice of hash keys to obtain a "hash slice"
        my %hBatchBalanceUpdate = map { $_ => $hBalanceUpdate->{$_} } @aBatchProductUuid;
        my $UpdateResultState;
        eval {
            my $hContent = $self->_getBulkUpdateInventoryContent(\%hBatchBalanceUpdate);

            # Do not send "void" updates (occurs when shop stock levels match Zettle's)
            if (scalar @{$hContent->{'movements'}} > 0) {
                $self->sendPostRequest($BulkUpdateBalanceUrl, $hHeadersParams, $hContent);
            }
        };
        if (ExistsError()) {
            # When an error occurs we add the associated UUIDs to a list
            # in order to inform the caller of 'failed' and 'updated' items
            $UpdateResultState = 'failed';
        } else {
            $UpdateResultState = 'updated';
        }

        # Compile the list of IDs for successful and failed updates summary
        for my $ProductUuid ( @aBatchProductUuid ) {
            my @VariantUuids = keys %{$hBatchBalanceUpdate{$ProductUuid}};
            push (@{$hUpdateResult->{$UpdateResultState}}, @VariantUuids);
        }
    }

    return $hUpdateResult;
}

#========================================================================================
# function     getTrackedProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductUuid = $self->getTrackedProducts();
#----------------------------------------------------------------------------------------
# description  Retrieve the full list of tracked products.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-tracking/fetch-tracking-products-status ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getTrackedProducts
#               See: https://inventory.izettle.com/swagger#/default/getTrackedProducts
#----------------------------------------------------------------------------------------
# return       $aProductUuid | list of superproduct Zettle identifiers (ZettleUuid)  | ref.array.string
#========================================================================================
sub getTrackedProducts {
    my $self = shift;

    my $Url = GetTrackedProductsApiUrl();

    my $HttpResponse = $self->sendGetRequest($Url);
    my $aProductUuid = $self->decodeResponseContent($HttpResponse);

    return $aProductUuid;
}

#========================================================================================
# function     getTrackingStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aProductInventoryDTO = $self->getTrackingStatus($aProductUuid);
#----------------------------------------------------------------------------------------
# input        $aProductUuid | list of superproduct Zettle identifiers (ZettleUuid)  | ref.array.string
#----------------------------------------------------------------------------------------
# description  Retrieve the tracking status for the given products.
#               See: https://developer.zettle.com/docs/api/inventory/user-guides/manage-inventory-tracking/fetch-tracking-products-status#fetch-product-tracking-status ## no critic qw(ProhibitLongLines)
#               See: https://developer.zettle.com/docs/api/inventory/reference#getTrackingStatusForProducts
#               See: https://inventory.izettle.com/swagger#/default/getTrackingStatusForProducts
#----------------------------------------------------------------------------------------
# return       $aProductInventoryDTO | a list of ProductTrackingDTO object.
#                                       See DE_EPAGES::Zettle::API::WebService::DTO::ProductTrackingDTO | ref.array.object
#========================================================================================
sub getTrackingStatus {
    my $self = shift;

    my ($aProductUuid) = @_;

    my $Url = GetTrackingStatusApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };

    my $HttpResponse = $self->sendPostRequest($Url, $hHeadersParams, $aProductUuid);
    my $ahProductTracking = $self->decodeResponseContent($HttpResponse);

    my $aProductTrackingDTO = [];
    for my $hProductTracking (@{$ahProductTracking}) {
        my $ProductTrackingDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductTrackingDTO->fromHash($hProductTracking);
        push (@{$aProductTrackingDTO}, $ProductTrackingDTO);
    }

    return $aProductTrackingDTO;
}


#========================================================================================


#========================================================================================
# function     _getBulkUpdateInventoryContent
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hInventoryMovement = $self->_getBulkUpdateInventoryContent($hBalanceUpdate);
#----------------------------------------------------------------------------------------
# description  Generate the contents required for an InventoryMovement request.
#----------------------------------------------------------------------------------------
# input        $hBalanceUpdate | Map of balance updates indexed by ProductUUID
#                                 where values are a map of variation stock changes indexed by VariantUUID | ref.hash.hash
#----------------------------------------------------------------------------------------
# return       $hInventoryMovement | data used for an InventoryMovement request | ref.hash
#----------------------------------------------------------------------------------------
# notes        See https://inventory.izettle.com/swagger#/default/moveInventory
#========================================================================================
sub _getBulkUpdateInventoryContent {
    my $self = shift;
    my ($hBalanceUpdate) = @_;

    my $Environment = $self->getEnvironment();

    my $LocationSoldUuid = $Environment->locationSoldUuid();
    my $LocationStoreUuid = $Environment->locationStoreUuid();
    my $LocationSupplierUuid = $Environment->locationSupplierUuid();

    my $ahMovements = [];
    my ($FromLocationUuid, $ToLocationUuid);

    foreach my $ProductUuid (keys %{$hBalanceUpdate}) {
        my $hVariantUpdates = $hBalanceUpdate->{$ProductUuid};
        foreach my $VariantUuid (keys %{$hVariantUpdates}) {
            my $VariantBalanceChange = $hVariantUpdates->{$VariantUuid};
            next unless defined $VariantBalanceChange;

            # Zettle would raise an error if stock change is not greater than zero
            next if $VariantBalanceChange == 0;

            if ($VariantBalanceChange < 0) {
                $FromLocationUuid = $LocationStoreUuid;
                $ToLocationUuid = $LocationSoldUuid;
            } else {
                # Returned products may increase stock but we cannot tell when updates
                # are due to returns so we always work with SUPPLIER and STORE locations
                $FromLocationUuid = $LocationSupplierUuid;
                $ToLocationUuid = $LocationStoreUuid;
            }
            push(@{$ahMovements},
                    {
                        'productUuid' => $ProductUuid,
                        'variantUuid' => $VariantUuid,
                        'change' => abs($VariantBalanceChange),
                        'from' => $FromLocationUuid,
                        'to' => $ToLocationUuid
                    }
            );
        }
    }

    my $MovementIdentifier = $Environment->getMerchantUUID();

    # 'identifier' is sent back within the payload of some incoming callbacks
    # like InventoryBalanceChanged event (field externalUuid)
    # External identifier is used to avoid processing our own stock updates sent back
    # from Zettle via incoming stock callbacks
    my $hContent = {
        'identifier' => $MovementIdentifier,
        'movements' => $ahMovements
    };

    return $hContent;
}

1;
