use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( ExistsClassByAlias LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Zettle::API::Constants qw ( ZETTLE_ENVIRONMENT_CLASS ZETTLE_OBJECT_ALIAS );
use DE_EPAGES::Zettle::API::ZettleService;

# Store to be used if 'storename' argument is not present
use constant DEFAULT_STORE => 'Store';

# Check refresh token issues for all ZettleEnvironments.
sub Main {

    local $| = 1;

    my ($Help, $NoOutput, $Password, $StoreName, $ShopAlias);

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
    );

    $StoreName //= DEFAULT_STORE;

    usage() if $Help;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            # make sure that Zettle cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::Zettle')) {
                print "Store $StoreName: Cartridge Zettle is not installed.\n" unless $NoOutput;
                return;
            }

            # Run script on a specific shop if 'shop' argument is present (and different from 'all')
            my $hNotificationInfo;
            if (defined $ShopAlias && $ShopAlias ne 'all') {
                my $Shop;
                eval {
                    $Shop = LoadObjectByPath('/Shops')->child( $ShopAlias );
                };
                if (ExistsError()) {
                    printf STDERR ("[%s] WARN: cannot load shop '%s' (error: %s).\n", $StoreName, $ShopAlias, GetError()->message());
                    return;
                }

                # Skip if Zettle is not connected
                return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

                my $ZettleEnvironment = $Shop->child(ZETTLE_OBJECT_ALIAS);

                $hNotificationInfo = $ZettleEnvironment->verifyRefreshTokenAndNotifyMerchant();
                if ($hNotificationInfo->{'IsNotified'}) {
                    printf STDOUT ("[%s] Refresh token issue notification sent to merchant for shop: %s (email: %s)\n",
                                    $StoreName, $Shop->alias, $hNotificationInfo->{'NotificationEmail'}) unless $NoOutput;
                }
            } else {
                unless(ExistsClassByAlias(ZETTLE_ENVIRONMENT_CLASS)){
                    printf STDERR ("[%s] WARN: Class '%s' does not exist.\n", ZETTLE_ENVIRONMENT_CLASS);
                    return;
                }

                # Retrieve all store Zettle instances
                my $aZettleEnvironments = LoadClassByAlias(ZETTLE_ENVIRONMENT_CLASS)->objects;

                my $Feature = LoadFeatureByAlias('Zettle');
                foreach my $ZettleEnvironment ( @{ $aZettleEnvironments } ) {
                    my $EnvPath = $ZettleEnvironment->pathString // 'path unknown';
                    eval {
                        my $Shop = $ZettleEnvironment->getSite();

                        # skip export if Zettle feature is disabled for this shop
                        my $FeatureValue = $Shop->featureMaxValue($Feature);
                        next if ( not $FeatureValue );

                        # skip if shop is ClosedByProvider
                        next if ($Shop->get('ClosedByProvider'));

                        $hNotificationInfo = $ZettleEnvironment->verifyRefreshTokenAndNotifyMerchant();
                        if ($hNotificationInfo->{'IsNotified'}) {
                            printf STDOUT ("[%s] Refresh token issue notification sent to merchant for shop: %s (email: %s)\n",
                                            $StoreName, $Shop->alias, $hNotificationInfo->{'NotificationEmail'}) unless $NoOutput;
                        }
                    };
                    if (ExistsError()) {
                        my $Error = GetError();
                        printf STDERR ( "[%s] WARN: %s: %s\n", $StoreName, $EnvPath, $Error->message) unless $NoOutput;
                    }
                }
            }
        },
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $timemsg = sprintf( 'need %.3f seconds', $seconds );
    print $timemsg. "\n" unless $NoOutput;
    GetLog()->debug("Zettle refresh token checks on Store '$StoreName': $timemsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Description:
    Check Zettle environments for refresh token issues and, when necessary, send off email notice to inform merchants.

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Default: Store)
    -shop         shop alias             (optional)   (Default: all)
flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
